/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvCommon.txx,v $
 Language:  C++
 Date:      $Date: 2008/04/10 06:59:36 $
 Version:   $Revision: 1.2 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef VVCOMMON_TXX
#define VVCOMMON_TXX

//====================================================================
// Convert float, double ... to string
template<class T>
std::string toString(const T & t)
{
  std::ostringstream myStream;
  myStream << t << std::flush;
  return(myStream.str());
} ////
//====================================================================

//====================================================================
// Convert float, double ... to string
template<class T>
std::string toString(const T & t, int precision, int width)
{
  std::ostringstream myStream;
  if (t<1) myStream.precision(precision);
  else if (t<10) myStream.precision(precision+1);
  else if (t<100) myStream.precision(precision+2);
  else if (t<1000) myStream.precision(precision+3);
  myStream.width(width);
  myStream.flags(std::ios::right+std::ios::fixed);
  myStream << t << std::flush;
  return(myStream.str());
} ////
//====================================================================

//====================================================================
// Convert float*, double* ... to string
template<class T>
std::string toStringVector(const T * t, const int n)
{
  std::ostringstream myStream;
  for(int i=0; i<n-1; i++)
	myStream << vv::toString<T>(t[i]) << " ";
  myStream << vv::toString<T>(t[n-1]) << std::flush;
  return(myStream.str());
} ////
//====================================================================

//====================================================================
// Convert float*, double* ... to string
template<class T>
std::string toStringVector(const T & t, const int n)
{
  std::ostringstream myStream;
  for(int i=0; i<n-1; i++)
	myStream << t[i] << " ";
  myStream << t[n-1] << std::flush;
  return(myStream.str());
} ////
//====================================================================

//====================================================================
template<class TPixel>
std::string GetTypeAsString() {
  const std::type_info & PixType = typeid(TPixel);
  std::string pixelName;
  if (PixType == typeid(char)) pixelName = "char";
  else if (PixType == typeid(unsigned char)) pixelName = "uchar";
  else if (PixType == typeid(short)) pixelName = "short";
  else if (PixType == typeid(unsigned short)) pixelName = "ushort";
  else if (PixType == typeid(int)) pixelName = "int";
  else if (PixType == typeid(unsigned int)) pixelName = "uint";
  else if (PixType == typeid(float)) pixelName = "float";
  else if (PixType == typeid(double)) pixelName = "double";
  else pixelName = PixType.name();
  return pixelName;
}
//====================================================================

//====================================================================
// Compute the number of different intensities in an image
template<class ImageType>
int ComputeHowManyDifferentIntensity(const typename ImageType::Pointer & image, 
				     std::vector<typename ImageType::PixelType> & l)
{
  //std::set<typename ImageType::PixelType> listOfIntensities;
  std::map<typename ImageType::PixelType, bool> listOfIntensities;
  //  listOfIntensities.resize(0);
  typedef itk::ImageRegionConstIterator<ImageType> ConstIteratorType;
  ConstIteratorType pi(image, image->GetLargestPossibleRegion());
  pi.Begin();
  while (!pi.IsAtEnd()) {
	if (!listOfIntensities[pi.Get()]) listOfIntensities[pi.Get()] = true;
	// if (std::find(listOfIntensities.begin(), 
// 				  listOfIntensities.end(), 
// 				  pi.Get()) == listOfIntensities.end()) {
// 	  listOfIntensities.insert(pi.Get());
// 	}
	++pi;
  }
  
  //typename std::set<typename ImageType::PixelType>::const_iterator ppi = listOfIntensities.begin();
  typename std::map<typename ImageType::PixelType, bool>::const_iterator ppi = listOfIntensities.begin();
  while (ppi != listOfIntensities.end()) {
	l.push_back(ppi->first);
	++ppi;
  }

  return listOfIntensities.size();
}
//====================================================================

#endif /* end #define VVCOMMON_TXX */

