/*=========================================================================

Program:   vv
Module:    $RCSfile: vvConvertITKImageToVTKDataFilter.txx,v $
Language:  C++
Date:      $Date: 2008/07/04 08:16:43 $
Version:   $Revision: 1.4 $
Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef vvConvertITKImageToVTKDataFilter_TXX
#define vvConvertITKImageToVTKDataFilter_TXX

//====================================================================
template<unsigned int VImageDimension>
void vvConvertITKImageToVTKDataFilter::UpdateWithDim(std::string InputPixelType) 
{
  if(InputPixelType == "short")
    {
      UpdateWithDimAndInputPixelType<short,VImageDimension>();
    }
  else if (InputPixelType == "unsigned_short")
    {
      UpdateWithDimAndInputPixelType<unsigned short,VImageDimension>();
    }
  else if (InputPixelType == "char")
    {
      UpdateWithDimAndInputPixelType<char,VImageDimension>();
    }
  else if (InputPixelType == "unsigned_char")
    {
      UpdateWithDimAndInputPixelType<unsigned char,VImageDimension>();
    }
  else if (InputPixelType == "int")
    {
      UpdateWithDimAndInputPixelType<int,VImageDimension>();
    }
  else if (InputPixelType == "double")
    {
      UpdateWithDimAndInputPixelType<double,VImageDimension>();
    }
  else if (InputPixelType == "float")
    {
      UpdateWithDimAndInputPixelType<float,VImageDimension>();
    }
  else
    {
      std::cerr << "Error, input pixel type : " << InputPixelType << " unknown !" << std::endl;
    }
}
//====================================================================

//====================================================================
template<class InputPixelType, unsigned int VImageDimension>
void vvConvertITKImageToVTKDataFilter::UpdateWithDimAndInputPixelType() 
{
  if(VImageDimension == 4 || mType == 23 || mType == 24)
    {
      UpdateForDifferentDims<InputPixelType, VImageDimension>();
    }
  else
    {
      UpdateForSameDims<InputPixelType, VImageDimension>();
    }
}
//====================================================================

//====================================================================
template<class InputPixelType, unsigned int VImageDimension>
void vvConvertITKImageToVTKDataFilter::UpdateForDifferentDims() 
{
  mImage->Init();
  //Create the reader
  typedef itk::Image< InputPixelType, VImageDimension > InputImageType;
  typedef itk::ImageSeriesReader<InputImageType> ReaderType;
  typename ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileNames(mInputFilename);
  if (mUseAnObserver) {
    reader->AddObserver(itk::ProgressEvent(), mObserver);
  }
  try {
    reader->Update(); 
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while reading " << mInputFilename[0].c_str()
	      << " " << err << std::endl;
    std::stringstream error;
    error << err;
    mLastError = error.str();
    return;
  }

  typedef itk::Image< InputPixelType,  VImageDimension - 1 >    ConnectorImageType;
  typedef itk::ImageToVTKImageFilter <ConnectorImageType> ConnectorType;
  typedef itk::ExtractImageFilter<InputImageType,ConnectorImageType> FilterType;

  typename FilterType::Pointer filter = FilterType::New();
  typename ConnectorType::Pointer connector = ConnectorType::New();

  //extract the 3D slices and put them in a std::vector<vtkImageData*>
  typename InputImageType::RegionType inputRegion = reader->GetOutput()->GetLargestPossibleRegion();
  typename InputImageType::SizeType inputSize = inputRegion.GetSize();

  typename InputImageType::SizeType extractedRegionSize = inputSize;
  typename InputImageType::RegionType extractedRegion;
  extractedRegionSize[VImageDimension - 1] = 0;
  extractedRegion.SetSize(extractedRegionSize);

  filter->SetInput(reader->GetOutput());
  connector->SetInput(filter->GetOutput());

  typename InputImageType::IndexType start = inputRegion.GetIndex();

  for(unsigned int i = 0; i < inputSize[VImageDimension - 1]; i++) {
    start[VImageDimension - 1] = i;
    extractedRegion.SetIndex(start);
    filter->SetExtractionRegion(extractedRegion);
    try{
      filter->Update();
    }
    catch( itk::ExceptionObject & err ) {
      std::cerr << "Error while filtering " << mInputFilename[0].c_str()
		<< " " << err << std::endl;
      std::stringstream error;
      error << err;
      mLastError = error.str();
      return;
    }
    try{
      connector->Update();
    }
    catch( itk::ExceptionObject & err ) {
      std::cerr << "Error while converting " << mInputFilename[0].c_str()
		<< " " << err << std::endl;
      std::stringstream error;
      error << err;
      mLastError = error.str();
      return;
    }
    vtkImageData *image = vtkImageData::New();
    image->DeepCopy(connector->GetOutput());
    mImage->AddImage(image);
  }

}
//====================================================================

//====================================================================
template<class InputPixelType, unsigned int VImageDimension>
void vvConvertITKImageToVTKDataFilter::UpdateForSameDims() 
{
  mImage->Init();

  //Create the reader
  typedef itk::Image< InputPixelType, VImageDimension > InputImageType;
  typedef itk::ImageSeriesReader<InputImageType> ReaderType;
  typename ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileNames(mInputFilename);
  if (mUseAnObserver) {
    reader->AddObserver(itk::ProgressEvent(), mObserver);
  }
  try {
    reader->Update(); 
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while reading " << mInputFilename[0].c_str()
	      << " " << err << std::endl;
    std::stringstream error;
    error << err;
    mLastError = error.str();
    return;
  }

  typedef itk::Image< InputPixelType,  VImageDimension >    ConnectorImageType;
  typedef itk::ImageToVTKImageFilter <ConnectorImageType> ConnectorType;
  typename ConnectorType::Pointer connector = ConnectorType::New();
  connector->SetInput(reader->GetOutput());
  
  try{
    connector->Update();
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while converting " << mInputFilename[0].c_str()
	      << " " << err << std::endl;
    std::stringstream error;
    error << err;
    mLastError = error.str();
    return;
  }
  vtkImageData *image = vtkImageData::New();
  image->DeepCopy(connector->GetOutput());
  mImage->AddImage(image);
}
//====================================================================

#endif /* end #define vvConvertITKImageToVTKDataFilter_TXX */

