/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvConvertVTKImageToITKDataFilter.h,v $
 Language:  C++
 Date:      $Date: 2008/05/28 14:17:31 $
 Version:   $Revision: 1.1 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef vvConvertVTKImageToITKDataFilter_H
#define vvConvertVTKImageToITKDataFilter_H

#include <string.h>

// ITK includes
#include "itkImage.h"
#include "itkImageFileWriter.h"
#include "itkVTKImageToImageFilter.h"
#include "itkJoinSeriesImageFilter.h"
#include "itkCommand.h"
#include "vtkImageData.h"
#include "vvImage.h"

class vvConvertVTKImageToITKDataFilter {
    
  public: 
    // constructor
    vvConvertVTKImageToITKDataFilter();
    ~vvConvertVTKImageToITKDataFilter();

    void SetOutputFileName(std::string filename);
    void SetObserver(itk::Command::Pointer o) { mUseAnObserver = true; mObserver = o; }
    void DisableObserver() { mUseAnObserver = false; }

    void SetInput(vvImage* image){mImage = image;}

    std::string GetLastError() {return mLastError;}

    //====================================================================
    // Main function
    void Update(int dim, std::string OutputPixelType);

  protected:
    //====================================================================
    std::string mOutputFilename;
    itk::Command::Pointer mObserver;

    std::string mLastError;
    bool mUseAnObserver;

    //====================================================================
    template<unsigned int VImageDimension>
    void UpdateWithDim(std::string OutputPixelType);

    //====================================================================
    template<class OutputPixelType, unsigned int VImageDimension>
    void UpdateWithDimAndOutputPixelType();

    //====================================================================
    template<class OutputPixelType, unsigned int VImageDimension>
    void UpdateForDifferentDims();

    //====================================================================
    template<class OutputPixelType, unsigned int VImageDimension>
    void UpdateForSameDims();

 private:
   vvImage* mImage;

  }; // end class vvConvertVTKImageToITKDataFilter

#include "vvConvertVTKImageToITKDataFilter.txx"

#endif /* end #define vvConvertVTKImageToITKDataFilter_H */

