/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvIOCommon.txx,v $
 Language:  C++
 Date:      $Date: 2008/04/10 06:59:37 $
 Version:   $Revision: 1.2 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef VVIOCOMMON_TXX
#define VVIOCOMMON_TXX

//====================================================================
// To short the code for reading an image
template<typename ImageType>
typename ImageType::Pointer readImage(const std::string & filename, const bool verbose) {
  typedef itk::ImageFileReader<ImageType> ReaderType;
  typename ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(filename.c_str());
  if (verbose) {
	std::cout << "Reading " << filename << " ..." << std::endl;
  }
  try {
	reader->Update(); 
  }
  catch( itk::ExceptionObject & err ) {
	std::cerr << "Error while reading " << filename 
			  << " " << err << std::endl;
	exit(0);
  }
  return reader->GetOutput();
}
//====================================================================

//====================================================================
// To short the code for writing an image
template<typename ImageType>
void writeImage(const typename ImageType::Pointer image, 
				const std::string & filename, 
				const bool verbose) {
  typedef itk::ImageFileWriter<ImageType> WriterType;
  typename WriterType::Pointer writer = WriterType::New();
  writer->SetFileName(filename.c_str());
  writer->SetInput(image);
  if (verbose) {
	std::cout << "Writing " << filename << " ..." << std::endl;
  }
  try { 
	writer->Update(); 
  }
  catch( itk::ExceptionObject & err ) {
	std::cerr << "Error while writing " << filename 
			  << " " << err << std::endl;
	exit(0);
  }
}
//====================================================================

#endif /* end #define VVIOCOMMON_TXX */

