/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvLandmarks.cxx,v $
 Language:  C++
 Date:      $Date: 2008/07/15 12:34:19 $
 Version:   $Revision: 1.7 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#include "vvLandmarks.h"

#include <ios>
#include <fstream>
#include <sstream>

#include "vtkPolyData.h"
#include "vtkPoints.h"
#include "vtkFloatArray.h"
#include "vtkPointData.h"

vvLandmarks::vvLandmarks(int size)
{
  mLandmarks.resize(0);
  mFilename = "";

  for(int i = 0; i < size; i++)
    {
    vtkPoints *points = vtkPoints::New();
    mPoints.push_back(points);
    }
  mPolyData = vtkPolyData::New();
  mIds = vtkFloatArray::New();
}

vvLandmarks::~vvLandmarks()
{
  for(unsigned int i = 0; i < mPoints.size(); i++) {
    mPoints[i]->Delete();
    }
  /*for(unsigned int i = 0; i < mText.size(); i++) {
    mText[i]->Delete();
    }*/
  if(mIds)
    mIds->Delete();
  if(mPolyData)
    mPolyData->Delete();
}

void vvLandmarks::AddLandmark(float x,float y,float z,float t)
{
  //std::cout << "vvLandmarks::AddLandmark(" << x << "," << y << "," << z << "," << t << ")" << std::endl;
  vvLandmark point;
  point.coordinates[0] = x;
  point.coordinates[1] = y;
  point.coordinates[2] = z;
  point.coordinates[3] = t;
  mLandmarks.push_back(point);
  mPoints[int(t)]->InsertNextPoint(x,y,z);

  /*std::stringstream numberVal;
  numberVal << (mLandmarks.size()-1);
  vvLandmarksGlyph *number = vvLandmarksGlyph::New();
  number->SetText(numberVal.str().c_str());
  number->BackingOff();
  mText.push_back(number);*/
  mIds->InsertNextTuple1(0.55);
  //mIds->InsertTuple1(mLandmarks.size(),mLandmarks.size());
  SetTime(int(t));
}

void vvLandmarks::RemoveLastLandmark()
{
  //std::cout << "vvLandmarks::RemoveLastLandmark()" << std::endl;
  mPoints[mLandmarks.back().coordinates[3]]->SetNumberOfPoints(
		mPoints[mLandmarks.back().coordinates[3]]->GetNumberOfPoints()-1);
  mPolyData->Modified();
  //mText.pop_back();
  mLandmarks.pop_back();
  mIds->Resize(mIds->GetSize() -1);
  //mIds->RemoveLastTuple();
}

void vvLandmarks::ChangeComments(int index, std::string comments)
{
  //std::cout << "vvLandmarks::ChangeComments()" << std::endl;
  mLandmarks[index].comments = comments;
}

float* vvLandmarks::GetCoordinates(int index)
{
  return mLandmarks[index].coordinates;
}

std::string vvLandmarks::GetComments(int index)
{
  return mLandmarks[index].comments;
}

void vvLandmarks::LoadFile(std::string filename)
{
  //std::cout << "LoadFile (" << filename << ")" << std::endl;
  std::ifstream fp(filename.c_str(), std::ios::in|std::ios::binary);
  if(!fp.is_open())
    {
    std::cerr <<"Unable to open file " << filename << std::endl;
    return;
    }
  mFilename = filename;
  mLandmarks.clear();
  char line[255];
  for(unsigned int i = 0; i < mPoints.size();i++)
    mPoints[i]->SetNumberOfPoints(0);
  while(fp.getline(line,255))
    {
    std::string stringline = line;
    if(stringline.size() > 1)
      {
      vvLandmark point;
      int previousSpace = 0;
      int space = stringline.find(" ", previousSpace+1);
      if(space < -1 || space > stringline.size())
        {
        ErrorMsg(mLandmarks.size(),"index");
        continue;
        }
      int index = atoi(stringline.substr(previousSpace,space - previousSpace).c_str());
      previousSpace = space;
      space = stringline.find(" ", previousSpace+1);
      if(space < -1 || space > stringline.size())
        {
        ErrorMsg(mLandmarks.size(),"x position");
        continue;
        }
      point.coordinates[0] = atof(stringline.substr(previousSpace,space - previousSpace).c_str());
      previousSpace = space;
      space = stringline.find(" ", previousSpace+1);
      if(space < -1 || space > stringline.size())
        {
        ErrorMsg(mLandmarks.size(),"y position");
        continue;
        }
      point.coordinates[1] = atof(stringline.substr(previousSpace,space - previousSpace).c_str());
      previousSpace = space;
      space = stringline.find(" ", previousSpace+1);
      if(space < -1 || space > stringline.size())
        {
        ErrorMsg(mLandmarks.size(),"z position");
        continue;
        }
      point.coordinates[2] = atof(stringline.substr(previousSpace,space - previousSpace).c_str());
      previousSpace = space;
      space = stringline.find(" ", previousSpace+1);
      if(space < -1 || space > stringline.size())
        {
        ErrorMsg(mLandmarks.size(),"t position");
        continue;
        }
      point.coordinates[3] = atof(stringline.substr(previousSpace,space - previousSpace).c_str());
      previousSpace = space;
      //this is the maximum size of comments
      space = (stringline.find("\n", previousSpace+1) < 254 ? stringline.find("\n", previousSpace+1) : 254);

      point.comments = stringline.substr(previousSpace,space - (previousSpace)).c_str();
      mLandmarks.push_back(point);
      mIds->InsertNextTuple1(0.55);
      mPoints[int(point.coordinates[3])]->InsertNextPoint(
		point.coordinates[0],point.coordinates[1],point.coordinates[2]);
      }
    }
  SetTime(0);
}

bool vvLandmarks::ErrorMsg(int num,const char * text)
{
  std::cout << "error when loading point " << num << " at " << text << std::endl;
  return false;
}

void vvLandmarks::SaveFile(std::string filename)
{
  //std::cout << "vvLandmarks::SaveFile(" << filename << ")" << std::endl;
  std::string fileContent = "";
  for(unsigned int i = 0; i < mLandmarks.size(); i++) {
    std::stringstream index;
    index << i;
    std::stringstream x;
    x << mLandmarks[i].coordinates[0];
    std::stringstream y;
    y << mLandmarks[i].coordinates[1];
    std::stringstream z;
    z << mLandmarks[i].coordinates[2];
    std::stringstream t;
    t << mLandmarks[i].coordinates[3];
    fileContent += index.str() + " " + x.str() + " " + y.str() + " " + z.str() + " " + t.str() + " ";
    if(mLandmarks[i].comments.size() == 0)
      fileContent += " ";
    else
      fileContent += mLandmarks[i].comments;
    fileContent += "\n";
    }
  std::ofstream fp(filename.c_str(), std::ios::trunc);
  if ( !fp )
    {
    std::cerr << "Unable to open file" << std::endl;
    return;
    }
  fp << fileContent.c_str()<< std::endl;
  fp.close();
}

void vvLandmarks::SetTime(int time)
{
  //std::cout << "SetTime(" << time << ")" << std::endl;
  if(time >= 0 && time <= ((int)mPoints.size() -1))
    {
    mPolyData->SetPoints(mPoints[time]);
    mPolyData->GetPointData()->SetScalars(mIds);
    mPolyData->Modified();
    mPolyData->Update();
    }
}

