/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvSlicer.cxx,v $
 Language:  C++
 Date:      $Date: 2008/07/15 08:58:27 $
 Version:   $Revision: 1.36 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#include "vvSlicer.h"

#include "vvImage.h"
#include "vvVF.h"
#include "vvSlicersManagerCommand.h"

#include "vtkTextProperty.h"
#include "vtkTextActor.h"
#include "vtkTextSource.h"
#include "vtkActor2D.h"
#include "vvCursor2D.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkProperty2D.h"
#include "vtkCornerAnnotation.h"

#include "vtkWindowLevelLookupTable.h"
#include "vtkImageMapToWindowLevelColors.h"
#include "vtkImageMapToColors.h"
#include "vtkImageData.h"
#include "vtkImageActor.h"

#include "vtkToolkits.h"
#include "vtkObjectFactory.h"

#include "vtkStructuredGrid.h"
#include "vtkExtractGrid.h"
#include "vtkPointData.h"
#include "vtkDataArray.h"
#include "vtkFloatArray.h"
#include "vtkClipPolyData.h"

#include "vtkGlyph3D.h"
#include "vtkGlyphSource2D.h"
#include "vvGlyphSource.h"
#include "vtkMath.h"
#include "vvGlyph2D.h"
#include "vtkCursor3D.h"

#include "vtkProperty.h"
#include "vtkLight.h"
#include "vtkLightCollection.h"

#include "vvHelp.h"

#include <vtkRenderer.h>
#include <vtkRendererCollection.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkCamera.h>
#include <vtkCallbackCommand.h>
#include <vtkCommand.h>


vtkCxxRevisionMacro(vvSlicer, "$Revision: 1.36 $");
vtkStandardNewMacro(vvSlicer);
vvSlicer::vvSlicer()
{
  mImage = NULL;
  mCurrentTSlice = 0;

  mCurrent[0] = -VTK_DOUBLE_MAX;
  mCurrent[1] = -VTK_DOUBLE_MAX;
  mCurrent[2] = -VTK_DOUBLE_MAX;

  mCursor[0] = -VTK_DOUBLE_MAX;
  mCursor[1] = -VTK_DOUBLE_MAX;
  mCursor[2] = -VTK_DOUBLE_MAX;
  mCursor[3] = -VTK_DOUBLE_MAX;

  mSubSampling = 5;
  mScale = 1;
  mVFLog = 0;

  std::string text = "F1 = sagital; F2 = coronal; F3 = axial\n";
             text += "F5 = horizontal flip; F6 = vertical flip\n\n";
             text += "0,1,2,3,4,5 : preset windowing\n";
             text += "6,7,8,9 : preset colormap\n";
             text += "r : reset view\n";
             text += "l : reload image\n";
             text += "f : fly to mouse position\n";
             text += "g : go to cross hair position\n\n";
             text += "Up,down : change slice\n";
             text += "Left,right : change tenporal slice\n\n";
             text += "Scrollbar (or w/x) : zoom in/out\n";
             text += "left button : synchronize all views\n";
             text += "middle button : grab image\n";
             text += "right button : change windowing\n";

  textActor = vvHelp::New();
  textActor->SetInput(text.c_str());
  textActor->GetTextProperty()->SetColor(255,10,212);
  textActor->GetTextProperty()->SetJustificationToCentered();
  textActor->GetTextProperty()->SetVerticalJustificationToCentered();
  textActor->ScaledTextOn();
  textActor->SetVisibility(0);
  textActor->UseBorderAlignOn();

  crossCursor = vvCursor2D::New();
  crossCursor->AllOff();
  crossCursor->AxesOn();
  crossCursor->SetTranslationMode(1);
  crossCursor->SetRadius(2);

  pdm = vtkPolyDataMapper2D::New();
  pdm->SetInput(crossCursor->GetOutput());

  pdmA = vtkActor2D::New();
  pdmA->SetMapper(pdm);
  pdmA->GetProperty()->SetColor(255,10,212);
  pdmA->SetVisibility(0);
  pdmA->SetPickable(0);

  ca = vtkCornerAnnotation::New();
  ca->GetTextProperty()->SetColor(255,10,212);
  ca->SetVisibility(1);
  mFileName = "";

  mVF = NULL;
  mArrow = NULL;
  mSampleFilter = NULL;
  mGlyphFilter = NULL;
  mVFMapper = NULL;
  mVFActor = NULL;

  mOverlay = NULL;
  mOverlayMapper = NULL;
  mOverlayActor = NULL;

  mFusion = NULL;
  mFusionMapper = NULL;
  mFusionActor = NULL;

  mLandmarks = NULL;
  mLandClipper = NULL;
  mLandGlyph = NULL;
  mLandMapper = NULL;
  mLandActor = NULL;
  mCross = NULL;
  mPlane = NULL;
}

vvSlicer::~vvSlicer()
{
  if(ca)
   ca->Delete();
  if(textActor)
    textActor->Delete();
  if(crossCursor)
    crossCursor->Delete();
  if(pdm)
    pdm->Delete();
  if(pdmA)
    pdmA->Delete();
  if(mGlyphFilter)
    mGlyphFilter->Delete();
  if(mSampleFilter)
    mSampleFilter->Delete();
  if(mArrow)
    mArrow->Delete();
  if(mVFMapper)
    mVFMapper->Delete();
  if(mVFActor)
    mVFActor->Delete();
  if(mOverlayMapper)
    mOverlayMapper->Delete();
  if(mOverlayActor)
    mOverlayActor->Delete();
  if(mFusionMapper)
    mFusionMapper->Delete();
  if(mFusionActor)
    mFusionActor->Delete();
  /*if(mLandmarks)
    delete mLandmarks;*/
  if(mLandClipper)
    mLandClipper->Delete();
  if(mLandGlyph)
    mLandGlyph->Delete();
  if(mLandMapper)
    mLandMapper->Delete();
  if(mLandActor)
    mLandActor->Delete();
  if(mCross)
    mCross->Delete();
  if(mPlane)
    mPlane->Delete();
}

void vvSlicer::SetCurrentPosition(double x, double y, double z, int t)
  {
  //std::cout <<"vvSlicer::SetCurrentPosition(" << x << "," << y << "," << z << "," << t << ")" << std::endl;
  mCurrent[0] = x;
  mCurrent[1] = y;
  mCurrent[2] = z;
  mCurrentTSlice = t;
  }

void vvSlicer::SetImage(vvImage* image)
{
  //std::cout <<"vvSlicer::SetImage()" << std::endl;
  if(image->GetImage().size())
    {
    mImage = image;
    this->Superclass::SetInput(image->GetImage()[0]);
    this->UpdateDisplayExtent();
    mCurrentTSlice = 0;
    ca->SetText(0,mFileName.c_str());
    }
}

void vvSlicer::SetOverlay(vvImage* overlay)
{
  //std::cout <<"vvSlicer::SetOverlay()" << std::endl;
  if(overlay->GetImage().size())
    {
    mOverlay = overlay;

    if(!mOverlayMapper)
      mOverlayMapper = vtkImageMapToWindowLevelColors::New();
    mOverlayMapper->SetInput(overlay->GetImage()[0]);

    if(!mOverlayActor)
      mOverlayActor = vtkImageActor::New();
    mOverlayActor->SetInput(mOverlayMapper->GetOutput());
    mOverlayActor->SetPickable(0);
    mOverlayActor->SetVisibility(0);
    mOverlayActor->SetOpacity(0.5);
    this->UpdateDisplayExtent();

    //stupid but necessary : the Overlay need to be render before fusion
    if(mFusionActor)
      {
      this->GetRenderer()->RemoveActor(mFusionActor);
      this->GetRenderer()->AddActor(mOverlayActor);
      this->GetRenderer()->AddActor(mFusionActor);
      }
    else
      this->GetRenderer()->AddActor(mOverlayActor);

    //Synchronize slice
    SetTSlice(mCurrentTSlice);
    }
}

void vvSlicer::SetFusion(vvImage* fusion)
{
  //std::cout <<"vvSlicer::SetFusion()" << std::endl;
  if(fusion->GetImage().size())
    {
    mFusion = fusion;

    if(!mFusionMapper)
      mFusionMapper = vtkImageMapToWindowLevelColors::New();
    mFusionMapper->SetInput(fusion->GetImage()[0]);

    if(!mFusionActor)
      mFusionActor = vtkImageActor::New();
    mFusionActor->SetInput(mFusionMapper->GetOutput());
    mFusionActor->SetPickable(0);
    mFusionActor->SetVisibility(0);
    mFusionActor->SetOpacity(0.7);
    this->UpdateDisplayExtent();
    this->GetRenderer()->AddActor(mFusionActor);

    //Synchronize slice
    SetTSlice(mCurrentTSlice);
    }
}

void vvSlicer::SetVF(vvVF* vf)
{
  //std::cout <<"vvSlicer::SetVF()" << std::endl;
  if(vf->GetImage().size())
    {
    mVF = vf;

    if(!mSampleFilter)
      mSampleFilter = vtkExtractGrid::New();
      mSampleFilter->SetInput(vf->GetImage()[0]);

    if(!mArrow)
      mArrow = vvGlyphSource::New();
    mArrow->SetGlyphTypeToSpecificArrow();
    mArrow->SetScale(mScale);
    mArrow->FilledOff();

    // Glyph the gradient vector (with arrows)
    if(!mGlyphFilter)
      mGlyphFilter = vvGlyph2D::New();
    mGlyphFilter->SetInput(mSampleFilter->GetOutput());
    mGlyphFilter->SetSource(mArrow->GetOutput());
    mGlyphFilter->ScalingOn();
    mGlyphFilter->SetScaleModeToScaleByVector();
    mGlyphFilter->OrientOn();
    mGlyphFilter->SetVectorModeToUseVector();
    mGlyphFilter->SetColorModeToColorByVector();

    if(!mVFMapper)
      mVFMapper = vtkPolyDataMapper::New();
    mVFMapper->SetInputConnection(mGlyphFilter->GetOutputPort());
    mVFMapper->ImmediateModeRenderingOn();

    if(!mVFActor)
      mVFActor = vtkLODActor::New();
    mVFActor->SetMapper(mVFMapper);
    mVFActor->SetPickable(0);
    mVFActor->SetVisibility(0);
    this->UpdateDisplayExtent();
    this->GetRenderer()->AddActor(mVFActor);

    //Synchronize slice
    SetTSlice(mCurrentTSlice);
    }
}

void vvSlicer::SetLandmarks(vvLandmarks* landmarks)
{
  //std::cout <<"vvSlicer::SetLandmarks()" << std::endl;
  mLandmarks = landmarks;
  if(landmarks)
    {

    if(!mCross)
      mCross = vtkCursor3D::New();
    mCross->SetFocalPoint(0.0,0.0,0.0);
    mCross->SetModelBounds(-10,10,-10,10,-10,10);
    mCross->AllOff();
    mCross->AxesOn();

    if(!mLandGlyph)
      mLandGlyph = vtkGlyph3D::New();
    //mLandGlyph->SetInput(landmarks->GetOutput());
    mLandGlyph->SetSource(mCross->GetOutput());
    mLandGlyph->SetInput(landmarks->GetOutput());
    //mLandGlyph->SetIndexModeToScalar();
    mLandGlyph->SetRange(0,1);
    mLandGlyph->ScalingOff();

    mLandGlyph->SetColorModeToColorByScalar();

    if(!mPlane)
      mPlane = vtkBox::New();
    if(!mLandClipper)
      mLandClipper = vtkClipPolyData::New();
    mLandClipper->InsideOutOn();
    mLandClipper->SetInput(mLandGlyph->GetOutput());
    mLandClipper->SetClipFunction(mPlane);

    if(!mLandMapper)
      mLandMapper = vtkPolyDataMapper::New();
    mLandMapper->SetInputConnection(mLandClipper->GetOutputPort());
    //mLandMapper->ScalarVisibilityOff();

    if(!mLandActor)
       mLandActor = vtkActor::New();
    mLandActor->SetMapper(mLandMapper);
    mLandActor->GetProperty()->SetColor(255,10,212);
    mLandActor->SetPickable(0);
    mLandActor->SetVisibility(1);
    this->UpdateDisplayExtent();
    this->GetRenderer()->AddActor(mLandActor);
    }
}

void vvSlicer::RemoveVF()
{
  //std::cout <<"vvSlicer::RemoveVF()" << std::endl;
  if(mGlyphFilter)
    mGlyphFilter->Delete();
  if(mSampleFilter)
    mSampleFilter->Delete();
  if(mArrow)
    mArrow->Delete();
  if(mVFMapper)
    mVFMapper->Delete();
  if(mVFActor)
    {
    Renderer->RemoveActor(mVFActor);
    mVFActor->Delete();
    }
  mVF = NULL;
  mArrow = NULL;
  mSampleFilter = NULL;
  mGlyphFilter = NULL;
  mVFMapper = NULL;
  mVFActor = NULL;
}

void vvSlicer::RemoveOverlay()
{
  //std::cout <<"vvSlicer::RemoveOverlay()" << std::endl;
  if(mOverlayMapper)
    mOverlayMapper->Delete();
  if(mOverlayActor)
    {
    Renderer->RemoveActor(mOverlayActor);
    mOverlayActor->Delete();
    }
  mOverlay = NULL;
  mOverlayActor = NULL;
  mOverlayMapper = NULL;
}

void vvSlicer::RemoveFusion()
{
  //std::cout <<"vvSlicer::RemoveFusion()" << std::endl;
  if(mFusionMapper)
    mFusionMapper->Delete();
  if(mFusionActor)
    {
    Renderer->RemoveActor(mFusionActor);
    mFusionActor->Delete();
    }
  mFusion = NULL;
  mFusionActor = NULL;
  mFusionMapper = NULL;
}

void vvSlicer::SetVFSubSampling(int sub)
{
  //std::cout <<"vvSlicer::SetVFSubSampling(" << sub << ")" << std::endl;
  if(mSampleFilter)
    mSubSampling = sub;
  UpdateDisplayExtent();
  Render();
}

void vvSlicer::SetVFScale(int scale)
{
  //std::cout <<"vvSlicer::SetVFScale(" << scale << ")" << std::endl;
  mScale = scale;
  if(mArrow)
    mArrow->SetScale(mScale);
  UpdateDisplayExtent();
  Render();
}

void vvSlicer::SetVFVisibility(int vis)
{
  //std::cout <<"vvSlicer::SetVFVisibility(" << vis << ")" << std::endl;
  if(mVFActor)
  {
    if(vis > 0)
      mVFActor->SetVisibility(1);
    else
      mVFActor->SetVisibility(0);
  }
  UpdateDisplayExtent();
  Render();
}

void vvSlicer::SetOverlayVisibility(int vis)
{
  //std::cout <<"vvSlicer::SetOverlayVisibility(" << vis << ")" << std::endl;
  if(mOverlay)
  {
    if(vis > 0)
      mOverlayActor->SetVisibility(1);
    else
      mOverlayActor->SetVisibility(0);
  }
  UpdateDisplayExtent();
  Render();
}

void vvSlicer::SetFusionVisibility(int vis)
{
  //std::cout <<"vvSlicer::SetFusionVisibility(" << vis << ")" << std::endl;
  if(mFusion)
  {
    if(vis > 0)
      mFusionActor->SetVisibility(1);
    else
      mFusionActor->SetVisibility(0);
  }
  UpdateDisplayExtent();
  Render();
}

void vvSlicer::SetVFLog(int log)
{
  //std::cout <<"vvSlicer::SetVFLog(" << log << ")" << std::endl;
  mVFLog = log;
  if(mGlyphFilter)
    {
    mGlyphFilter->SetUseLog(mVFLog);
    mGlyphFilter->Modified();
    }
  UpdateDisplayExtent();
  Render();
}

void vvSlicer::SetTSlice(int t)
{
  //std::cout <<"vvSlicer::SetTSlice(" << t << ")" << std::endl;
  if(t < 0)
    t = 0;
  else if ((unsigned int)t >= mImage->GetImage().size())
    t = mImage->GetImage().size() -1;
  mCurrentTSlice = t;
  this->SetInput(mImage->GetImage()[t]);
  if(mVF && mVFActor->GetVisibility())
    {
    if(mVF->GetImage().size() == 1)
      {
      mSampleFilter->SetInput(mVF->GetImage()[0]);
      }
    else if (mVF->GetImage().size() <= (unsigned int)mCurrentTSlice)
      {
      Renderer->RemoveActor(mVFActor);
      }
    else
      {
      if(!Renderer->HasViewProp(mVFActor))
        {
        Renderer->AddActor(mVFActor);
        }
      mSampleFilter->SetInput(mVF->GetImage()[mCurrentTSlice]);
      }
    }
  if(mOverlay && mOverlayActor->GetVisibility())
    {
    if(mOverlay->GetImage().size() == 1)
      {
      mOverlayMapper->SetInput(mOverlay->GetImage()[0]);
      }
    else if (mOverlay->GetImage().size() <= (unsigned int)mCurrentTSlice)
      {
      Renderer->RemoveActor(mOverlayActor);
      }
    else
      {
      if(!Renderer->HasViewProp(mOverlayActor))
        {
        Renderer->AddActor(mOverlayActor);
        }
      mOverlayMapper->SetInput(mOverlay->GetImage()[mCurrentTSlice]);
      }
    }
  if(mFusion && mFusionActor->GetVisibility())
    {
    if(mFusion->GetImage().size() == 1)
      {
      mFusionMapper->SetInput(mFusion->GetImage()[0]);
      }
    else if (mFusion->GetImage().size() <= (unsigned int)mCurrentTSlice)
      {
      Renderer->RemoveActor(mFusionActor);
      }
    else
      {
      if(!Renderer->HasViewProp(mFusionActor))
        {
        Renderer->AddActor(mFusionActor);
        }
      mFusionMapper->SetInput(mFusion->GetImage()[mCurrentTSlice]);
      }
    }
  UpdateDisplayExtent();
}

void vvSlicer::SetNextTSlice()
{
  //std::cout <<"vvSlicer::SetNextTSlice()" << std::endl;
  mCurrentTSlice++;
  if((unsigned int)mCurrentTSlice >= mImage->GetImage().size())
    mCurrentTSlice = 0;
  //this->SetInput(mImage->GetImage()[mCurrentTSlice]);
  SetTSlice(mCurrentTSlice);
}

void vvSlicer::SetPreviousTSlice()
{
  //std::cout <<"vvSlicer::SetPreviousTSlice()" << std::endl;
  mCurrentTSlice--;
  if(mCurrentTSlice < 0)
    mCurrentTSlice = mImage->GetImage().size() -1;
  SetTSlice(mCurrentTSlice);
}

int vvSlicer::GetTSlice()
{
  //std::cout <<"vvSlicer::GetTSlice" << std::endl;
  return mCurrentTSlice;
}

void vvSlicer::SetSliceOrientation(int slice)
{
  //std::cout <<"vvSlicer::SetSliceOrientation(" << slice << ")" << std::endl;
  int extent[6];
  this->GetInput()->GetWholeExtent(extent);
  //if 2D image, force to watch in Axial View
  if(extent[5]-extent[4] > 2)
    {
    this->Superclass::SetSliceOrientation(slice);
    }
  else
    {
    this->SliceOrientation = -1;
    this->Superclass::SetSliceOrientation(2);
    }
}

//----------------------------------------------------------------------------
void vvSlicer::UpdateDisplayExtent()
{
  //std::cout <<"vvSlicer::UpdateDisplayExtent()" << std::endl;
  vtkImageData *input = this->GetInput();
  if (!input || !this->ImageActor)
    {
    return;
    }

  input->UpdateInformation();
  int *w_ext = input->GetWholeExtent();

  // Is the slice in range ? If not, fix it

  int slice_min = w_ext[this->SliceOrientation * 2];
  int slice_max = w_ext[this->SliceOrientation * 2 + 1];
  if (this->Slice < slice_min || this->Slice > slice_max)
    {
    this->Slice = static_cast<int>((slice_min + slice_max) * 0.5);
    }

  // Set the image actor

  switch (this->SliceOrientation)
    {
    case vtkImageViewer2::SLICE_ORIENTATION_XY:
      this->ImageActor->SetDisplayExtent(
        w_ext[0], w_ext[1], w_ext[2], w_ext[3], this->Slice, this->Slice);
      if(mVF && mVFActor->GetVisibility())
        {
        int vfExtent[6];
        ComputeVFDisplayedExtent(w_ext[0], w_ext[1], w_ext[2], w_ext[3], this->Slice, this->Slice,vfExtent);
        mSampleFilter->SetVOI(vfExtent);
        mSampleFilter->SetSampleRate(mSubSampling,mSubSampling,1);
        mGlyphFilter->SetOrientation(1,1,0);
        mVFMapper->Update();
        // put the vector field between the image and the camera
        if(Renderer->GetActiveCamera()->GetPosition()[2] > this->Slice)
          mVFActor->SetPosition(0,0,ImageActor->GetBounds()[5]+2);
        else
          mVFActor->SetPosition(0,0,ImageActor->GetBounds()[4]-2);
        }
      if(mOverlay && mOverlayActor->GetVisibility())
        {
        int overExtent[6];
        ComputeOverlayDisplayedExtent(w_ext[0], w_ext[1], w_ext[2], w_ext[3], this->Slice, this->Slice,overExtent);
        mOverlayActor->SetDisplayExtent(overExtent);
        if(Renderer->GetActiveCamera()->GetPosition()[2] > this->Slice)
          mOverlayActor->SetPosition(0,0,1);
        else
          mOverlayActor->SetPosition(0,0,-1);
        }
      if(mFusion && mFusionActor->GetVisibility())
        {
        int fusExtent[6];
        ComputeFusionDisplayedExtent(w_ext[0], w_ext[1], w_ext[2], w_ext[3], this->Slice, this->Slice,fusExtent);
        mFusionActor->SetDisplayExtent(fusExtent);
        if(Renderer->GetActiveCamera()->GetPosition()[2] > this->Slice)
          mFusionActor->SetPosition(0,0,1.5);
        else
          mFusionActor->SetPosition(0,0,-1.5);
        }
      if(mLandActor)
        {
        if(mPlane)
          {
          double bounds [6];
          bounds[0] = ImageActor->GetBounds()[0];
          bounds[1] = ImageActor->GetBounds()[1];
          bounds[2] = ImageActor->GetBounds()[2];
          bounds[3] = ImageActor->GetBounds()[3];
          bounds[4] = ImageActor->GetBounds()[4]-(0.9/this->GetInput()->GetSpacing()[2]);
          bounds[5] = ImageActor->GetBounds()[5]+(0.9/this->GetInput()->GetSpacing()[2]);
          mPlane->SetBounds(bounds);
          UpdateLandmarks();
          }
        if(Renderer->GetActiveCamera()->GetPosition()[2] > this->Slice)
            mLandActor->SetPosition(0,0,1.5);
          else
            mLandActor->SetPosition(0,0,-1.5);
        }
      break;

    case vtkImageViewer2::SLICE_ORIENTATION_XZ:
      this->ImageActor->SetDisplayExtent(
        w_ext[0], w_ext[1], this->Slice, this->Slice, w_ext[4], w_ext[5]);
      if(mVF && mVFActor->GetVisibility())
        {
        int vfExtent[6];
        ComputeVFDisplayedExtent(w_ext[0], w_ext[1], this->Slice, this->Slice, w_ext[4], w_ext[5],vfExtent);
        mSampleFilter->SetVOI(vfExtent);
        mSampleFilter->SetSampleRate(mSubSampling,1,mSubSampling);
        mGlyphFilter->SetOrientation(1,0,1);
        mVFMapper->Update();
        // put the vector field between the image aSpacingnd the camera
        if(Renderer->GetActiveCamera()->GetPosition()[1] > this->Slice)
          mVFActor->SetPosition(0,ImageActor->GetBounds()[3]+2,0);
        else
          mVFActor->SetPosition(0,ImageActor->GetBounds()[2]-2,0);
        }
      if(mOverlay && mOverlayActor->GetVisibility())
        {
        int overExtent[6];
        ComputeOverlayDisplayedExtent(w_ext[0], w_ext[1], this->Slice, this->Slice, w_ext[4], w_ext[5],overExtent);
        mOverlayActor->SetDisplayExtent(overExtent);
        if(Renderer->GetActiveCamera()->GetPosition()[1] > this->Slice)
          mOverlayActor->SetPosition(0,1,0);
        else
          mOverlayActor->SetPosition(0,-1,0);
        }
      if(mFusion && mFusionActor->GetVisibility())
        {
        int fusExtent[6];
        ComputeFusionDisplayedExtent(w_ext[0], w_ext[1], this->Slice, this->Slice, w_ext[4], w_ext[5],fusExtent);
        mFusionActor->SetDisplayExtent(fusExtent);
        if(Renderer->GetActiveCamera()->GetPosition()[1] > this->Slice)
          mFusionActor->SetPosition(0,1.5,0);
        else
          mFusionActor->SetPosition(0,-1.5,0);
        }
      if(mLandActor)
        {
        if(mPlane)
          {
          double bounds [6];
          bounds[0] = ImageActor->GetBounds()[0];
          bounds[1] = ImageActor->GetBounds()[1];
          bounds[2] = ImageActor->GetBounds()[2]-(0.5/this->GetInput()->GetSpacing()[1]);
          bounds[3] = ImageActor->GetBounds()[3]+(0.5/this->GetInput()->GetSpacing()[1]);
          bounds[4] = ImageActor->GetBounds()[4];
          bounds[5] = ImageActor->GetBounds()[5];
          mPlane->SetBounds(bounds);
          UpdateLandmarks();
          }
        if(Renderer->GetActiveCamera()->GetPosition()[1] > this->Slice)
            mLandActor->SetPosition(0,1.5,0);
          else
            mLandActor->SetPosition(0,-1.5,0);
        }
      break;

    case vtkImageViewer2::SLICE_ORIENTATION_YZ:
      this->ImageActor->SetDisplayExtent(
        this->Slice, this->Slice, w_ext[2], w_ext[3], w_ext[4], w_ext[5]);
      if(mVF && mVFActor->GetVisibility())
        {
        int vfExtent[6];
        ComputeVFDisplayedExtent(this->Slice, this->Slice, w_ext[2], w_ext[3], w_ext[4], w_ext[5],vfExtent);
        mSampleFilter->SetVOI(vfExtent);
        mSampleFilter->SetSampleRate(1,mSubSampling,mSubSampling);
        mGlyphFilter->SetOrientation(0,1,1);
        mVFMapper->Update();
        // put the vector field between the image and the camera
        if(Renderer->GetActiveCamera()->GetPosition()[0] > this->Slice)
          mVFActor->SetPosition(ImageActor->GetBounds()[1]+2,0,0);
        else
          mVFActor->SetPosition(ImageActor->GetBounds()[0]-2,0,0);
        }
      if(mOverlay && mOverlayActor->GetVisibility())
        {
        int overExtent[6];
        ComputeOverlayDisplayedExtent(this->Slice, this->Slice, w_ext[2], w_ext[3], w_ext[4], w_ext[5],overExtent);
        mOverlayActor->SetDisplayExtent(overExtent);
        if(Renderer->GetActiveCamera()->GetPosition()[0] > this->Slice)
          mOverlayActor->SetPosition(1,0,0);
        else
          mOverlayActor->SetPosition(-1,0,0);
        }
      if(mFusion && mFusionActor->GetVisibility())
        {
        int fusExtent[6];
        ComputeFusionDisplayedExtent(this->Slice, this->Slice, w_ext[2], w_ext[3], w_ext[4], w_ext[5],fusExtent);
        mFusionActor->SetDisplayExtent(fusExtent);
        if(Renderer->GetActiveCamera()->GetPosition()[0] > this->Slice)
          mFusionActor->SetPosition(1.5,0,0);
        else
          mFusionActor->SetPosition(-1.5,0,0);
        }
      if(mLandActor)
        {
        if(mPlane)
          {
          double bounds [6];
          bounds[0] = ImageActor->GetBounds()[0]-(0.5/this->GetInput()->GetSpacing()[0]);
          bounds[1] = ImageActor->GetBounds()[1]+(0.5/this->GetInput()->GetSpacing()[0]);
          bounds[2] = ImageActor->GetBounds()[2];
          bounds[3] = ImageActor->GetBounds()[3];
          bounds[4] = ImageActor->GetBounds()[4];
          bounds[5] = ImageActor->GetBounds()[5];
          mPlane->SetBounds(bounds);
          UpdateLandmarks();
          }
        if(Renderer->GetActiveCamera()->GetPosition()[0] > this->Slice)
          mLandActor->SetPosition(1.5,0,0);
        else
          mLandActor->SetPosition(-1.5,0,0);
        }
      break;
    }

  // Figure out the correct clipping range

  if (this->Renderer)
    {
    if (this->InteractorStyle && 
        this->InteractorStyle->GetAutoAdjustCameraClippingRange())
      {
      this->Renderer->ResetCameraClippingRange();
      }
    else
      {
      vtkCamera *cam = this->Renderer->GetActiveCamera();
      if (cam)
        {
        double bounds[6];
        this->ImageActor->GetBounds(bounds);
        double spos = (double)bounds[this->SliceOrientation * 2];
        double cpos = (double)cam->GetPosition()[this->SliceOrientation];
        double range = fabs(spos - cpos);
        double *spacing = input->GetSpacing();
        double avg_spacing = 
          ((double)spacing[0] + (double)spacing[1] + (double)spacing[2]) / 3.0;
        cam->SetClippingRange(
          range - avg_spacing * 3.0, range + avg_spacing * 3.0);
        }
      }
    }
}

void vvSlicer::ComputeVFDisplayedExtent(int x1,int x2,int y1,int y2,int z1,int z2,int vfExtent[6])
{
  //std::cout <<"ComputeVFExtent()" << std::endl;
  vfExtent[0] = (int)(((x1 - this->GetInput()->GetOrigin()[0]) /
		mVF->GetSpacing()[0])*this->GetInput()->GetSpacing()[0] + mVF->GetOrigin()[0]);
  vfExtent[1] = (int)(((x2 - this->GetInput()->GetOrigin()[0]) /
		mVF->GetSpacing()[0])*this->GetInput()->GetSpacing()[0] + mVF->GetOrigin()[0]);
  vfExtent[2] = (int)(((y1 - this->GetInput()->GetOrigin()[1]) /
		mVF->GetSpacing()[1])*this->GetInput()->GetSpacing()[1] + mVF->GetOrigin()[1]);
  vfExtent[3] = (int)(((y2 - this->GetInput()->GetOrigin()[1]) /
		mVF->GetSpacing()[1])*this->GetInput()->GetSpacing()[1] + mVF->GetOrigin()[1]);
  vfExtent[4] = (int)(((z1 - this->GetInput()->GetOrigin()[2]) /
		mVF->GetSpacing()[2])*this->GetInput()->GetSpacing()[2] + mVF->GetOrigin()[2]);
  vfExtent[5] = (int)(((z2 - this->GetInput()->GetOrigin()[2]) /
		mVF->GetSpacing()[2])*this->GetInput()->GetSpacing()[2] + mVF->GetOrigin()[2]);

  ClipDisplayedExtent(vfExtent,mSampleFilter->GetInput()->GetWholeExtent());
}

void vvSlicer::ComputeOverlayDisplayedExtent(int x1,int x2,int y1,int y2,int z1,int z2,int overExtent[6])
{
  //std::cout <<"ComputeOverlayExtent()" << std::endl;
  overExtent[0] = (int)(((x1 - this->GetInput()->GetOrigin()[0]) /
		mOverlay->GetSpacing()[0])*this->GetInput()->GetSpacing()[0] + mOverlay->GetOrigin()[0]);
  overExtent[1] = (int)(((x2 - this->GetInput()->GetOrigin()[0]) /
		mOverlay->GetSpacing()[0])*this->GetInput()->GetSpacing()[0] + mOverlay->GetOrigin()[0]);
  overExtent[2] = (int)(((y1 - this->GetInput()->GetOrigin()[1]) /
		mOverlay->GetSpacing()[1])*this->GetInput()->GetSpacing()[1] + mOverlay->GetOrigin()[1]);
  overExtent[3] = (int)(((y2 - this->GetInput()->GetOrigin()[1]) /
		mOverlay->GetSpacing()[1])*this->GetInput()->GetSpacing()[1] + mOverlay->GetOrigin()[1]);
  overExtent[4] = (int)(((z1 - this->GetInput()->GetOrigin()[2]) /
		mOverlay->GetSpacing()[2])*this->GetInput()->GetSpacing()[2] + mOverlay->GetOrigin()[2]);
  overExtent[5] = (int)(((z2 - this->GetInput()->GetOrigin()[2]) /
		mOverlay->GetSpacing()[2])*this->GetInput()->GetSpacing()[2] + mOverlay->GetOrigin()[2]);
  ClipDisplayedExtent(overExtent, mOverlayMapper->GetInput()->GetWholeExtent());
}

void vvSlicer::ComputeFusionDisplayedExtent(int x1,int x2,int y1,int y2,int z1,int z2,int fusExtent[6])
{
  //std::cout <<"ComputeFusionDisplayedExtent()" << std::endl;
  fusExtent[0] = (int)(((x1 - this->GetInput()->GetOrigin()[0]) /
		mFusion->GetSpacing()[0])*this->GetInput()->GetSpacing()[0] + mFusion->GetOrigin()[0]);
  fusExtent[1] = (int)(((x2 - this->GetInput()->GetOrigin()[0]) /
		mFusion->GetSpacing()[0])*this->GetInput()->GetSpacing()[0] + mFusion->GetOrigin()[0]);
  fusExtent[2] = (int)(((y1 - this->GetInput()->GetOrigin()[1]) /
		mFusion->GetSpacing()[1])*this->GetInput()->GetSpacing()[1] + mFusion->GetOrigin()[1]);
  fusExtent[3] = (int)(((y2 - this->GetInput()->GetOrigin()[1]) /
		mFusion->GetSpacing()[1])*this->GetInput()->GetSpacing()[1] + mFusion->GetOrigin()[1]);
  fusExtent[4] = (int)(((z1 - this->GetInput()->GetOrigin()[2]) /
		mFusion->GetSpacing()[2])*this->GetInput()->GetSpacing()[2] + mFusion->GetOrigin()[2]);
  fusExtent[5] = (int)(((z2 - this->GetInput()->GetOrigin()[2]) /
		mFusion->GetSpacing()[2])*this->GetInput()->GetSpacing()[2] + mFusion->GetOrigin()[2]);
  ClipDisplayedExtent(fusExtent, mFusionMapper->GetInput()->GetWholeExtent());
}

void vvSlicer::ClipDisplayedExtent(int extent[6], int refExtent[6])
{
  //std::cout << "ClipDisplayedExtent()" << std::endl;
  bool out = false;
  int maxBound = 6;

  //2D overlay on 3D image specific case
  if(refExtent[4] == refExtent[5])
    {
    maxBound = 4;
    extent[4] = refExtent[4];
    extent[5] = refExtent[5];
    }

  for(int i = 0; i < maxBound; i = i+2)
    {
    //if we are totally outside the image
    if((extent[i] < refExtent[i] ||
	extent[i] > refExtent[i+1]) &&
	(extent[i+1] < refExtent[i] ||
	extent[i+1] > refExtent[i+1]))
	{
	out = true;
        break;
        }
    //crop to the limit of the image
    extent[i] = (extent[i] > refExtent[i]) ? extent[i] : refExtent[i];
    extent[i] = (extent[i] < refExtent[i+1]) ? extent[i] : refExtent[i+1];
    extent[i+1] = (extent[i+1] > refExtent[i]) ? extent[i+1] : refExtent[i];
    extent[i+1] = (extent[i+1] < refExtent[i+1]) ? extent[i+1] : refExtent[i+1];
    }
  if(out)
    for(int i = 0; i < maxBound; i = i+2)
      {
      extent[i] = refExtent[i];
      extent[i+1] = refExtent[i];
      }
}

void vvSlicer::UpdateOrientation()
{
 //std::cout <<"vvSlicer::UpdateOrientation" << std::endl;
  // Set the camera position
  vtkCamera *cam = this->Renderer ? this->Renderer->GetActiveCamera() : NULL;
  if (cam)
    {
    switch (this->SliceOrientation)
      {
      case vtkImageViewer2::SLICE_ORIENTATION_XY:
        cam->SetFocalPoint(0,0,0);
        cam->SetPosition(0,0,-1); // -1 if medical ?
        cam->SetViewUp(0,-1,0);
        break;
        
      case vtkImageViewer2::SLICE_ORIENTATION_XZ:
        cam->SetFocalPoint(0,0,0);
        cam->SetPosition(0,-1,0); // 1 if medical ?
        cam->SetViewUp(0,0,1);
        break;
        
      case vtkImageViewer2::SLICE_ORIENTATION_YZ:
        cam->SetFocalPoint(0,0,0);
        cam->SetPosition(-1,0,0); // -1 if medical ?
        cam->SetViewUp(0,0,1);
        break;
      }
    }
}

void vvSlicer::SetOpacity(double s)
{
  this->GetImageActor()->SetOpacity(s);
}

void vvSlicer::SetRenderWindow(int orientation, vtkRenderWindow * rw)
{
  this->Superclass::SetRenderWindow(rw);
  this->SetRenderer(rw->GetRenderers()->GetFirstRenderer());
  this->SetupInteractor(rw->GetInteractor());
  ca->SetImageActor(this->GetImageActor());
  ca->SetWindowLevel(this->GetWindowLevel());
  ca->SetText(2, "<slice>");
  ca->SetText(3, "<window>\n<level>");
  

  double bounds[6];
  double max = 65000;

  bounds[0] = -max;
  bounds[1] = max;
  bounds[2] = -max;
  bounds[3] = max;
  bounds[4] = -max;
  bounds[5] = max;

  crossCursor->SetModelBounds(bounds);
  this->GetRenderer()->AddActor(pdmA);
  this->GetRenderer()->AddActor(ca);
  this->GetRenderer()->AddActor(textActor);
  this->GetRenderer()->ResetCamera();

  int extent[6];
  this->GetInput()->GetWholeExtent(extent);

  //if 2D image, force to watch in Axial View
  if(extent[5]-extent[4] > 2)
    {
    if(orientation == 0)
      this->SliceOrientation = orientation;
    if(orientation < 3)
      this->SliceOrientation = 2-orientation;
    }
  else
    {
    this->SliceOrientation = 2;
    }

}

void vvSlicer::SetDisplayMode(int i)
{
  //std::cout <<"vvSlicer::SetDisplayMode (" << i << ")" << std::endl;
  if(i == 0)
    {
    this->GetImageActor()->SetVisibility(0);
    this->GetAnnotation()->SetVisibility(0);
    if(mVFActor)
      mVFActor->SetVisibility(0);
    if(mOverlayActor)
      mOverlayActor->SetVisibility(0);
    if(mFusionActor)
      mFusionActor->SetVisibility(0);
    if(mLandActor)
      mLandActor->SetVisibility(0);
    pdmA->SetVisibility(0);
    }
  else
    {
    this->GetImageActor()->SetVisibility(1);
    int orientation = this->GetSliceOrientation();
    this->SliceOrientation = -1;
    this->Superclass::SetSliceOrientation(orientation);
    this->GetAnnotation()->SetVisibility(1);
    if(mVFActor)
      mVFActor->SetVisibility(1);
    if(mOverlayActor)
      mOverlayActor->SetVisibility(1);
    if(mFusionActor)
      mFusionActor->SetVisibility(1);
    if(mLandActor)
      mLandActor->SetVisibility(1);
    UpdateDisplayExtent();
    }
    this->Render();
}

void vvSlicer::FlipHorizontalView()
{
 vtkCamera *cam = this->Renderer ? this->Renderer->GetActiveCamera() : NULL;
  if (cam)
    {
    double *position = cam->GetPosition();
    switch (this->SliceOrientation)
      {
       case vtkImageViewer2::SLICE_ORIENTATION_XY:
        cam->SetPosition(position[0],position[1],-position[2]);
        break;
        
      case vtkImageViewer2::SLICE_ORIENTATION_XZ:
        cam->SetPosition(position[0],-position[1],position[2]);
        break;
        
      case vtkImageViewer2::SLICE_ORIENTATION_YZ:
        cam->SetPosition(-position[0],position[1],position[2]);
        break;
      }
    this->Renderer->ResetCameraClippingRange();
    this->UpdateDisplayExtent();
    }
}

void vvSlicer::FlipVerticalView()
{
 vtkCamera *cam = this->Renderer ? this->Renderer->GetActiveCamera() : NULL;
  if (cam)
    {
    FlipHorizontalView();
    double *viewup = cam->GetViewUp();
    cam->SetViewUp(-viewup[0],-viewup[1],-viewup[2]);
    this->UpdateDisplayExtent();
    }
}

void vvSlicer::Render()
{
  //std::cout <<"vvSlicer::render " << this->GetImage()->GetNumberOfDimensions() << "D et orientation : " << this->GetSliceOrientation() << std::endl;
  if(ca->GetVisibility())
    {
    std::string worldPos = "";
    std::stringstream world1;
    std::stringstream world2;
    std::stringstream world3;
    world1 << (int)mCurrent[0];
    world2 << (int)mCurrent[1];
    world3 << (int)mCurrent[2];
    double X = (mCurrent[0] - this->GetInput()->GetOrigin()[0])/this->GetInput()->GetSpacing()[0];
    double Y = (mCurrent[1] - this->GetInput()->GetOrigin()[1])/this->GetInput()->GetSpacing()[1];
    double Z = (mCurrent[2] - this->GetInput()->GetOrigin()[2])/this->GetInput()->GetSpacing()[2];

    if(pdmA->GetVisibility())
      {
      double x = mCursor[0];
      double y = mCursor[1];
      double z = mCursor[2];
      double t = mCursor[3];
      double xCursor = (x - this->GetInput()->GetOrigin()[0])/this->GetInput()->GetSpacing()[0];
      double yCursor = (y - this->GetInput()->GetOrigin()[1])/this->GetInput()->GetSpacing()[1];
      double zCursor = (z - this->GetInput()->GetOrigin()[2])/this->GetInput()->GetSpacing()[2];

      if(xCursor >= this->GetImageActor()->GetDisplayExtent()[0] && 
       xCursor < this->GetImageActor()->GetDisplayExtent()[1]+1 && 
       yCursor >= this->GetImageActor()->GetDisplayExtent()[2] && 
       yCursor < this->GetImageActor()->GetDisplayExtent()[3]+1 && 
       zCursor >= this->GetImageActor()->GetDisplayExtent()[4] && 
       zCursor < this->GetImageActor()->GetDisplayExtent()[5]+1 && 
       t == mCurrentTSlice)
        {
        vtkRenderer * renderer = this->Renderer;

        renderer->WorldToView(x,y,z);
        renderer->ViewToNormalizedViewport(x,y,z);
        renderer->NormalizedViewportToViewport(x,y);
        renderer->ViewportToNormalizedDisplay(x,y);
        renderer->NormalizedDisplayToDisplay(x,y);
        crossCursor->SetFocalPoint(x,y,z);
        }
      else
        crossCursor->SetFocalPoint(-1,-1,z);
      }

    if(X >= this->GetInput()->GetWholeExtent()[0] && 
       X <= this->GetInput()->GetWholeExtent()[1] && 
       Y >= this->GetInput()->GetWholeExtent()[2] && 
       Y <= this->GetInput()->GetWholeExtent()[3] && 
       Z >= this->GetInput()->GetWholeExtent()[4] && 
       Z <= this->GetInput()->GetWholeExtent()[5])
      {
      std::stringstream pixel1;
      std::stringstream pixel2;
      std::stringstream pixel3;
      std::stringstream temps;
      pixel1 << (int)X;
      pixel2 << (int)Y;
      pixel3 << (int)Z;
      temps << mCurrentTSlice;
      double value = this->GetInput()->GetScalarComponentAsDouble(
		(int)X,
		(int)Y,
		(int)Z,0);

      std::stringstream val;
      val << value;
      worldPos += "data value : " + val.str();
      worldPos += "\n mm : " + world1.str() + " " + world2.str() + " " + world3.str() + " " + temps.str();
      worldPos += "\n pixel : " + pixel1.str() + " " + pixel2.str() + " " + pixel3.str() + " " + temps.str();
      }
    ca->SetText(1,worldPos.c_str());
    }
  if(mOverlay && mOverlayActor->GetVisibility())
    {
    mOverlayMapper->SetWindow(this->GetColorWindow());
    mOverlayMapper->SetLevel(this->GetColorLevel());
    mOverlayMapper->Update();
    }
  if(mLandMapper)
    {
    UpdateLandmarks();
    /*mPlane->Modified();
    mLandClipper->Update();
    mLandMapper->Update();*/
    }
  this->Superclass::Render();
}

void vvSlicer::UpdateCursorPosition()
{
  if(this->GetImageActor()->GetVisibility())
  {
    pdmA->SetVisibility(1);
    mCursor[0] = mCurrent[0];
    mCursor[1] = mCurrent[1];
    mCursor[2] = mCurrent[2];
    mCursor[3] = mCurrentTSlice;
  }
}

void vvSlicer::UpdateLandmarks()
{
  //std::cout << "UpdateLandmarks()" << std::endl;
  vtkPolyData *pd = static_cast<vtkPolyData*>(mLandClipper->GetInput());
  if(pd->GetPoints())
    {
    /*for(int i = 0; i < mLandmarks->GetNumberOfSources(); i ++)
      {
      mLandGlyph->SetSource(mCross->GetOutput());
      }*/
    mLandGlyph->SetRange(0,1);
    mLandGlyph->Modified();
    mLandGlyph->Update();

    mPlane->Modified();
    mLandClipper->Update();
    mLandMapper->Update();
    }

}

void vvSlicer::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os, indent);
}
