/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvSlicer.h,v $
 Language:  C++
 Date:      $Date: 2008/07/11 15:44:28 $
 Version:   $Revision: 1.9 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef __vvSlicer_h
#define __vvSlicer_h

#include <iostream>
#include <vector>

#include "vvLandmarks.h"
#include "vvImage.h"
#include "vvVF.h"
#include "vtkImageViewer2.h"
#include "vtkActor2D.h"
#include "vvCursor2D.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkProperty2D.h"

#include "vtkClipPolyData.h"
#include "vtkBox.h"

#include "vtkCornerAnnotation.h"

#include "vtkStructuredGrid.h"
#include "vtkExtractGrid.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkPolyDataMapper.h"
#include "vtkGlyph3D.h"
#include "vtkLODActor.h"
#include "vvGlyph2D.h"
#include "vvGlyphSource.h"
#include "vtkCursor3D.h"

#include "vvHelp.h"

class vvSlicer: public vtkImageViewer2
{
public:
  static vvSlicer *New();
  vtkTypeRevisionMacro(vvSlicer,vtkImageViewer2);
  void PrintSelf(ostream& os, vtkIndent indent);

  void SetImage(vvImage* inputImages);
  vvImage *GetImage(){return mImage;}

  void SetOverlay(vvImage* inputOverlay);
  vvImage* GetOverlay(){return mOverlay;}
  vtkImageMapToWindowLevelColors* GetOverlayMapper(){return mOverlayMapper;}
  vtkImageActor* GetOverlayActor(){return mOverlayActor;}
  void SetOverlayVisibility(int vis);
  void RemoveOverlay();

  void SetFusion(vvImage* inputFusion);
  vvImage* GetFusion(){return mFusion;}
  vtkImageMapToWindowLevelColors* GetFusionMapper(){return mFusionMapper;}
  vtkImageActor* GetFusionActor(){return mFusionActor;}
  void SetFusionVisibility(int vis);
  void RemoveFusion();

  void SetVF(vvVF* vf);
  vvVF *GetVF(){return mVF;}
  vtkLODActor* GetVFActor(){return mVFActor;}
  void RemoveVF();

  void SetLandmarks(vvLandmarks* landmarks);
  void SetTSlice(int t);
  void SetNextTSlice();
  void SetPreviousTSlice();
  void SetSliceOrientation(int slice);
  int GetTSlice();
  int GetTMax(){return mImage->GetImage().size() - 1;}

  void SetOpacity(double s);
  void SetRenderWindow(int orientation, vtkRenderWindow * rw);
  void SetDisplayMode(int i);
  void FlipHorizontalView();
  void FlipVerticalView();
  void Render();

  void SetVFSubSampling(int sub);
  int GetVFSubSampling(){return mSubSampling;}
  void SetVFScale(int scale);
  int GetVFScale(){return mScale;}
  void SetVFLog(int log);
  int GetVFLog(){return mVFLog;}
  void SetVFVisibility(int vis);

  void SetFileName(std::string filename){mFileName = filename;}
  std::string GetFileName(){return mFileName;}

  vtkCornerAnnotation* GetAnnotation(){return ca;}

  vvHelp* GetHelp(){return textActor;}

  void SetCurrentPosition(double x, double y, double z, int t);
  double* GetCurrentPosition(){return mCurrent;}

  void UpdateCursorPosition();
  double* GetCursorPosition(){return mCursor;}
  void SetCursorVisibility(int s){pdmA->SetVisibility(s);}
  int GetCursorVisibility(){return pdmA->GetVisibility();}
  void SetCursorColor(int r,int g, int b){pdmA->GetProperty()->SetColor(r,g,b);}

  void UpdateLandmarks();


protected:
  vvSlicer();
  ~vvSlicer();

  std::string mFileName;
  vvImage* mImage;

  vvImage* mOverlay;
  vtkImageMapToWindowLevelColors * mOverlayMapper;
  vtkImageActor *mOverlayActor;

  vvImage* mFusion;
  vtkImageMapToWindowLevelColors * mFusionMapper;
  vtkImageActor *mFusionActor;

  vtkCornerAnnotation *ca;
  vvCursor2D *crossCursor;
  vtkPolyDataMapper2D *pdm;
  vtkActor2D *pdmA;

  vvVF* mVF;
  vvGlyphSource* mArrow;
  vtkExtractGrid *mSampleFilter;
  vvGlyph2D *mGlyphFilter;
  vtkPolyDataMapper *mVFMapper;
  vtkLODActor *mVFActor;

  vvLandmarks* mLandmarks;
  vtkGlyph3D* mLandGlyph;
  vtkCursor3D* mCross;
  vtkClipPolyData* mLandClipper;
  vtkPolyDataMapper* mLandMapper;
  vtkActor* mLandActor;
  vtkBox* mPlane;

  vvHelp *textActor;

  int mCurrentTSlice;
  double mCurrent[3];
  double mCursor[4];
  int mSubSampling;
  int mScale;
  int mVFLog;

private:
  void UpdateOrientation();
  void UpdateDisplayExtent();
  void ComputeVFDisplayedExtent(int x1,int x2,int y1,int y2,int z1,int z2,int extent[6]);
  void ComputeOverlayDisplayedExtent(int x1,int x2,int y1,int y2,int z1,int z2,int overExtent[6]);
  void ComputeFusionDisplayedExtent(int x1,int x2,int y1,int y2,int z1,int z2,int overExtent[6]);
  void ClipDisplayedExtent(int extent[6], int refExtent[6]);
  

};
#endif
