/*=========================================================================

Program:   vv
Module:    $RCSfile: vvSlicersManagerCommand.cxx,v $
Language:  C++
Date:      $Date: 2008/07/15 11:26:35 $
Version:   $Revision: 1.18 $
Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#include "vvSlicersManagerCommand.h"
#include "vvSlicersManager.h"

#include "vtkTextProperty.h"
#include "vtkRenderer.h"
#include "vtkImageActor.h"
#include "vvInteractorStyleNavigator.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkPropPicker.h"
#include "vtkCamera.h"
#include "vtkImageMapToWindowLevelColors.h"
#include "vtkLookupTable.h"
#include "vtkMath.h"
#include "vtkAbstractPropPicker.h"
#include "vtkAssemblyPath.h"


vvSlicersManagerCallback::vvSlicersManagerCallback()
{
  mCurrentSlicer = -1;
  mStartSlicer = -1;
}

//this function return the number of the picked slicer
// or -1 if no slicer of the manager is picked
int vvSlicersManagerCallback::IsOneOfTheSlicer(int x, int y,
					       vvInteractorStyleNavigator* isi, vvSlicersManager *IV)
{
  vtkPropPicker *interactionPicker = vtkPropPicker::New();
  vtkRenderer * renderer = isi->GetInteractor()->FindPokedRenderer(x,y);
  interactionPicker->PickProp(x, y, renderer);
  vtkProp *prop = interactionPicker->GetViewProp();
  vtkImageActor* pickedActor;
  if (!prop)
    {
      interactionPicker->Delete();
      return -1;
    }
  else
    {
      pickedActor = vtkImageActor::SafeDownCast(prop);
    }
  int found = -1;
  for(int i = 0; i < IV->NumberOfSlicers(); i++)
    {
      if(IV->GetSlicer(i)->GetImageActor() 
	 && prop 
	 && IV->GetSlicer(i)->GetImageActor() == pickedActor)
	{
	  found = i;
	  break;
	}
    }
  interactionPicker->Delete();
  return found;
}

//return the window number
int vvSlicersManagerCallback::IsWindowNumber(int x, int y,
					     vvInteractorStyleNavigator* isi, vvSlicersManager *IV)
{
  vtkRenderer * renderer = isi->GetInteractor()->FindPokedRenderer(x,y);
  for(int i = 0; i < IV->NumberOfSlicers(); i++)
    {
      if(IV->GetSlicer(i)->GetRenderer() == renderer)
	{
	  return i;
	}
    }
  return -1;
}

//return the num of the current window if visible (-1 else)
int vvSlicersManagerCallback::IsVisibleInWindow(int x, int y,
						vvInteractorStyleNavigator* isi, vvSlicersManager *IV)
{
  vtkRenderer * renderer = isi->GetInteractor()->FindPokedRenderer(x,y);
  for(int i = 0; i < IV->NumberOfSlicers(); i++)
    {
      if(IV->GetSlicer(i)->GetRenderer() == renderer
	 && IV->GetSlicer(i)->GetImageActor()->GetVisibility())
	{
	  return i;
	}
    }
  return -1;
}

void vvSlicersManagerCallback::Execute(vtkObject *caller, 
				       unsigned long event, 
				       void *vtkNotUsed(callData))
{
  //std::cout << "Execute" << std::endl;
  //KeyPress event
  vvInteractorStyleNavigator *isi = 
    static_cast<vvInteractorStyleNavigator *>(caller);
  double x = isi->GetInteractor()->GetEventPosition()[0];
  double y = isi->GetInteractor()->GetEventPosition()[1];
  double z;

  double xAbsolu = x;
  double yAbsolu = y;
  double zAbsolu = z;
  mCurrentSlicer = IsOneOfTheSlicer((int)floor(x),(int)floor(y),isi,this->SM);
  int VisibleInWindow = this->IsVisibleInWindow((int)floor(x),(int)floor(y),isi,this->SM);
  newLandmark = false;

  if(event == vtkCommand::StartPickEvent && VisibleInWindow == -1)
    {
      for(int i = 0; i < this->SM->NumberOfSlicers(); i++)
	{
	  if(this->SM->GetSlicer(i)->GetCursorVisibility())
	    {
	      this->SM->GetSlicer(i)->SetCursorVisibility(0);
	      this->SM->GetSlicer(i)->Render();
	    }
	}
    }

  if (event == vtkCommand::KeyPressEvent)
    {
    std::string KeyPress = isi->GetInteractor()->GetKeySym();
    // if the mouse is on the good slicer
    if(mCurrentSlicer > -1)
      {
      if (KeyPress == "Left")
        {
        this->SM->SetPreviousTSlice();
        this->SM->UpdateTSlice(mCurrentSlicer);
        }
      if (KeyPress == "Right")
        {
        this->SM->SetNextTSlice();
        this->SM->UpdateTSlice(mCurrentSlicer);
        }
      if (KeyPress == "f" || KeyPress == "F")
        {
        FlyToPosition(isi->GetInteractor(),this->SM->GetSlicer(mCurrentSlicer));
        }
      if (KeyPress == "0")
        {
        this->SM->SetPreset(0);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "1")
        {
        this->SM->SetPreset(1);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "2")
        {
        this->SM->SetPreset(2);
        this->SM->UpdateWindowLevel();

        return;
        }
      if (KeyPress == "3")
        {
        this->SM->SetPreset(3);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "4")
        {
        this->SM->SetPreset(4);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "5")
        {
        this->SM->SetPreset(5);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "6")
        {
        this->SM->SetColorMap(0);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "7")
        {
        this->SM->SetColorMap(1);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "8")
        {
        this->SM->SetColorMap(2);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "9")
        {
        this->SM->SetColorMap(3);
        this->SM->UpdateWindowLevel();
        return;
        }
      if (KeyPress == "l")
        {
        this->SM->Reload();
        this->SM->Render();
        return;
        }
      if(KeyPress == "g")
        {
        double* cursorPos = this->SM->GetSlicer(mCurrentSlicer)->GetCursorPosition();
        this->SM->GetSlicer(mCurrentSlicer)->SetCurrentPosition(
		cursorPos[0],cursorPos[1],cursorPos[2],cursorPos[3]);
        this->SM->UpdateViews(1,mCurrentSlicer);
        this->SM->UpdateLinked(mCurrentSlicer);
        return;
        }
      if (KeyPress == "F5")
        {
        this->SM->GetSlicer(mCurrentSlicer)->FlipHorizontalView();
        this->SM->GetSlicer(mCurrentSlicer)->Render();
        }
      if (KeyPress == "F6")
        {
        this->SM->GetSlicer(mCurrentSlicer)->FlipVerticalView();
        this->SM->GetSlicer(mCurrentSlicer)->Render();
        }
      if (KeyPress == "Up")
        {
        this->SM->GetSlicer(mCurrentSlicer)->SetSlice(this->SM->GetSlicer(mCurrentSlicer)->GetSlice()+1);
        this->SM->UpdateSlice(mCurrentSlicer);
        this->SM->UpdateInfoOnCursorPosition(mCurrentSlicer);
        }
      if (KeyPress == "Down")
        {
        this->SM->GetSlicer(mCurrentSlicer)->SetSlice(this->SM->GetSlicer(mCurrentSlicer)->GetSlice()-1);
        this->SM->UpdateSlice(mCurrentSlicer);
        this->SM->UpdateInfoOnCursorPosition(mCurrentSlicer);
        }
      if(KeyPress == "space")
        {
        newLandmark = true;
        }
      }

    //If we are in the good window
    int WindowNumber = this->IsWindowNumber((int)floor(x),(int)floor(y),isi,this->SM);

    if (WindowNumber > -1 && KeyPress == "h")
      {
      if(this->SM->GetSlicer(WindowNumber)->GetHelp()->GetVisibility())
        this->SM->GetSlicer(WindowNumber)->GetHelp()->SetVisibility(0);
      else
        this->SM->GetSlicer(WindowNumber)->GetHelp()->SetVisibility(1);
      this->SM->GetSlicer(WindowNumber)->Render();
      return;
      }
    if(VisibleInWindow > -1)
      {
      if (KeyPress == "F1")
        {
        this->SM->GetSlicer(VisibleInWindow)->GetAnnotation()->SetText(2,"Sagital\n<slice>");
          this->SM->GetSlicer(VisibleInWindow)->SetSliceOrientation(0);
        this->SM->UpdateSliceRange(VisibleInWindow);
        this->SM->UpdateInfoOnCursorPosition(VisibleInWindow);
        }
      if (KeyPress == "F2")
        {
        this->SM->GetSlicer(VisibleInWindow)->GetAnnotation()->SetText(2,"Coronal\n<slice>");
          this->SM->GetSlicer(VisibleInWindow)->SetSliceOrientation(1);
        this->SM->UpdateSliceRange(VisibleInWindow);
        this->SM->UpdateInfoOnCursorPosition(VisibleInWindow);
        }
      if (KeyPress == "F3")
        {
        this->SM->GetSlicer(VisibleInWindow)->GetAnnotation()->SetText(2,"Axial\n<slice>");
        this->SM->GetSlicer(VisibleInWindow)->SetSliceOrientation(2);
        this->SM->UpdateSliceRange(VisibleInWindow);
        this->SM->UpdateInfoOnCursorPosition(VisibleInWindow);
        }
      }
    }

  //All type of mouse events
  if (event == vtkCommand::LeaveEvent && VisibleInWindow > -1)
    {
      this->SM->GetSlicer(VisibleInWindow)->SetCurrentPosition(-VTK_DOUBLE_MAX,-VTK_DOUBLE_MAX,
							       -VTK_DOUBLE_MAX,this->SM->GetSlicer(VisibleInWindow)->GetTSlice());
      this->SM->GetSlicer(VisibleInWindow)->Render();
      return;
    }

  if (event == vtkCommand::StartWindowLevelEvent)
    {
      mStartSlicer = -1;
      this->InitialWindow = this->SM->GetColorWindow();
      this->InitialLevel = this->SM->GetColorLevel();

      if(mCurrentSlicer > -1)
	{
	  mStartSlicer = mCurrentSlicer;
	}
      return;
    }

  if(event == vtkCommand::EndWindowLevelEvent)
    {
      mStartSlicer = -1;
    }

  if(mCurrentSlicer > -1)
    {
      this->SM->Activated();
      //if(!this->SM->GetSlicer(mCurrentSlicer)->GetAnnotation()->GetVisibility())
      this->SM->GetSlicer(mCurrentSlicer)->GetAnnotation()->SetVisibility(1);

      if(event == vtkCommand::MouseWheelForwardEvent && !isi->GetInteractor()->GetControlKey())
	{
	  this->SM->GetSlicer(mCurrentSlicer)->SetSlice(this->SM->GetSlicer(mCurrentSlicer)->GetSlice()+1);
	  this->SM->UpdateSlice(mCurrentSlicer);
          this->SM->UpdateInfoOnCursorPosition(mCurrentSlicer);
	}
      else if(event == vtkCommand::MouseWheelForwardEvent && isi->GetInteractor()->GetControlKey())
	{
	  double factor = 2;
	  this->Dolly(pow((double)1.1, factor),isi->GetInteractor());
	}
      else if(event == vtkCommand::MouseWheelBackwardEvent && !isi->GetInteractor()->GetControlKey())
	{
	  this->SM->GetSlicer(mCurrentSlicer)->SetSlice(this->SM->GetSlicer(mCurrentSlicer)->GetSlice()-1);
	  this->SM->UpdateSlice(mCurrentSlicer);
          this->SM->UpdateInfoOnCursorPosition(mCurrentSlicer);
	}
      else if(event == vtkCommand::MouseWheelBackwardEvent && isi->GetInteractor()->GetControlKey())
	{
	  double factor = -2;
	  this->Dolly(pow((double)1.1, factor),isi->GetInteractor());
	}
      double xWorld, yWorld, zWorld;

      //Move into World Coordinate
      vtkRenderer * renderer = isi->GetInteractor()->FindPokedRenderer((int)floor(x),(int)floor(y));
      renderer->DisplayToNormalizedDisplay(x,y);
      renderer->NormalizedDisplayToViewport(x,y);
      renderer->ViewportToNormalizedViewport(x,y);
      renderer->NormalizedViewportToView(x,y,z);
      renderer->ViewToWorld(x,y,z);
      switch (this->SM->GetSlicer(mCurrentSlicer)->GetSliceOrientation())
	{
	case vtkImageViewer2::SLICE_ORIENTATION_XY:
	  xWorld = x;
	  yWorld = y;
	  zWorld = this->SM->GetSlicer(mCurrentSlicer)->GetSlice()*
	    this->SM->GetSlicer(mCurrentSlicer)->GetInput()->GetSpacing()[2] +
	    this->SM->GetSlicer(mCurrentSlicer)->GetInput()->GetOrigin()[2];
	  break;

	case vtkImageViewer2::SLICE_ORIENTATION_XZ:
	  xWorld = x;
	  yWorld = this->SM->GetSlicer(mCurrentSlicer)->GetSlice()*
	    this->SM->GetSlicer(mCurrentSlicer)->GetInput()->GetSpacing()[1] +
	    this->SM->GetSlicer(mCurrentSlicer)->GetInput()->GetOrigin()[1];
	  zWorld = z;
	  break;

	case vtkImageViewer2::SLICE_ORIENTATION_YZ:
	  xWorld = this->SM->GetSlicer(mCurrentSlicer)->GetSlice()*
	    this->SM->GetSlicer(mCurrentSlicer)->GetInput()->GetSpacing()[0] +
	    this->SM->GetSlicer(mCurrentSlicer)->GetInput()->GetOrigin()[0];
	  yWorld = y;
	  zWorld = z;
	  break;
	}
      this->SM->GetSlicer(mCurrentSlicer)->SetCurrentPosition(xWorld,yWorld,zWorld,
							      this->SM->GetSlicer(mCurrentSlicer)->GetTSlice());
      if(newLandmark)
	{
	  this->SM->AddLandmark(xWorld,yWorld,zWorld,
				this->SM->GetSlicer(mCurrentSlicer)->GetTSlice());
          this->SM->GetSlicer(mCurrentSlicer)->UpdateLandmarks();
	  this->SM->Render();
	}
      if(event == vtkCommand::PickEvent || event == vtkCommand::StartPickEvent)
	{
	  this->SM->UpdateViews(1,mCurrentSlicer);
	  this->SM->UpdateLinked(mCurrentSlicer);
	  this->SM->UpdateInfoOnCursorPosition(mCurrentSlicer);
	}
      else
	{
	  this->SM->GetSlicer(mCurrentSlicer)->Render();
	}
    }
  else if(VisibleInWindow > -1)
    {
      this->SM->GetSlicer(VisibleInWindow)->SetCurrentPosition(-VTK_DOUBLE_MAX,-VTK_DOUBLE_MAX,
							       -VTK_DOUBLE_MAX,this->SM->GetSlicer(VisibleInWindow)->GetTSlice());
      //this->SM->GetSlicer(VisibleInWindow)->Render();
    }

  if(event == vtkCommand::WindowLevelEvent && mStartSlicer > -1)
    {
      this->SM->GetSlicer(mStartSlicer)->GetAnnotation()->SetVisibility(1);
      // Adjust the window level here
      int *size = isi->GetInteractor()->GetRenderWindow()->GetSize();
      double window = this->InitialWindow;
      double level = this->InitialLevel;

      // Compute normalized delta
      double dx = 4.0 * 
	(isi->GetWindowLevelCurrentPosition()[0] - 
	 isi->GetWindowLevelStartPosition()[0]) / size[0];
      double dy = 4.0 * 
	(isi->GetWindowLevelStartPosition()[1] - 
	 isi->GetWindowLevelCurrentPosition()[1]) / size[1];

      // Scale by current values
      if (fabs(window) > 0.5)
	{
	  dx = dx * window;
	}
      else
	{
	  dx = dx * (window < 0 ? -0.5 : 0.5);
	}
      if (fabs(level) > 0.5)
	{
	  dy = dy * level;
	}
      else
	{
	  dy = dy * (level < 0 ? -0.5 : 0.5);
	}

      // Abs so that direction does not flip
      if (window < 0.0) 
	{
	  dx = -1*dx;
	}
      if (level < 0.0) 
	{
	  dy = -1*dy;
	}

      // Compute new window level
      double newWindow = dx + window;
      double newLevel;
      newLevel = level - dy;

      if (fabs(newWindow) < 0.5)
	{
	  newWindow = 0.5*(newWindow < 0 ? -1 : 1);
	}
      if (fabs(newLevel) < 0.5)
	{
	  newLevel = 0.5*(newLevel < 0 ? -1 : 1);
	}

      this->SM->SetColorWindow(newWindow);
      this->SM->SetColorLevel(newLevel);
      this->SM->SetPreset(6);
      this->SM->UpdateWindowLevel();
      this->SM->Render();
      return;
    }
}

void vvSlicersManagerCallback::Dolly(double factor, vtkRenderWindowInteractor *interactor)
{
  
  vtkRenderer * renderer = interactor->FindPokedRenderer(
							 interactor->GetEventPosition()[0],interactor->GetEventPosition()[1]);
  if (renderer == NULL)
    {
      return;
    }

  double viewFocus[4],viewPoint[4],motionVector[3], focalDepth;
  double oldPos[3], newPos[3], distance[2];
  vtkCamera *camera = renderer->GetActiveCamera();
  camera->GetFocalPoint(viewFocus);

  renderer->SetWorldPoint(viewFocus[0], viewFocus[0], viewFocus[0], 1.0);
  renderer->WorldToDisplay();
  renderer->GetDisplayPoint(viewFocus);

  focalDepth = viewFocus[2];

  oldPos[0] = renderer->GetCenter()[0];
  oldPos[1] = renderer->GetCenter()[1];
  oldPos[2] = focalDepth;

  distance[0] = 1/factor*
    (interactor->GetEventPosition()[0]-renderer->GetCenter()[0]);
  distance[1] = 1/factor*
    (interactor->GetEventPosition()[1]-renderer->GetCenter()[1]);

  newPos[0] = interactor->GetEventPosition()[0] - distance[0];
  newPos[1] = interactor->GetEventPosition()[1] - distance[1];
  newPos[2] = focalDepth;

  renderer->DisplayToNormalizedDisplay(oldPos[0],oldPos[1]);
  renderer->NormalizedDisplayToViewport(oldPos[0],oldPos[1]);
  renderer->ViewportToNormalizedViewport(oldPos[0],oldPos[1]);
  renderer->NormalizedViewportToView(oldPos[0],oldPos[1],oldPos[2]);
  renderer->ViewToWorld(oldPos[0],oldPos[1],oldPos[2]);

  renderer->DisplayToNormalizedDisplay(newPos[0],newPos[1]);
  renderer->NormalizedDisplayToViewport(newPos[0],newPos[1]);
  renderer->ViewportToNormalizedViewport(newPos[0],newPos[1]);
  renderer->NormalizedViewportToView(newPos[0],newPos[1],newPos[2]);
  renderer->ViewToWorld(newPos[0],newPos[1],newPos[2]);

  motionVector[0] = newPos[0] - oldPos[0];
  motionVector[1] = newPos[1] - oldPos[1];
  motionVector[2] = newPos[2] - oldPos[2];

  camera->GetFocalPoint(viewFocus);
  camera->GetPosition(viewPoint);
  camera->SetFocalPoint(motionVector[0] + viewFocus[0],
                        motionVector[1] + viewFocus[1],
                        motionVector[2] + viewFocus[2]);

  camera->SetPosition(motionVector[0] + viewPoint[0],
                      motionVector[1] + viewPoint[1],
                      motionVector[2] + viewPoint[2]);

  if (camera->GetParallelProjection())
    {
      camera->SetParallelScale(camera->GetParallelScale() / factor);
    }
  else
    {
      camera->Dolly(factor);
    }
  
  if (interactor->GetLightFollowCamera()) 
    {
      renderer->UpdateLightsGeometryToFollowCamera();
    }
  renderer->ResetCameraClippingRange();
  //interactor->Render();
}

void vvSlicersManagerCallback::FlyToPosition(vtkRenderWindowInteractor *interactor,vvSlicer* slicer)
{
  //std::cout << "FlyToPosition" << std::endl;
  double flyFrom[3], flyTo[3];
  double d[3], focalPt[3], position[3], positionFrom[3];
  int i, j;
  vtkRenderer * renderer = interactor->FindPokedRenderer(
	interactor->GetEventPosition()[0],interactor->GetEventPosition()[1]);
  if (renderer == NULL)
    {
    return;
    }

  interactor->GetPicker()->Pick(interactor->GetEventPosition()[0],
                             interactor->GetEventPosition()[1], 0.0,
                             renderer);

  vtkAssemblyPath *path=NULL;
  vtkAbstractPropPicker *picker;
  if ( (picker=vtkAbstractPropPicker::SafeDownCast(interactor->GetPicker())))
    {
    path = picker->GetPath();
    }
  if ( path != NULL )
    {
    flyTo[0] = picker->GetPickPosition()[0];
    flyTo[1] = picker->GetPickPosition()[1];
    flyTo[2] = picker->GetPickPosition()[2];
    renderer->GetActiveCamera()->GetFocalPoint(flyFrom);
    renderer->GetActiveCamera()->GetPosition(positionFrom);

    switch (slicer->GetSliceOrientation())
      {
      case vtkImageViewer2::SLICE_ORIENTATION_XY:
      flyTo[2] = flyFrom[2];
      break;

      case vtkImageViewer2::SLICE_ORIENTATION_XZ:
      flyTo[1] = flyFrom[1];
      break;

      case vtkImageViewer2::SLICE_ORIENTATION_YZ:
      flyTo[0] = flyFrom[0];
      break;
      }


    for (i=0; i<3; i++)
      {
      d[i] = flyTo[i] - flyFrom[i];
      }
    double distance = vtkMath::Normalize(d);
    double delta = distance/15;
  
    for (i=1; i<=15; i++)
      {
      for (j=0; j<3; j++)
        {
        focalPt[j] = flyFrom[j] + d[j]*i*delta;
        position[j] = positionFrom[j] + d[j]*i*delta;
        }
      renderer->GetActiveCamera()->SetFocalPoint(focalPt);
      renderer->GetActiveCamera()->SetPosition(position);
      renderer->GetActiveCamera()->Dolly(0.3/15 + 1.0);
      renderer->ResetCameraClippingRange();
      interactor->Render();
      }
    }
}
