/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvVoxImageIO.h,v $
 Language:  C++
 Date:      $Date: 2008/04/10 06:59:37 $
 Version:   $Revision: 1.2 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef VVVOXIMAGEIO_H
#define VVVOXIMAGEIO_H



// vv include
#include "vvIOCommon.h"

// itk include
#include "itkImageIOBase.h"

namespace vv {
  
  //====================================================================
  // Class for reading Vox Image file format
  class VoxImageIO: public itk::ImageIOBase
  {
  public: 
	/** Standard class typedefs. */
	typedef VoxImageIO              Self;
	typedef itk::ImageIOBase        Superclass;
	typedef itk::SmartPointer<Self> Pointer;	
	typedef signed short int        PixelType;
	
	VoxImageIO():Superclass() { mustWriteHeader = false; }

	/** Method for creation through the object factory. */
	itkNewMacro(Self);
	
	/** Run-time type information (and related methods). */
	itkTypeMacro(VoxImageIO, ImageIOBase);
	
	/*-------- This part of the interface deals with reading data. ------ */
	virtual void ReadImageInformation();
	virtual bool CanReadFile( const char* FileNameToRead );
	virtual void Read(void * buffer);

	/*-------- This part of the interfaces deals with writing data. ----- */
	virtual void WriteImageInformation(bool keepOfStream);
	virtual void WriteImageInformation() { WriteImageInformation(false); }
	virtual bool CanWriteFile(const char* filename);
	virtual void Write(const void* buffer);
	
  protected:
	bool mustWriteHeader;
	int m_HeaderSize;
	std::ofstream file;

  }; // end class VoxImageIO  

} // end namespace

  // explicit template instantiation
template class itk::CreateObjectFunction<vv::VoxImageIO>;

#endif /* end #define VVVOXIMAGEIO_H */

