#ifndef VVIMAGERESAMPLEFILTER_H
#define VVIMAGERESAMPLEFILTER_H

/*=========================================================================

Program:   vv
Module:    $RCSfile: vvImageResampleFilter.h,v $
Language:  C++
Date:      $Date: 2008/06/23 14:34:32 $
Version:   $Revision: 1.1 $
Author :   David Sarrut (david.sarrut@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/

// clitk include
#include "vvCommon.h"
#include "vvIOCommon.h"
#include "vvImage.h"
#include "vvQProgressDialogITKCommand.h"
#include "vvSlicersManager.h"

// itk include
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageSeriesReader.h"
#include "itkImageFileWriter.h"
#include "itkRecursiveGaussianImageFilter.h"
#include "itkResampleImageFilter.h"
#include "itkAffineTransform.h"
#include "itkNearestNeighborInterpolateImageFunction.h"
#include "itkLinearInterpolateImageFunction.h"
#include "itkBSplineInterpolateImageFunction.h"
#include "itkBSplineInterpolateImageFunctionWithLUT.h"
#include "itkCommand.h"
#include "itkVTKImageToImageFilter.h"
#include "itkJoinSeriesImageFilter.h"

class vvImageResampleFilter: public itk::Object {
  
public: 
  // constructor
  vvImageResampleFilter();

  // Types
  typedef vvImageResampleFilter Self;
  typedef Object Superclass;
  typedef itk::SmartPointer<Self> Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  // New
  itkNewMacro(Self);

  // Input
  void SetInput(vvImage * input) { mInputvvImage = input; }
  
  // Options
  void SetOutputSize(const std::vector<int> & size);
  void SetOutputSpacing(const std::vector<double> & spacing);
  void SetGaussianSigma(const std::vector<double> & sigma);
  void SetInterpolationName(const std::string & inter);
  void SetDefaultPixelValue(double dpv) { mDefaultPixelValue = dpv;}
  void SetBSplineOrder(int o) { mBSplineOrder = o; }
  void SetBLUTSampling(int b) { mSamplingFactors.resize(1); mSamplingFactors[0] = b; }
  void SetObserver(itk::Command::Pointer o) { mUseAnObserver = true; mObserver = o; }
  void DisableObserver() { mUseAnObserver = false; }

  // Go !
  void Update();

  // Get result
  vvSlicersManager * GetOutputSlicerManager() { return mOutputSlicersManager; }  

protected:
  //====================================================================
  vvImage * mInputvvImage;
  unsigned int mDim;
  std::string mPixelTypeName;
  unsigned int mNbOfComponents;
  std::vector<vtkImageData*> mOutput;
  itk::Command::Pointer mObserver;
  bool mUseAnObserver;  
  vvSlicersManager * mOutputSlicersManager;

  //====================================================================
  std::string mInterpolatorName;
  std::vector<int> mOutputSize;
  std::vector<double> mOutputSpacing;
  std::vector<int> mOutputOrigin;
  double mDefaultPixelValue;
  bool mApplyGaussianFilterBefore;
  std::vector<double> mSigma;
  int mBSplineOrder;
  std::vector<int> mSamplingFactors;

  //====================================================================
  template<unsigned int Dim> void Update_WithDim();
  template<unsigned int Dim, class PixelType> void Update_WithDimAndPixelType();

  //====================================================================
  template<class ImageType>
  typename ImageType::Pointer ComputeImage(const ImageType * inputImage);
  
}; // end class vvImageResampleFilter

#include "vvImageResampleFilter.txx"

#endif /* end #define VVIMAGERESAMPLEFILTER_H */

