/*=========================================================================

Program:   vv
Module:    $RCSfile: vvMainWindow.cxx,v $
Language:  C++
Date:      $Date: 2008/07/16 08:49:41 $
Version:   $Revision: 1.53 $
Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 200COLUMN_IMAGE_NAME
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#include <QtGui>
#include <Qt>
#include "QTreePushButton.h"

#include "vvMainWindow.h"
#include "vvDocumentation.h"
#include "vvProgressDialog.h"
#include "vvQDicomSeriesSelector.h"
#include "vvSlicersManager.h"
#include "vvIOCommon.h"
#include "vvSlicer.h"
#include "vvInteractorStyleNavigator.h"
#include "vvConvertVTKImageToITKDataFilter.h"
#include "vvResamplerDialog.h"

#ifdef CLITK_VV_USE_BDCM
#include <bdcmDicomFilesSelectorDialog.h>
#endif

#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkByteSwapper.h"
#include "itkCommand.h"
#include "itkNumericSeriesFileNames.h"

#include "vtkImageData.h"
#include "vtkImageActor.h"
#include "vtkCornerAnnotation.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkRenderer.h"
#include "vtkRendererCollection.h"

#include "vtkWindowToImageFilter.h"
#include "vtkBMPWriter.h"
#include "vtkTIFFWriter.h"
#include "vtkPNMWriter.h"
#include "vtkPNGWriter.h"
#include "vtkJPEGWriter.h"

// Standard includes
#include <iostream>

#define IMAGE 20
#define DICOM 21
#define MERGED 22
#define IMAGEWITHTIME 23
#define MERGEDWITHTIME 24

#define COLUMN_TREE 0
#define COLUMN_UL_VIEW 1
#define COLUMN_UR_VIEW 2
#define COLUMN_DL_VIEW 3
#define COLUMN_DR_VIEW 4
#define COLUMN_CLOSE_IMAGE 5
#define COLUMN_RELOAD_IMAGE 6
#define COLUMN_IMAGE_NAME 7

/*Data Tree values
  0,32 full filename
  1,10 checkbutton UL View
  1,32 overlay, fusion or vector
  2,10 checkbutton UR View
  3,10 checkbutton DL View
  4,10 checkbutton DR View
  5,0  short filename
  5,32 mSlicersManager id*/

//====================================================================
vvMainWindow::vvMainWindow() {
  setupUi(this); // this sets up the GUI

  mInputPathName = "";

  //Init the contextMenu
  this->setContextMenuPolicy(Qt::CustomContextMenu);
  contextActions.resize(0);
  QAction* actionOpen_new_image = contextMenu.addAction(QIcon(QString::fromUtf8(":/new/prefix1/fileopen.png")),
							tr("Open new Image"));
  connect(actionOpen_new_image,SIGNAL(triggered()),this,SLOT(OpenImages()));
  contextActions.push_back(actionOpen_new_image);

  contextMenu.addSeparator();

  QAction* actionClose_Image = contextMenu.addAction(QIcon(QString::fromUtf8(":/new/prefix1/exit.png")),
						     tr("Close Current Image"));
  connect(actionClose_Image,SIGNAL(triggered()),this,SLOT(CloseImage()));
  contextActions.push_back(actionClose_Image);

  QAction* actionReload_image = contextMenu.addAction(QIcon(QString::fromUtf8(":/new/prefix1/rotateright.png")),
						      tr("Reload Current Image"));
  connect(actionReload_image,SIGNAL(triggered()),this,SLOT(ReloadImage()));
  contextActions.push_back(actionReload_image);

  QAction* actionSave_image = contextMenu.addAction(QIcon(QString::fromUtf8(":/new/prefix1/filesave.png")),
						    tr("Save Current Image"));
  connect(actionSave_image,SIGNAL(triggered()),this,SLOT(SaveAs()));
  contextActions.push_back(actionSave_image);

  contextMenu.addSeparator();

  QAction* actionSplit_image = contextMenu.addAction(QIcon(QString::fromUtf8(":/new/prefix1/cut.png")),
						     tr("Split Current Image"));
  connect(actionSplit_image,SIGNAL(triggered()),this,SLOT(SplitImage()));
  contextActions.push_back(actionSplit_image);

  contextMenu.addSeparator();

  contextMenu.addAction(actionAdd_VF_to_current_Image);
  contextActions.push_back(actionAdd_VF_to_current_Image);

  QAction* actionAdd_Overlay_to_current_Image = menuOverlay->addAction(QIcon(QString::fromUtf8(":/new/prefix1/GPSup.png")),
			tr("Add overlay image to current image"));
  contextMenu.addAction(actionAdd_Overlay_to_current_Image);
  contextActions.push_back(actionAdd_Overlay_to_current_Image);

  connect(actionAdd_Overlay_to_current_Image,SIGNAL(triggered()), this,SLOT(SelectOverlayImage()));

  contextMenu.addAction(actionAdd_fusion_image);
  connect(actionAdd_fusion_image,SIGNAL(triggered()),this,SLOT(AddFusionImage()));
  contextActions.push_back(actionAdd_fusion_image);


  //init the DataTree
  mSlicersManagers.resize(0);

  QStringList header;
  header.append("");
  header.append("TL");
  header.append("TR");
  header.append("BL");
  header.append("BR");
  header.append("");
  header.append("");
  header.append("Name");
  
  DataTree->setHeaderLabels(header);
  DataTree->resizeColumnToContents(COLUMN_TREE);
  DataTree->resizeColumnToContents(COLUMN_UL_VIEW);
  DataTree->resizeColumnToContents(COLUMN_UR_VIEW);
  DataTree->resizeColumnToContents(COLUMN_DL_VIEW);
  DataTree->resizeColumnToContents(COLUMN_DR_VIEW);
  DataTree->resizeColumnToContents(COLUMN_CLOSE_IMAGE);
  DataTree->resizeColumnToContents(COLUMN_RELOAD_IMAGE);
  DataTree->resizeColumnToContents(COLUMN_IMAGE_NAME);

  viewMode = 1;
  documentation = new vvDocumentation();
  dicomSeriesSelector = new vvDicomSeriesSelector();

  inverseButton->setEnabled(0);
  actionAdd_Overlay_to_current_Image->setEnabled(0);
  actionSave_As->setEnabled(0);
  actionAdd_VF_to_current_Image->setEnabled(0);
  actionAdd_fusion_image->setEnabled(0);

  //init the sliders
  verticalSliders.push_back(NOVerticalSlider);
  verticalSliders.push_back(NEVerticalSlider);
  verticalSliders.push_back(SOVerticalSlider);
  verticalSliders.push_back(SEVerticalSlider);

  for(int i =0; i < 4; i++)
    verticalSliders[i]->hide();

  horizontalSliders.push_back(NOHorizontalSlider);
  horizontalSliders.push_back(NEHorizontalSlider);
  horizontalSliders.push_back(SOHorizontalSlider);
  horizontalSliders.push_back(SEHorizontalSlider);

  for(int i =0; i < 4; i++)
    horizontalSliders[i]->hide();


  connect(NOVerticalSlider,SIGNAL(valueChanged(int)),this,SLOT(NOVerticalSliderChanged()));
  connect(NEVerticalSlider,SIGNAL(valueChanged(int)),this,SLOT(NEVerticalSliderChanged()));
  connect(SOVerticalSlider,SIGNAL(valueChanged(int)),this,SLOT(SOVerticalSliderChanged()));
  connect(SEVerticalSlider,SIGNAL(valueChanged(int)),this,SLOT(SEVerticalSliderChanged()));

  connect(NOHorizontalSlider,SIGNAL(valueChanged(int)),this,SLOT(NOHorizontalSliderMoved()));
  connect(NEHorizontalSlider,SIGNAL(valueChanged(int)),this,SLOT(NEHorizontalSliderMoved()));
  connect(SOHorizontalSlider,SIGNAL(valueChanged(int)),this,SLOT(SOHorizontalSliderMoved()));
  connect(SEHorizontalSlider,SIGNAL(valueChanged(int)),this,SLOT(SEHorizontalSliderMoved()));

  //connect everything
  connect(actionLoad_images,SIGNAL(triggered()),this,SLOT(OpenImages()));
  connect(actionOpen_Dicom,SIGNAL(triggered()),this,SLOT(OpenDicom()));
  connect(actionOpen_Multiple_Images_As_One,SIGNAL(triggered()),this,SLOT(MergeImages()));
  connect(actionOpen_Image_With_Time,SIGNAL(triggered()),this,SLOT(OpenImageWithTime()));
  connect(actionMerge_images_as_n_dim_t, SIGNAL(triggered()), this, SLOT(MergeImagesWithTime()));
  connect(actionSave_As,SIGNAL(triggered()),this,SLOT(SaveAs()));
  connect(actionExit,SIGNAL(triggered()),this,SLOT(close()));
  connect(actionAdd_VF_to_current_Image,SIGNAL(triggered()),this,SLOT(AddField()));
  connect(actionDocumentation,SIGNAL(triggered()),this,SLOT(ShowDocumentation()));

  ///////////////////////////////////////////////
  /*contextMenu.addAction(actionResampler);
  connect(actionResampler,SIGNAL(triggered()),this,SLOT(ResampleCurrentImage()));*/
  ///////////////////////////////////////////////

  actionNorth_East_Window->setEnabled(0);
  actionNorth_West_Window->setEnabled(0);
  actionSouth_East_Window->setEnabled(0);
  actionSouth_West_Window->setEnabled(0);

  connect(actionNorth_East_Window,SIGNAL(triggered()),this,SLOT(SaveNEScreenshot()));
  connect(actionNorth_West_Window,SIGNAL(triggered()),this,SLOT(SaveNOScreenshot()));
  connect(actionSouth_East_Window,SIGNAL(triggered()),this,SLOT(SaveSEScreenshot()));
  connect(actionSouth_West_Window,SIGNAL(triggered()),this,SLOT(SaveSOScreenshot()));

  connect(DataTree,SIGNAL(itemSelectionChanged()),this,SLOT(ImageInfoChanged()));
  connect(DataTree,SIGNAL(itemClicked(QTreeWidgetItem*, int)),this,
	  SLOT(DisplayChanged(QTreeWidgetItem*, int)));

  connect(viewButton,SIGNAL(clicked()),this, SLOT(ChangeViewMode()) );
  connect(windowSpinBox,SIGNAL(editingFinished()),this,SLOT(WindowLevelEdited()));
  connect(levelSpinBox,SIGNAL(editingFinished()),this,SLOT(WindowLevelEdited()));
  connect(colorMapComboBox,SIGNAL(currentIndexChanged(int)),this,SLOT(UpdateWindowLevel()));
  connect(presetComboBox,SIGNAL(currentIndexChanged(int)),this,SLOT(UpdateWindowLevel()));
  connect(inverseButton,SIGNAL(clicked()),this,SLOT(SwitchWindowLevel()));


  connect(this,SIGNAL(customContextMenuRequested(QPoint)),this,SLOT(ShowContextMenu(QPoint)));

  connect(linkPanel,SIGNAL(addLink(QString,QString)),this,SLOT(AddLink(QString,QString)));
  connect(linkPanel,SIGNAL(removeLink(QString,QString)),this,SLOT(RemoveLink(QString,QString)));
  connect(overlayPanel,SIGNAL(VFPropertyUpdated(int,int,int)),this,SLOT(SetVFProperty(int,int,int)));
  connect(overlayPanel,SIGNAL(OverlayPropertyUpdated(int)),this,SLOT(SetOverlayProperty(int)));
  connect(overlayPanel,SIGNAL(FusionPropertyUpdated(int,int,double,double)),
	this,SLOT(SetFusionProperty(int,int,double,double)));
  connect(landmarksPanel,SIGNAL(UpdateRenderWindows()),this,SLOT(UpdateRenderWindows()));

  playMode = 0;//pause
  mFrameRate = 10;
  playButton->setEnabled(0);
  frameRateLabel->setEnabled(0);
  frameRateSpinBox->setEnabled(0);
  connect(playButton, SIGNAL(clicked()),this,SLOT(PlayPause()));
  connect(frameRateSpinBox, SIGNAL(valueChanged(int)),this,SLOT(ChangeFrameRate(int)));

  goToCursorPushButton->setEnabled(0);
  connect(goToCursorPushButton, SIGNAL(clicked()),this,SLOT(GoToCursor()));

  NOViewWidget->hide();
  NEViewWidget->hide();
  SOViewWidget->hide();
  SEViewWidget->hide();
}
//====================================================================

//====================================================================
vvMainWindow::~vvMainWindow() {
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(mSlicersManagers[i] != NULL)
	delete mSlicersManagers[i];
    }
}
//====================================================================

//====================================================================
void vvMainWindow::MergeImages() {
  //std::cout << "vvMainWindow::MergeImages()" << std::endl;
  QString Extensions = "All Files (*)";
  Extensions += ";;Images ( *.bmp)";
  Extensions += ";;Images ( *.png)";
  Extensions += ";;Images ( *.jpeg)";
  Extensions += ";;Images ( *.tif)";
  Extensions += ";;Images ( *.mhd)";
  Extensions += ";;Images ( *.hdr)";
  Extensions += ";;Images ( *.vox)";
  QStringList files = QFileDialog::getOpenFileNames(this,tr("Merge Images"),mInputPathName,Extensions);
  if(files.isEmpty())
    return;
  mInputPathName = itksys::SystemTools::GetFilenamePath(files[0].toStdString()).c_str();
  std::vector<std::string> vector;

  unsigned int currentDim = 0;
  std::vector<double> currentSpacing;
  std::vector<int> currentSize;
  std::vector<double> currentOrigin;

  for(int i = 0; i < files.size(); i++)
    {
      itk::ImageIOBase::Pointer reader = itk::ImageIOFactory::CreateImageIO(
									    files[i].toStdString().c_str(), itk::ImageIOFactory::ReadMode);
      reader->SetFileName(files[i].toStdString().c_str());
      reader->ReadImageInformation();
      if(reader)  NOViewWidget->hide();
      NEViewWidget->hide();
      SOViewWidget->hide();
      SEViewWidget->hide();
      {
	if(i == 0)
	  currentDim = reader->GetNumberOfDimensions();
	bool IsOk = true;
	for(unsigned int j = 0;j < currentDim; j++)
	  {
	    if(i == 0)
	      {
		if(j == 0)
		  {
		    currentSpacing.resize(currentDim);
		    currentSize.resize(currentDim);
		    currentOrigin.resize(currentDim);
		  }
		currentOrigin[j] = reader->GetOrigin(j);
		currentSpacing[j] = reader->GetSpacing(j);
		currentSize[j] = reader->GetDimensions(j);
	      }
	    else if (currentDim != reader->GetNumberOfDimensions() 
		     || currentSpacing[j] != reader->GetSpacing(j)
		     || currentSize[j] != (int)reader->GetDimensions(j)
		     || currentOrigin[j] != reader->GetOrigin(j))
	      {
		QString error = "Cannot read file (too different from others ";
		error += files[i].toStdString().c_str();
		QMessageBox::information(this,tr("Reading problem"),error);
		IsOk = false;
		break;
	      }
	  }
	if(IsOk)
	  vector.push_back(files[i].toStdString());
      }
    }
  if(vector.size() > 0)
    LoadImages(vector, MERGED);
}
//====================================================================

//====================================================================
void vvMainWindow::MergeImagesWithTime() {
  //std::cout << "vvMainWindow::MergeImagesWithTime" << std::endl;
  QString Extensions = "All Files (*)";
  Extensions += ";;Images ( *.bmp)";
  Extensions += ";;Images ( *.png)";
  Extensions += ";;Images ( *.jpeg)";
  Extensions += ";;Images ( *.tif)";
  Extensions += ";;Images ( *.mhd)";
  Extensions += ";;Images ( *.hdr)";
  Extensions += ";;Images ( *.vox)";
  QStringList files = QFileDialog::getOpenFileNames(this,tr("Merge Images With Time"),mInputPathName,Extensions);
  if(files.isEmpty())
    return;
  mInputPathName = itksys::SystemTools::GetFilenamePath(files[0].toStdString()).c_str();
  std::vector<std::string> vector;

  unsigned int currentDim = 0;
  std::vector<double> currentSpacing;
  std::vector<int> currentSize;
  std::vector<double> currentOrigin;

  for(int i = 0; i < files.size(); i++)
    {
      itk::ImageIOBase::Pointer reader = itk::ImageIOFactory::CreateImageIO(
									    files[i].toStdString().c_str(), itk::ImageIOFactory::ReadMode);
      reader->SetFileName(files[i].toStdString().c_str());
      reader->ReadImageInformation();
      if(reader)
	{
	  if(i == 0)
	    currentDim = reader->GetNumberOfDimensions();
	  bool IsOk = true;
	  for(unsigned int j = 0;j < currentDim; j++)
	    {
	      if(i == 0)
		{
		  if(j == 0)
		    {
		      currentSpacing.resize(currentDim);
		      currentSize.resize(currentDim);
		      currentOrigin.resize(currentDim);
		    }
		  currentOrigin[j] = reader->GetOrigin(j);
		  currentSpacing[j] = reader->GetSpacing(j);
		  currentSize[j] = reader->GetDimensions(j);
		}
	      else if (currentDim != reader->GetNumberOfDimensions() 
		       || currentSpacing[j] != reader->GetSpacing(j)
		       || currentSize[j] != (int)reader->GetDimensions(j)
		       || currentOrigin[j] != reader->GetOrigin(j))
		{
		  QString error = "Cannot read file (too different from others ";
		  error += files[i].toStdString().c_str();
		  QMessageBox::information(this,tr("Reading problem"),error);
		  IsOk = false;
		  break;
		}
	    }
	  if(IsOk)
	    vector.push_back(files[i].toStdString());
	}
    }
  if(vector.size() > 0)
    LoadImages(vector, MERGEDWITHTIME);
}
//====================================================================

//====================================================================
void vvMainWindow::OpenDicom() {
  //std::cout << "vvMainWindow::OpenDicom()" << std::endl;
  std::vector<std::string> files;

#ifdef CLITK_VV_USE_BDCM
  bool r = bdcm::OpenDicomFilesSelectorDialog(files,
					      "DicomFilesSelectorDialog test",
					      0,0,800,800,1);
  
  if (r) {
    std::cout << "$$$$ main : user clicked 'OK' $$$$"<<std::endl;
    std::cout << "$$$$ selected files : "<<std::endl;
    std::vector<std::string>::iterator i;
    for (i=files.begin();i!=files.end();++i) {
      std::cout << *i << std::endl;
    }
    std::cout << "$$$$ "<<std::endl;
    
    LoadImages(files,DICOM);
  }
  else {
    std::cout << "$$$$ main : user clicked 'CANCEL' $$$$"<<std::endl;
  }
#else
  std::cout << "dicomSeriesSelector " << std::endl;
  if (dicomSeriesSelector->exec() == QDialog::Accepted) {
    files = *(dicomSeriesSelector->GetFilenames());
    LoadImages(files,DICOM);
  }
#endif

}
//====================================================================

//====================================================================
void vvMainWindow::OpenImages() {
  //std::cout << "vvMainWindow::OpenImages()" << std::endl;
  QString Extensions = "All Files (*)";
  Extensions += ";;Images ( *.bmp)";
  Extensions += ";;Images ( *.png)";
  Extensions += ";;Images ( *.jpeg)";
  Extensions += ";;Images ( *.tif)";
  Extensions += ";;Images ( *.mhd)";
  Extensions += ";;Images ( *.hdr)";
  Extensions += ";;Images ( *.vox)";

  QStringList files = QFileDialog::getOpenFileNames(this,tr("Load Images"),mInputPathName,Extensions);
  if(files.isEmpty())
    return;
  mInputPathName = itksys::SystemTools::GetFilenamePath(files[0].toStdString()).c_str();
  std::vector<std::string> vector;
  for(int i = 0; i < files.size(); i++)
    {
      vector.push_back(files[i].toStdString());
    }
  LoadImages(vector, IMAGE);
}
//====================================================================

//====================================================================
void vvMainWindow::OpenImageWithTime() {
  //std::cout << "vvMainWindow::OpenImages()" << std::endl;
  QString Extensions = "All Files (*)";
  Extensions += ";;Images ( *.bmp)";
  Extensions += ";;Images ( *.png)";
  Extensions += ";;Images ( *.jpeg)";
  Extensions += ";;Images ( *.tif)";
  Extensions += ";;Images ( *.mhd)";
  Extensions += ";;Images ( *.hdr)";
  Extensions += ";;Images ( *.vox)";

  QStringList files = QFileDialog::getOpenFileNames(this,tr("Load Images With Time"),mInputPathName,Extensions);
  if(files.isEmpty())
    return;
  mInputPathName = itksys::SystemTools::GetFilenamePath(files[0].toStdString()).c_str();
  std::vector<std::string> vector;
  for(int i = 0; i < files.size(); i++)
    {
      vector.push_back(files[i].toStdString());
    }
  LoadImages(vector, IMAGEWITHTIME);
}
//====================================================================

//====================================================================
void vvMainWindow::LoadImages(std::vector<std::string> files, int filetype) {
  //std::cout << "vvMainWindow::LoadImages(std::vector<std::string> files, int filetype)" << std::endl;
  //Separate the way to open images and dicoms
  int fileSize;
  if(filetype == IMAGE || filetype == IMAGEWITHTIME)
    fileSize = files.size();
  else
    fileSize = 1;

  //init the progress events
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
  vvProgressDialog progress;
  progress.show();
  qApp->processEvents();

  //open images as 1 or multiples
  for(int i = 0; i < fileSize; i++) {
    //std::cout << i << " " << files[i] << std::endl;

    std::string action = "Opening";
    progress.start(action,files[i]);
    qApp->processEvents();

    itk::ImageIOBase::Pointer reader = itk::ImageIOFactory::CreateImageIO(files[i].c_str(), itk::ImageIOFactory::ReadMode);

    if (!reader) {  
      QApplication::restoreOverrideCursor();
      QString error = "Cannot read file ";
      error += files[i].c_str();
      QMessageBox::information(this,tr("Reading problem"),error);
    }
    
    else {
      //read the image and put it in mSlicersManagers
      //std::cout << files[i] << std::endl;
      reader->SetFileName(files[i]);
      reader->ReadImageInformation();
      std::string component = reader->GetComponentTypeAsString(reader->GetComponentType());
      vvSlicersManager* imageManager = new vvSlicersManager(4);
      qApp->processEvents();
      
      std::string filename = "";
      
      bool SetImageSucceed=false;
      
      //Separate the way to open images and dicoms
      if(filetype == IMAGE)
	SetImageSucceed = imageManager->SetImage(files[i],reader->GetNumberOfDimensions(), component,IMAGE);
      else if(filetype == IMAGEWITHTIME)
	SetImageSucceed = imageManager->SetImage(files[i],reader->GetNumberOfDimensions(), component,IMAGEWITHTIME);
      else if(filetype == DICOM)
	{
	  //std::cout << "dicom" << std::endl;
	  filename += "dicom_";
	  SetImageSucceed = imageManager->SetImages(files,reader->GetNumberOfDimensions(), component, DICOM);
	}
      else if(filetype == MERGED)
	{
	  filename += "merged_";
	  SetImageSucceed = imageManager->SetImages(files,reader->GetNumberOfDimensions(), component, MERGED);
	}
      else if(filetype == MERGEDWITHTIME)
	{
	  filename += "merged_";
	  SetImageSucceed = imageManager->SetImages(files,reader->GetNumberOfDimensions(), component, MERGEDWITHTIME);
	}
      filename += itksys::SystemTools::GetFilenameWithoutExtension(files[i]).c_str();
      if(SetImageSucceed == false)
	{
	  QApplication::restoreOverrideCursor();
	  QString error = "Cannot open file \n";
	  error += imageManager->GetLastError().c_str();
	  QMessageBox::information(this,tr("Reading problem"),error);
	  delete imageManager;
	}
      else
	{
	  mSlicersManagers.push_back(imageManager);

	  //create an item in the tree with good settings
	  QTreeWidgetItem *item = new QTreeWidgetItem();
	  item->setData(0,32,files[i].c_str());
	  item->setData(COLUMN_IMAGE_NAME,0,filename.c_str());
	  qApp->processEvents();

	  for(int j = 1; j <= 4; j++)
	    {
	      item->setData(j,10,1);
	    }

	  //Create the buttons for reload and close
	  qApp->processEvents();
	  QTreePushButton* cButton = new QTreePushButton;
	  cButton->setItem(item);
	  cButton->setColumn(COLUMN_CLOSE_IMAGE);
	  cButton->setToolTip(tr("close image"));
	  cButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/exit.png")));
	  connect(cButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
		  this,SLOT(CloseImage(QTreeWidgetItem*, int)));

	  QTreePushButton* rButton = new QTreePushButton;
	  rButton->setItem(item);
	  rButton->setColumn(COLUMN_RELOAD_IMAGE);
	  rButton->setToolTip(tr("reload image"));
	  rButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/rotateright.png")));
	  connect(rButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
		  this,SLOT(ReloadImage(QTreeWidgetItem*, int)));

	  DataTree->addTopLevelItem(item);
	  DataTree->setItemWidget(item, COLUMN_CLOSE_IMAGE, cButton);
	  DataTree->setItemWidget(item, COLUMN_RELOAD_IMAGE, rButton);

	  //set the id of the image 
	  QString id = files[i].c_str() + QString::number(mSlicersManagers.size()-1);
	  item->setData(COLUMN_IMAGE_NAME,32,id.toStdString().c_str());
	  mSlicersManagers.back()->SetId(id.toStdString());

	  linkPanel->addImage(filename, id.toStdString());

	  connect(mSlicersManagers.back(),SIGNAL(currentImageChanged(std::string)),
		  this,SLOT(CurrentImageChanged(std::string)));
	  connect(mSlicersManagers.back(),SIGNAL(
						 UpdatePosition(int, double, double, double, double, double, double, double)),this,
		  SLOT(MousePositionChanged(int,double, double, double, double, double, double, double)));
	  connect(mSlicersManagers.back(),SIGNAL(UpdateVector(int, double, double, double, double)),
		  this, SLOT(VectorChanged(int,double,double,double, double)));
	  connect(mSlicersManagers.back(),SIGNAL(UpdateOverlay(int, double, double)),
		  this, SLOT(OverlayChanged(int,double,double)));
	  connect(mSlicersManagers.back(),SIGNAL(UpdateFusion(int, double)),
		  this, SLOT(FusionChanged(int,double)));
	  connect(mSlicersManagers.back(),SIGNAL(UpdateWindows(int, int, int)),
		  this,SLOT(WindowsChanged(int, int, int)));
	  connect(mSlicersManagers.back(),SIGNAL(WindowLevelChanged(double, double,int, int)),
		  this,SLOT(WindowLevelChanged(double, double, int, int)));
	  connect(mSlicersManagers.back(),SIGNAL(UpdateSlice(int,int)),
		  this,SLOT(UpdateSlice(int,int)));
	  connect(mSlicersManagers.back(),SIGNAL(UpdateTSlice(int, int)),
		  this,SLOT(UpdateTSlice(int, int)));
	  connect(mSlicersManagers.back(),SIGNAL(UpdateSliceRange(int,int,int,int,int)),
		  this,SLOT(UpdateSliceRange(int,int,int,int,int)));
	  connect(mSlicersManagers.back(),SIGNAL(UpdateLinkManager(std::string,int,double,double,double,int)),
		  this,SLOT(UpdateLinkManager(std::string,int,double,double,double,int)));
	  connect(mSlicersManagers.back(),SIGNAL(LandmarkAdded()),landmarksPanel,SLOT(AddPoint()));
	  UpdateTree();
	  qApp->processEvents();
	  NOViewWidget->show();
	  NEViewWidget->show();
	  SOViewWidget->show();
	  SEViewWidget->show();

	  InitDisplay();
	  qApp->processEvents();
	}
    }
  }
  progress.reject();
  QApplication::restoreOverrideCursor();
}
//====================================================================

//====================================================================
void vvMainWindow::UpdateTree() {
  //std::cout << "vvMainWindow::UpdateTree()" << std::endl;
  DataTree->resizeColumnToContents(COLUMN_TREE);
  DataTree->resizeColumnToContents(COLUMN_UL_VIEW);
  DataTree->resizeColumnToContents(COLUMN_UR_VIEW);
  DataTree->resizeColumnToContents(COLUMN_DL_VIEW);
  DataTree->resizeColumnToContents(COLUMN_DR_VIEW);
  DataTree->resizeColumnToContents(COLUMN_IMAGE_NAME);
  DataTree->resizeColumnToContents(COLUMN_CLOSE_IMAGE);
  DataTree->resizeColumnToContents(COLUMN_RELOAD_IMAGE);
}
//====================================================================

//====================================================================
void vvMainWindow::CurrentImageChanged(std::string id) {
  //std::cout << "vvMainWindow::CurrentImageChanged()" << std::endl;
  int selected = 0;
  for(int i = 0; i < DataTree->topLevelItemCount(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_IMAGE_NAME,32).toString().toStdString() == id)
	{
	  selected = i;
	}
      else
	{
	  DataTree->topLevelItem(i)->setSelected(0);
	}
      for(int child = 0; child < DataTree->topLevelItem(i)->childCount();child++)
	DataTree->topLevelItem(i)->child(child)->setSelected(0);

    }
  DataTree->topLevelItem(selected)->setSelected(1);
}
//====================================================================

//====================================================================
void vvMainWindow::ImageInfoChanged() {
  //std::cout << "vvMainWindow::ImageInfoChanged()" << std::endl;
  contextActions[7]->setEnabled(1);
  contextActions[6]->setEnabled(1);
  actionSave_As->setEnabled(1);
  actionAdd_VF_to_current_Image->setEnabled(1);
  actionAdd_fusion_image->setEnabled(1);
  actionNorth_East_Window->setEnabled(1);
  actionNorth_West_Window->setEnabled(1);
  actionSouth_East_Window->setEnabled(1);
  actionSouth_West_Window->setEnabled(1);
  inverseButton->setEnabled(1);

  goToCursorPushButton->setEnabled(1);

  if(DataTree->selectedItems().size()) {
    int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);

    colorMapComboBox->setEnabled(1);
    for(int i = 0; i < DataTree->topLevelItem(index)->childCount();i++)
      {
      if(DataTree->topLevelItem(index)->child(i)->data(1,32).toString() == "overlay" ||
	DataTree->topLevelItem(index)->child(i)->data(1,32).toString() == "fusion")
        {
        colorMapComboBox->setEnabled(0);
        break;
        }
      }

    std::vector<double> origin;
    std::vector<double> inputSpacing;
    std::vector<int> inputSize;
    std::vector<double> sizeMM;
    int dimension=0;
    QString pixelType;
    QString inputSizeInBytes;
    QString image = DataTree->selectedItems()[0]->data(COLUMN_IMAGE_NAME,0).toString();

    if(mSlicersManagers[index]->GetSlicer(0)->GetImage()->GetImage().size() > 1 || playMode == 1)
      {
      playButton->setEnabled(1);
      frameRateLabel->setEnabled(1);
      frameRateSpinBox->setEnabled(1);
      }
    else
      {
      playButton->setEnabled(0);
      frameRateLabel->setEnabled(0);
      frameRateSpinBox->setEnabled(0);
      }

    //read image header
    int NPixel = 1;

    if(DataTree->topLevelItem(index) == DataTree->selectedItems()[0])
      {
	vvImage* imageSelected = mSlicersManagers[index]->GetSlicer(0)->GetImage();
	dimension = imageSelected->GetNumberOfDimensions();
	origin.resize(dimension);
	inputSpacing.resize(dimension);
	inputSize.resize(dimension);
	sizeMM.resize(dimension);
	pixelType = mSlicersManagers[index]->GetComponent().c_str();
	for(int i = 0;i < dimension;i++)
	  {
	    origin[i] = imageSelected->GetOrigin()[i];
	    inputSpacing[i] = imageSelected->GetSpacing()[i];
	    inputSize[i] = imageSelected->GetSize()[i];
	    sizeMM[i] = inputSize[i]*inputSpacing[i];
	    NPixel *= inputSize[i];
	  }
	inputSizeInBytes = GetSizeInBytes(imageSelected->GetActualMemorySize()*1000);
      }
    else if(DataTree->selectedItems()[0]->data(1,32).toString() == "vector")
      {
	vvVF* imageSelected = mSlicersManagers[index]->GetSlicer(0)->GetVF();
	dimension = imageSelected->GetNumberOfDimensions();
	origin.resize(dimension);
	inputSpacing.resize(dimension);
	inputSize.resize(dimension);
	sizeMM.resize(dimension);
	pixelType = mSlicersManagers[index]->GetVFComponent().c_str();
	for(int i = 0;i < dimension;i++)
	  {
	    origin[i] = imageSelected->GetOrigin()[i];
	    inputSpacing[i] = imageSelected->GetSpacing()[i];
	    inputSize[i] = imageSelected->GetSize()[i];
	    sizeMM[i] = inputSize[i]*inputSpacing[i];
	    NPixel *= inputSize[i];
	  }
	inputSizeInBytes = GetSizeInBytes(imageSelected->GetActualMemorySize()*1000);
      }
    else if(DataTree->selectedItems()[0]->data(1,32).toString() == "overlay")
      {
	vvImage* imageSelected = mSlicersManagers[index]->GetSlicer(0)->GetOverlay();
	dimension = imageSelected->GetNumberOfDimensions();
	origin.resize(dimension);
	inputSpacing.resize(dimension);
	inputSize.resize(dimension);
	sizeMM.resize(dimension);
	pixelType = mSlicersManagers[index]->GetComponent().c_str();
	for(int i = 0;i < dimension;i++)
	  {
	    origin[i] = imageSelected->GetOrigin()[i];
	    inputSpacing[i] = imageSelected->GetSpacing()[i];
	    inputSize[i] = imageSelected->GetSize()[i];
	    sizeMM[i] = inputSize[i]*inputSpacing[i];
	    NPixel *= inputSize[i];
	  }
	inputSizeInBytes = GetSizeInBytes(imageSelected->GetActualMemorySize()*1000);
      }
    else if(DataTree->selectedItems()[0]->data(1,32).toString() == "fusion")
      {
	vvImage* imageSelected = mSlicersManagers[index]->GetSlicer(0)->GetFusion();
	dimension = imageSelected->GetNumberOfDimensions();
	origin.resize(dimension);
	inputSpacing.resize(dimension);
	inputSize.resize(dimension);
	sizeMM.resize(dimension);
	pixelType = mSlicersManagers[index]->GetComponent().c_str();
	for(int i = 0;i < dimension;i++)
	  {
	    origin[i] = imageSelected->GetOrigin()[i];
	    inputSpacing[i] = imageSelected->GetSpacing()[i];
	    inputSize[i] = imageSelected->GetSize()[i];
	    sizeMM[i] = inputSize[i]*inputSpacing[i];
	    NPixel *= inputSize[i];
	  }
	inputSizeInBytes = GetSizeInBytes(imageSelected->GetActualMemorySize()*1000);
      }

    QString dim = QString::number(dimension) + " (";
    dim += pixelType + ")";

    infoPanel->setFileName(image);
    infoPanel->setDimension(dim);
    infoPanel->setSizePixel(GetVectorIntAsString(inputSize));
    infoPanel->setSizeMM(GetVectorDoubleAsString(sizeMM));
    infoPanel->setSpacing(GetVectorDoubleAsString(inputSpacing));
    infoPanel->setNPixel(QString::number(NPixel));
    infoPanel->setMemory(inputSizeInBytes);

    landmarksPanel->SetCurrentLandmarks(mSlicersManagers[index]->GetLandmarks(),
	mSlicersManagers[index]->GetSlicer(0)->GetImage()->GetImage().size());
    landmarksPanel->SetCurrentPath(mInputPathName.toStdString());
    landmarksPanel->SetCurrentImage(mSlicersManagers[index]->GetFileName().c_str());

    overlayPanel->getCurrentImageName(mSlicersManagers[index]->GetFileName().c_str());
    for(int i = 0; i < 4;i++)
      {
	if(DataTree->selectedItems()[0]->data(i+1,10).toInt() > 0 || i == 3)
	  {
	    mSlicersManagers[index]->UpdateInfoOnCursorPosition(i);
	    break;
	  }
      }
    windowSpinBox->setValue(mSlicersManagers[index]->GetColorWindow());
    levelSpinBox->setValue(mSlicersManagers[index]->GetColorLevel());
    colorMapComboBox->setCurrentIndex(mSlicersManagers[index]->GetColorMap());
    presetComboBox->setCurrentIndex(mSlicersManagers[index]->GetPreset());

    if(mSlicersManagers[index]->GetSlicer(0)->GetVF())
      {
	overlayPanel->getVFName(mSlicersManagers[index]->GetVFName().c_str());
	overlayPanel->getVFProperty(mSlicersManagers[index]->GetSlicer(0)->GetVFSubSampling(),
				    mSlicersManagers[index]->GetSlicer(0)->GetVFScale(),
				    mSlicersManagers[index]->GetSlicer(0)->GetVFLog());
      }
    else 
      {
	overlayPanel->getVFName(mSlicersManagers[index]->GetVFName().c_str());
	overlayPanel->getVFProperty(-1,-1,-1);
      }
    if(mSlicersManagers[index]->GetSlicer(0)->GetOverlay())
      {
	overlayPanel->getOverlayName(mSlicersManagers[index]->GetOverlayName().c_str());
	overlayPanel->getOverlayProperty(mSlicersManagers[index]->GetOverlayColor());
      }
    else 
      {
	overlayPanel->getOverlayName(mSlicersManagers[index]->GetOverlayName().c_str());
	overlayPanel->getOverlayProperty(-1);
      }
    if(mSlicersManagers[index]->GetSlicer(0)->GetFusion())
      {
	overlayPanel->getFusionName(mSlicersManagers[index]->GetFusionName().c_str());
	overlayPanel->getFusionProperty(mSlicersManagers[index]->GetFusionOpacity(),
					mSlicersManagers[index]->GetFusionColorMap(),
					mSlicersManagers[index]->GetFusionWindow(),
					mSlicersManagers[index]->GetFusionLevel());
      }
    else 
      {
	overlayPanel->getFusionName(mSlicersManagers[index]->GetFusionName().c_str());
	overlayPanel->getFusionProperty(-1, -1,-1,-1);
      }
  }
}
//====================================================================

//====================================================================
void vvMainWindow::ShowDocumentation() {
  //std::cout << "vvMainWindow::ShowDocumentation()" << std::endl;
  documentation->show();
}
//====================================================================

//====================================================================
void vvMainWindow::ChangeViewMode() {
  //std::cout << "vvMainWindow::ChangeViewMode()" << std::endl;
  QListIterator<int> it0(splitter_4->sizes());
  QListIterator<int> it1(splitter_4->sizes());
  int max0 = 0;
  int max1 = 1;
  while (it0.hasNext())
    {
      max0 += it0.next();
    }
  while (it1.hasNext())
    {
      max1 += it1.next();
    }
  QList<int> size0;
  QList<int> size1;
  if(viewMode == 1)
    {
      viewMode = 0;
      size0.push_back(max0);
      size0.push_back(0);
      size1.push_back(max1);
      size1.push_back(0);
      splitter_4->setSizes(size0);
      OSplitter->setSizes(size1);
      DataTree->setColumnHidden(2,1);
      DataTree->setColumnHidden(3,1);
      DataTree->setColumnHidden(4,1);
    }
  else
    {
      viewMode = 1;
      size0.push_back(int(max0/2));
      size0.push_back(int(max0/2));
      size1.push_back(int(max1/2));
      size1.push_back(int(max1/2));
      splitter_4->setSizes(size0);
      OSplitter->setSizes(size1);
      DataTree->setColumnHidden(2,0);
      DataTree->setColumnHidden(3,0);
      DataTree->setColumnHidden(4,0);
    }
}
//====================================================================

//====================================================================
QString vvMainWindow::GetSizeInBytes(unsigned long size) {
  //std::cout << "vvMainWindow::GetSizeInBytes(unsigned long size)" << std::endl;
  QString result = "";// QString::number(size);
  //result += " bytes (";
  if(size > 1000000000)
    {
      size /= 1000000000;
      result += QString::number(size);
      result += "Gb";//)";
    }
  else if(size > 1000000)
    {
      size /= 1000000;
      result += QString::number(size);
      result += "Mb";//)";
    }
  else if(size > 1000)
    {
      size /= 1000;
      result += QString::number(size);
      result += "kb";//)";
    }
  return result;
}
//====================================================================

//====================================================================
QString vvMainWindow::GetVectorDoubleAsString(std::vector<double> vectorDouble) {
  //std::cout << "vvMainWindow::GetVectorDoubleAsString(std::vector<double> vectorDouble)" << std::endl;
  QString result;
  for(unsigned int i= 0; i < vectorDouble.size(); i++)
    {
      if(i != 0)
	result += " ";
      result += QString::number(vectorDouble[i]);
    }
  return result;
}
//====================================================================

//====================================================================
QString vvMainWindow::GetVectorIntAsString(std::vector<int> vectorInt) {
  //std::cout << "vvMainWindow::GetVectorIntAsString(std::vector<int> vectorInt)" << std::endl;
  QString result;
  for(unsigned int i= 0; i < vectorInt.size(); i++)
    {
      if(i != 0)
	result += " ";
      result += QString::number(vectorInt[i]);
    }
  return result;
}
//====================================================================

//====================================================================
int vvMainWindow::GetSlicerIndexFromItem(QTreeWidgetItem* item) {
  //std::cout << "vvMainWindow::GetSlicerIndexFromItem(QTreeWidgetItem* item)" << std::endl;
  QString id = item->data(COLUMN_IMAGE_NAME,32).toString();
  for(int i = 0; i < DataTree->topLevelItemCount(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_IMAGE_NAME,32).toString() == id)
	return i;
    }
  return -1;
}
//====================================================================

//====================================================================
void vvMainWindow::DisplayChanged(QTreeWidgetItem *item, int column) {
  //std::cout << "vvMainWindow::DisplayChanged(" << item->data(COLUMN_IMAGE_NAME,0).toString().toStdString() << "," << column << ")" << std::endl;
  int index = GetSlicerIndexFromItem(item);
  if( column < COLUMN_CLOSE_IMAGE && column > 0)
    {
      for (unsigned int i = 0; i < mSlicersManagers.size(); i++)
	{
	  //if we clicked on the mSlicersManagers[i]
	  if(item->data(COLUMN_IMAGE_NAME,32).toString().toStdString() == mSlicersManagers[i]->GetId())
	    {
	      //if we clicked on the vector(or overlay) and not the image
	      if(DataTree->topLevelItem(index) != DataTree->selectedItems()[0])
		{
		  if(DataTree->topLevelItem(index)->data(column,10).toInt() > 0)
		    {
		      if(DataTree->selectedItems()[0]->data(1,32).toString() == "vector" &&
			 mSlicersManagers[i]->GetSlicer(column-1)->GetVF())
			{
			  mSlicersManagers[i]->SetColorMap(-1);
			  mSlicersManagers[i]->GetSlicer(column-1)->SetVFVisibility(
										    DataTree->selectedItems()[0]->data(column,10).toInt());
			}
		      if(DataTree->selectedItems()[0]->data(1,32).toString() == "overlay" && 
			 mSlicersManagers[i]->GetSlicer(column-1)->GetOverlay())
			{
			  mSlicersManagers[i]->GetSlicer(column-1)->SetOverlayVisibility(
											 DataTree->selectedItems()[0]->data(column,10).toInt());
			  mSlicersManagers[i]->SetColorMap(0);
			  mSlicersManagers[i]->GetSlicer(column-1)->Render();
			}
		      if(DataTree->selectedItems()[0]->data(1,32).toString() == "fusion" && 
			 mSlicersManagers[i]->GetSlicer(column-1)->GetFusion())
			{
			  mSlicersManagers[i]->GetSlicer(column-1)->SetFusionVisibility(
											DataTree->selectedItems()[0]->data(column,10).toInt());
			  mSlicersManagers[i]->SetColorMap(0);
			  mSlicersManagers[i]->GetSlicer(column-1)->Render();
			}
		    }
		  else
		    {
		      DataTree->selectedItems()[0]->setData(column,10,0);
		      if(DataTree->selectedItems()[0]->data(1,32).toString() == "vector" &&
			 mSlicersManagers[i]->GetSlicer(column-1)->GetVF())
			{
			  mSlicersManagers[i]->SetColorMap(-1);
			  mSlicersManagers[i]->GetSlicer(column-1)->SetVFVisibility(0);
			}
		      if(DataTree->selectedItems()[0]->data(1,32).toString() == "overlay" && 
			 mSlicersManagers[i]->GetSlicer(column-1)->GetOverlay())
			{
			  mSlicersManagers[i]->GetSlicer(column-1)->SetOverlayVisibility(0);
			  mSlicersManagers[i]->SetColorMap(0);
			  mSlicersManagers[i]->GetSlicer(column-1)->Render();
			}
		      if(DataTree->selectedItems()[0]->data(1,32).toString() == "fusion" && 
			 mSlicersManagers[i]->GetSlicer(column-1)->GetFusion())
			{
			  mSlicersManagers[i]->GetSlicer(column-1)->SetFusionVisibility(0);
			  mSlicersManagers[i]->SetColorMap(0);
			  mSlicersManagers[i]->GetSlicer(column-1)->Render();
			}
		    }
		  continue;
		}
              mSlicersManagers[i]->SetColorMap(-1);
	      mSlicersManagers[i]->UpdateSlicer(column-1,item->data(column,10).toInt());
	      mSlicersManagers[i]->UpdateInfoOnCursorPosition(column-1);
	      DisplaySliders(i,column-1);
	    }
	  else if (item->data(column,10).toInt() > 0)
	    {
	      DataTree->topLevelItem(i)->setData(column,10,0);
              mSlicersManagers[i]->SetColorMap(-1);
	      mSlicersManagers[i]->UpdateSlicer(column-1,0);
	    }
	  for(int child = 0; child < DataTree->topLevelItem(i)->childCount(); child++)
	    DataTree->topLevelItem(i)->child(child)->setData(column,10,
							     DataTree->topLevelItem(i)->data(column,10));
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::InitDisplay() {
  ////std::cout << "vvMainWindow::InitDisplay()" << std::endl;
  if(mSlicersManagers.size())
    {
      mSlicersManagers.back()->SetSlicerWindow(0,NOViewWidget->GetRenderWindow());
      mSlicersManagers.back()->SetSlicerWindow(1,NEViewWidget->GetRenderWindow());
      mSlicersManagers.back()->SetSlicerWindow(2,SOViewWidget->GetRenderWindow());
      mSlicersManagers.back()->SetSlicerWindow(3,SEViewWidget->GetRenderWindow());

      mSlicersManagers.back()->GenerateDefaultLookupTable();

      //BE CAREFULL : this is absolutely necessary to set the interactor style 
      //in order to have the same style instanciation for all SlicersManagers in 
      // a same window

      for(int j = 0; j < 4; j++)
	{
	  vvInteractorStyleNavigator* style = vvInteractorStyleNavigator::New();
	  style->SetAutoAdjustCameraClippingRange(1);
	  bool AlreadySelected = false;
	  for(int i = 0; i < DataTree->topLevelItemCount(); i++)
	    {
	      mSlicersManagers[i]->SetInteractorStyleNavigator(j,style);

	      //select the image only if previous are not selected
	      if(DataTree->topLevelItem(i)->data(j+1,10).toInt() > 1)
		{
		  mSlicersManagers[i]->UpdateSlicer(j,1);
		  AlreadySelected = true;
		}
	      else if (DataTree->topLevelItem(i)->data(j+1,10).toInt() < 1)
		{
		  mSlicersManagers[i]->UpdateSlicer(j,0);}
	      else if (DataTree->topLevelItem(i)->data(j+1,10).toInt() == 1 && !AlreadySelected)
		{
		  if(DataTree->selectedItems().size() == 0)
		    DataTree->topLevelItem(i)->setSelected(1);
		  DataTree->topLevelItem(i)->setData(j+1,10,2);
		  mSlicersManagers[i]->UpdateSlicer(j,1);
		  DisplaySliders(i,j);
		}
	      else
		{
		  DataTree->topLevelItem(i)->setData(j+1,10,0);
		  mSlicersManagers[i]->UpdateSlicer(j,0);
		}
	    }
	  style->Delete();
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::DisplaySliders(int slicer, int window) {
  int range[2];
  mSlicersManagers[slicer]->GetSlicer(window)->GetSliceRange(range);
  int position = mSlicersManagers[slicer]->GetSlicer(window)->GetSlice();

  int tRange[2];
  tRange[0] = 0;
  tRange[1] = mSlicersManagers[slicer]->GetSlicer(window)->GetTMax();
  int tPosition = mSlicersManagers[slicer]->GetSlicer(window)->GetTSlice();
  bool showHorizontal = false;
  bool showVertical = false;
  if(mSlicersManagers[slicer]->GetSlicer(window)->GetImage()->GetNumberOfDimensions() > 3
     || (mSlicersManagers[slicer]->GetSlicer(window)->GetImage()->GetNumberOfDimensions() > 2 
	 && mSlicersManagers[slicer]->GetType() != IMAGEWITHTIME 
	 && mSlicersManagers[slicer]->GetType() != MERGEDWITHTIME))
    showVertical = true;
  if(mSlicersManagers[slicer]->GetSlicer(window)->GetImage()->GetNumberOfDimensions() > 3 
     || mSlicersManagers[slicer]->GetType() == IMAGEWITHTIME 
     || mSlicersManagers[slicer]->GetType() == MERGEDWITHTIME)
    showHorizontal = true;

  if(showVertical)
    verticalSliders[window]->show();
  else
    verticalSliders[window]->hide();
  verticalSliders[window]->setRange(range[0],range[1]);
  verticalSliders[window]->setValue(position);

  if(showHorizontal)
    horizontalSliders[window]->show();
  else
    horizontalSliders[window]->hide();
  horizontalSliders[window]->setRange(tRange[0],tRange[1]);
  horizontalSliders[window]->setValue(tPosition);
}
//====================================================================

//====================================================================
void vvMainWindow::CloseImage(QTreeWidgetItem* item, int column) {
  int index = GetSlicerIndexFromItem(item);
  //std::cout << "vvMainWindow::CloseImage("<< index << "," << column << ")" << std::endl;

  if(DataTree->topLevelItem(index) != item)
    {
      QString warning = "Do you really want to close the overlay : ";
      warning += item->data(COLUMN_IMAGE_NAME,0).toString();
      QMessageBox msgBox(QMessageBox::Warning, tr("Close Overlay"),
			 warning, 0, this);
      msgBox.addButton(tr("Close"), QMessageBox::AcceptRole);
      msgBox.addButton(tr("Cancel"), QMessageBox::RejectRole);
      if (msgBox.exec() == QMessageBox::AcceptRole)
	{
	  if(item->data(1,32).toString() == "overlay")
	    mSlicersManagers[index]->RemoveOverlay();
	  else if(item->data(1,32).toString() == "fusion")
	    mSlicersManagers[index]->RemoveFusion();
	  else
	    mSlicersManagers[index]->RemoveVF();
	  DataTree->topLevelItem(index)->takeChild(DataTree->topLevelItem(index)->indexOfChild(item));
	}
    }
  else if(DataTree->topLevelItemCount() <= 1)
    {
      QString warning = "Do you really want to close the image : ";
      warning += item->data(COLUMN_IMAGE_NAME,0).toString();
      warning += "\nThis is the last image, you're about to close vv2 !!!";
      QMessageBox msgBox(QMessageBox::Warning, tr("Close Image"),
			 warning, 0, this);
      msgBox.addButton(tr("Close vv2"), QMessageBox::AcceptRole);
      msgBox.addButton(tr("Cancel"), QMessageBox::RejectRole);
      if (msgBox.exec() == QMessageBox::AcceptRole)
	{
	  this->close();
	}
    }
  else
    {
      QString warning = "Do you really want to close the image : ";
      warning += item->data(COLUMN_IMAGE_NAME,0).toString();
      QMessageBox msgBox(QMessageBox::Warning, tr("Close Image"),
			 warning, 0, this);
      msgBox.addButton(tr("Close"), QMessageBox::AcceptRole);
      msgBox.addButton(tr("Cancel"), QMessageBox::RejectRole);
      if (msgBox.exec() == QMessageBox::AcceptRole)
	{
	  std::vector<vvSlicersManager*>::iterator Manageriter = mSlicersManagers.begin();
	  DataTree->takeTopLevelItem(index);
	  for(int i = 0; i < index; i++)
	    {
	      Manageriter++;
	    }
	  linkPanel->removeImage(index);
	  mSlicersManagers[index]->RemoveActors();
	  delete mSlicersManagers[index];
	  mSlicersManagers.erase(Manageriter);
	  InitDisplay();
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::ReloadImage(QTreeWidgetItem* item, int column) {
  int index = GetSlicerIndexFromItem(item);
  //std::cout << "vvMainWindow::ReloadImage("<< index << "," << column << ")" << std::endl;
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
  if(DataTree->topLevelItem(index) != item)
    mSlicersManagers[index]->ReloadVF();
  else
    mSlicersManagers[index]->Reload();
  QApplication::restoreOverrideCursor();
}
//====================================================================

//====================================================================
void vvMainWindow::SplitImage() {
  //std::cout << "vvMainWindow::SplitImage()" << std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
  int dim = mSlicersManagers[index]->GetDimension();
  QString warning = "Do you really want to split the "; 
  warning += QString::number(dim) + "D image ";
  warning += DataTree->selectedItems()[0]->data(COLUMN_IMAGE_NAME,0).toString() + " into ";
  warning += QString::number(mSlicersManagers[index]->GetSlicer(0)->GetImage()->GetSize()[dim-1]) + " ";
  warning += QString::number(dim-1) + "D images.";
  QMessageBox msgBox(QMessageBox::Warning, tr("Split Image"),
		     warning, 0, this);
  msgBox.addButton(tr("Split"), QMessageBox::AcceptRole);
  msgBox.addButton(tr("Cancel"), QMessageBox::RejectRole);
  if (msgBox.exec() == QMessageBox::AcceptRole)
    {
      QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
      if(dim > 2)
	{
	  std::string filename = DataTree->selectedItems()[0]->data(0,32).toString().toStdString();

	  itk::ImageIOBase::Pointer reader = itk::ImageIOFactory::CreateImageIO(
										filename.c_str(), itk::ImageIOFactory::ReadMode);
	  reader->SetFileName(filename);
	  reader->ReadImageInformation();
	  std::string component = reader->GetComponentTypeAsString(reader->GetComponentType());

	  int dim = mSlicersManagers[index]->GetDimension();
	  int numberOfSlice = mSlicersManagers[index]->GetSlicer(0)->GetImage()->GetSize()[dim-1];

	  std::string path = itksys::SystemTools::GetFilenamePath(
								  filename);
	  path += "/";
	  path += DataTree->selectedItems()[0]->data(COLUMN_IMAGE_NAME,0).toString().toStdString();
	  path += "%03d";
	  path += itksys::SystemTools::GetFilenameLastExtension(
								filename).c_str();

	  typedef itk::NumericSeriesFileNames NameGeneratorType;
	  NameGeneratorType::Pointer nameGenerator = NameGeneratorType::New();
	  nameGenerator->SetSeriesFormat(path.c_str());
	  nameGenerator->SetStartIndex(0);
	  nameGenerator->SetEndIndex(numberOfSlice-1);
	  nameGenerator->SetIncrementIndex(1);
  
	  for(int i = 0; i < numberOfSlice; i++)
	    {
	      vvSlicersManager* imageManager = new vvSlicersManager(4);
	      qApp->processEvents();
	      imageManager->SetExtractedImage(nameGenerator->GetFileNames()[i],
					      mSlicersManagers[index]->GetSlicer(0)->GetImage(),
					      mSlicersManagers[index]->GetDimension(),
					      component, i);
	      mSlicersManagers.push_back(imageManager);

	      //create an item in the tree with good settings
	      QTreeWidgetItem *item = new QTreeWidgetItem();
	      item->setData(0,32,nameGenerator->GetFileNames()[i].c_str());
	      std::string fileI = itksys::SystemTools::GetFilenameWithoutLastExtension(
										       nameGenerator->GetFileNames()[i]).c_str();
	      item->setData(COLUMN_IMAGE_NAME,0,fileI.c_str());
	      qApp->processEvents();
	      for(int j = 1; j <= 4; j++)
		{
		  for(int i = 0; i < DataTree->topLevelItemCount(); i++)
		    {
		      DataTree->topLevelItem(i)->setData(j,10,0);
		    }
		  item->setData(j,10,2);
		}

	      //Create the buttons for reload and close
	      qApp->processEvents();
	      QTreePushButton* cButton = new QTreePushButton;
	      cButton->setItem(item);
	      cButton->setColumn(COLUMN_CLOSE_IMAGE);
	      cButton->setToolTip(tr("close image"));
	      cButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/exit.png")));
	      connect(cButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
		      this,SLOT(CloseImage(QTreeWidgetItem*, int)));

	      QTreePushButton* rButton = new QTreePushButton;
	      rButton->setItem(item);
	      rButton->setColumn(COLUMN_RELOAD_IMAGE);
	      rButton->setToolTip(tr("reload image"));
	      rButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/rotateright.png")));
	      rButton->setEnabled(false);
	      connect(rButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
		      this,SLOT(ReloadImage(QTreeWidgetItem*, int)));

	      DataTree->addTopLevelItem(item);
	      DataTree->setItemWidget(item, COLUMN_CLOSE_IMAGE, cButton);
	      DataTree->setItemWidget(item, COLUMN_RELOAD_IMAGE, rButton);

	      //set the id of the image 
	      QString id = nameGenerator->GetFileNames()[i].c_str() + QString::number(mSlicersManagers.size()-1);
	      item->setData(COLUMN_IMAGE_NAME,32,id.toStdString().c_str());
	      mSlicersManagers.back()->SetId(id.toStdString());
	      linkPanel->addImage(fileI, id.toStdString());
	      connect(mSlicersManagers.back(),SIGNAL(currentImageChanged(std::string)),
		      this,SLOT(CurrentImageChanged(std::string)));
	      connect(mSlicersManagers.back(),SIGNAL(
						     UpdatePosition(int, double, double, double, double, double, double, double)),this,
		      SLOT(MousePositionChanged(int,double, double, double, double, double, double, double)));
	      connect(mSlicersManagers.back(),SIGNAL(UpdateVector(int, double, double, double, double)),
		      this, SLOT(VectorChanged(int,double,double,double, double)));
	      connect(mSlicersManagers.back(),SIGNAL(UpdateOverlay(int, double, double)),
		      this, SLOT(OverlayChanged(int,double,double)));
	      connect(mSlicersManagers.back(),SIGNAL(UpdateFusion(int, double)),
		      this, SLOT(FusionChanged(int,double)));
	      connect(mSlicersManagers.back(),SIGNAL(UpdateWindows(int, int, int)),
		      this,SLOT(WindowsChanged(int, int, int)));
	      connect(mSlicersManagers.back(),SIGNAL(WindowLevelChanged(double, double,int, int)),
		      this,SLOT(WindowLevelChanged(double, double, int, int)));
	      connect(mSlicersManagers.back(),SIGNAL(UpdateSlice(int,int)),
		      this,SLOT(UpdateSlice(int,int)));
	      connect(mSlicersManagers.back(),SIGNAL(UpdateTSlice(int, int)),
		      this,SLOT(UpdateTSlice(int, int)));
	      connect(mSlicersManagers.back(),SIGNAL(UpdateSliceRange(int,int,int,int,int)),
		      this,SLOT(UpdateSliceRange(int,int,int,int,int)));
	      connect(mSlicersManagers.back(),SIGNAL(UpdateLinkManager(std::string,int,double,double,double,int)),
		      this,SLOT(UpdateLinkManager(std::string,int,double,double,double,int)));
	      connect(mSlicersManagers.back(),SIGNAL(LandmarkAdded()),landmarksPanel,SLOT(AddPoint()));
	      UpdateTree();
	      qApp->processEvents();
	      InitDisplay();
	      qApp->processEvents();
	    }
	  QApplication::restoreOverrideCursor();
	}
      else
	{
	  QApplication::restoreOverrideCursor();
	  QString error = "Cannot split file (dimensions must be greater than 2) ";
	  QMessageBox::information(this,tr("Splitting problem"),error);
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::MousePositionChanged(int visibility,double x, double y, double z, double X, double Y, double Z , double value) {
  //std::cout << "vvMainWindow::MousePositionChanged(int visibility,int x, int y, int z, double X, double Y, double Z , double value)" << std::endl;
  infoPanel->setCurrentInfo(visibility,x,y,z,X,Y,Z,value);
}
//====================================================================

//====================================================================
void vvMainWindow::VectorChanged(int visibility,double x, double y, double z, double value) {
  //std::cout << "vvMainWindow::VectorChanged(int visibility,double x, double y, double z, double value)" << std::endl;
  overlayPanel->getCurrentVectorInfo(visibility,x,y,z,value);
}
//====================================================================

//====================================================================
void vvMainWindow::OverlayChanged(int visibility, double valueOver, double valueRef) {
  //std::cout << "vvMainWindow::OverlayChanged(int visibility,double x, double y, double z, double value)" << std::endl;
  overlayPanel->getCurrentOverlayInfo(visibility,valueOver, valueRef);
}
//====================================================================

//====================================================================
void vvMainWindow::FusionChanged(int visibility, double value) {
  //std::cout << "vvMainWindow::FusionChanged(int visibility,double x, double y, double z, double value)" << std::endl;
  overlayPanel->getCurrentFusionInfo(visibility,value);
}
//====================================================================

//====================================================================
void vvMainWindow::WindowsChanged(int window, int view, int slice) {
  //std::cout << "vvMainWindow::WindowsChanged(int window, int view, int slice)" << std::endl;
  infoPanel->setViews(window, view, slice);
}
//====================================================================

//====================================================================
void vvMainWindow::WindowLevelChanged(double window, double level,int preset,int colormap) {
  //std::cout << "vvMainWindow::WindowLevelChanged(double window, double level)" << std::endl;
  windowSpinBox->setValue(window);
  levelSpinBox->setValue(level);
  colorMapComboBox->setCurrentIndex(colormap);
  presetComboBox->setCurrentIndex(preset);
}
//====================================================================

//====================================================================
void vvMainWindow::WindowLevelEdited() {
  //std::cout << "vvMainWindow::WindowLevelEdited" << std::endl;
  presetComboBox->setCurrentIndex(6);
  UpdateWindowLevel();
}
//====================================================================

//====================================================================
void vvMainWindow::UpdateWindowLevel() {
  //std::cout << "vvMainWindow::UpdateWindowLevel"  <<std::endl;
  if(DataTree->selectedItems().size())
    {
      int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
      mSlicersManagers[index]->SetColorWindow(windowSpinBox->value());
      mSlicersManagers[index]->SetColorLevel(levelSpinBox->value());
      mSlicersManagers[index]->SetPreset(presetComboBox->currentIndex());
      mSlicersManagers[index]->SetColorMap(colorMapComboBox->currentIndex());
      mSlicersManagers[index]->Render();
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SwitchWindowLevel() {
  //std::cout << "vvMainWindow::SwitchWindowLevel"  <<std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
  int window = mSlicersManagers[index]->GetColorWindow();
  presetComboBox->setCurrentIndex(6);
  windowSpinBox->setValue(-window);
  UpdateWindowLevel();
}
//====================================================================

//====================================================================
void vvMainWindow::UpdateLinkManager(std::string id, int slicer, double x, double y, double z, int temps) {
  //std::cout << "vvMainWindow::UpdateLinkManager(std::string id, int slicer, double x, double y, double z, int temps)" << std::endl;
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(mSlicersManagers[i]->GetId() == id)
	{
	  //mSlicersManagers[i]->SetTSlice(temps);
	  mSlicersManagers[i]->GetSlicer(slicer)->SetCurrentPosition(x,y,z,temps);
	  mSlicersManagers[i]->UpdateViews(0,slicer);
	  break;
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::ShowContextMenu(QPoint point) {
  //std::cout << "vvMainWindow::ShowContextMenu(QPoint point)" << std::endl;
  if(!DataTree->selectedItems().size())
    {
      contextActions[1]->setEnabled(0);
      contextActions[2]->setEnabled(0);
      contextActions[3]->setEnabled(0);
      contextActions[4]->setEnabled(0);
      contextActions[5]->setEnabled(0);
      contextActions[6]->setEnabled(0);
      contextActions[7]->setEnabled(0);
    }
  else
    {
      int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
      contextActions[1]->setEnabled(1);
      contextActions[2]->setEnabled(
				    DataTree->itemWidget(DataTree->selectedItems()[0],
							 COLUMN_RELOAD_IMAGE)->isEnabled());
      contextActions[3]->setEnabled(1);
      contextActions[5]->setEnabled(1);
      contextActions[6]->setEnabled(1);
      contextActions[7]->setEnabled(1);

      if(mSlicersManagers[index]->GetDimension() < 3)
	contextActions[4]->setEnabled(0);
      else
	contextActions[4]->setEnabled(1);
    }
  contextMenu.exec(QCursor::pos());
}
//====================================================================

//====================================================================
void vvMainWindow::CloseImage() {
  //std::cout << "vvMainWindow::CloseImage()" << std::endl;
  CloseImage(DataTree->selectedItems()[0],0);
}
//====================================================================

//====================================================================
void vvMainWindow::ReloadImage() {
  //std::cout << "vvMainWindow::ReloadImage()" << std::endl;
  ReloadImage(DataTree->selectedItems()[0],0);
}
//====================================================================

//====================================================================
void vvMainWindow::SelectOverlayImage() {
  //std::cout << "vvMainWindow::SelectOverlayImage" << std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);

  //check if one overlay image is added
  for(int child = 0; child < DataTree->topLevelItem(index)->childCount(); child++)
    if(DataTree->topLevelItem(index)->child(child)->data(1,32).toString() == "overlay")
      {
	QString error = "Cannot add more than one compared image\n";
	error += "Please remove first ";
	error += DataTree->topLevelItem(index)->child(child)->data(COLUMN_IMAGE_NAME,0).toString();
	QMessageBox::information(this,tr("Problem adding compared image !"),error);
	return;
      }

  QString Extensions = "All Files (*)";
  Extensions += ";;Images ( *.bmp)";
  Extensions += ";;Images ( *.png)";
  Extensions += ";;Images ( *.jpeg)";
  Extensions += ";;Images ( *.tif)";
  Extensions += ";;Images ( *.mhd)";
  Extensions += ";;Images ( *.hdr)";
  Extensions += ";;Images ( *.vox)";
  QString file = QFileDialog::getOpenFileName(this,tr("Load Overlay image"),mInputPathName,Extensions);
  if(!file.isEmpty())
    {
      mInputPathName = itksys::SystemTools::GetFilenamePath(file.toStdString()).c_str();
      itk::ImageIOBase::Pointer reader = itk::ImageIOFactory::CreateImageIO(
									    file.toStdString().c_str(), itk::ImageIOFactory::ReadMode);
      reader->SetFileName(file.toStdString().c_str());
      reader->ReadImageInformation();
      std::string component = reader->GetComponentTypeAsString(reader->GetComponentType());
      int dimension = reader->GetNumberOfDimensions();
      AddOverlayImage(index,file,component,dimension);
    }
}
//====================================================================

//====================================================================
void vvMainWindow::AddOverlayImage(int index, QString file, std::string component, int dimension) {
  //std::cout << "vvMainWindow::Add OverlayImage (" << index << "," << file.toStdString() << "," << component << ")" << std::endl;
  //init the progress events
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
  vvProgressDialog progress;
  progress.show();
  qApp->processEvents();

  std::string filename = itksys::SystemTools::GetFilenameWithoutExtension(file.toStdString()).c_str();
  if(mSlicersManagers[index]->SetOverlay(file.toStdString(),dimension, component))
    {
      //create an item in the tree with good settings
      QTreeWidgetItem *item = new QTreeWidgetItem();
      item->setData(0,32,file.toStdString().c_str());
      item->setData(1,32,tr("overlay"));
      item->setData(COLUMN_IMAGE_NAME,0,filename.c_str());
      qApp->processEvents();

      for(int j = 1; j <= 4; j++)
	{
	  item->setData(j,10,DataTree->topLevelItem(index)->data(j,10));
	  mSlicersManagers[index]->GetSlicer(j-1)->SetOverlayVisibility(
		DataTree->topLevelItem(index)->data(j,10).toInt());
	}

      //Create the buttons for reload and close
      qApp->processEvents();
      QTreePushButton* cButton = new QTreePushButton;
      cButton->setItem(item);
      cButton->setColumn(COLUMN_CLOSE_IMAGE);
      cButton->setToolTip(tr("close image"));
      cButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/exit.png")));
      connect(cButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
	      this,SLOT(CloseImage(QTreeWidgetItem*, int)));

      QTreePushButton* rButton = new QTreePushButton;
      rButton->setItem(item);
      rButton->setColumn(COLUMN_RELOAD_IMAGE);
      rButton->setToolTip(tr("reload image"));
      rButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/rotateright.png")));
      connect(rButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
	      this,SLOT(ReloadImage(QTreeWidgetItem*, int)));

      DataTree->topLevelItem(index)->setExpanded(1);
      DataTree->topLevelItem(index)->addChild(item);
      DataTree->setItemWidget(item, COLUMN_CLOSE_IMAGE, cButton);
      DataTree->setItemWidget(item, COLUMN_RELOAD_IMAGE, rButton);

      //set the id of the image 
      QString id = DataTree->topLevelItem(index)->data(COLUMN_IMAGE_NAME,32).toString();
      item->setData(COLUMN_IMAGE_NAME,32,id.toStdString().c_str());
      UpdateTree();
      qApp->processEvents();
      DisplayChanged(DataTree->topLevelItem(index),1);
      qApp->processEvents();
      ImageInfoChanged();
      progress.reject();
      QApplication::restoreOverrideCursor();
    }
  else
    {
      progress.reject();
      QApplication::restoreOverrideCursor();
      QString error = "Cannot import the new image.\n";
      error += mSlicersManagers[index]->GetLastError().c_str();
      QMessageBox::information(this,tr("Problem reading image !"),error);
    }
}

void vvMainWindow::AddFusionImage()
{
  //std::cout << "vvMainWindow::AddFusionImage" << std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);

  //check if one fusion image is added
  for(int child = 0; child < DataTree->topLevelItem(index)->childCount(); child++)
    if(DataTree->topLevelItem(index)->child(child)->data(1,32).toString() == "fusion")
      {
	QString error = "Cannot add more than one fusion image\n";
	error += "Please remove first ";
	error += DataTree->topLevelItem(index)->child(child)->data(COLUMN_IMAGE_NAME,0).toString();
	QMessageBox::information(this,tr("Problem adding fusion image !"),error);
	return;
      }

  QString Extensions = "All Files (*)";
  Extensions += ";;Images ( *.bmp)";
  Extensions += ";;Images ( *.png)";
  Extensions += ";;Images ( *.jpeg)";
  Extensions += ";;Images ( *.tif)";
  Extensions += ";;Images ( *.mhd)";
  Extensions += ";;Images ( *.hdr)";
  Extensions += ";;Images ( *.vox)";
  QString file = QFileDialog::getOpenFileName(this,tr("Load Fusion image"),mInputPathName,Extensions);
  if(!file.isEmpty())
    {
      mInputPathName = itksys::SystemTools::GetFilenamePath(file.toStdString()).c_str();
      itk::ImageIOBase::Pointer reader = itk::ImageIOFactory::CreateImageIO(
									    file.toStdString().c_str(), itk::ImageIOFactory::ReadMode);
      reader->SetFileName(file.toStdString().c_str());
      reader->ReadImageInformation();
      std::string component = reader->GetComponentTypeAsString(reader->GetComponentType());
      if(reader)
	{
	  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	  vvProgressDialog progress;
	  progress.show();
	  qApp->processEvents();

	  std::string filename = itksys::SystemTools::GetFilenameWithoutExtension(file.toStdString()).c_str();
	  if(mSlicersManagers[index]->SetFusion(file.toStdString(),
						reader->GetNumberOfDimensions(), component))
	    {
	      //create an item in the tree with good settings
	      QTreeWidgetItem *item = new QTreeWidgetItem();
	      item->setData(0,32,file.toStdString().c_str());
	      item->setData(1,32,tr("fusion"));
	      item->setData(COLUMN_IMAGE_NAME,0,filename.c_str());
	      qApp->processEvents();

	      for(int j = 1; j <= 4; j++)
		{
		  item->setData(j,10,DataTree->topLevelItem(index)->data(j,10));
		  mSlicersManagers[index]->GetSlicer(j-1)->SetFusionVisibility(
			DataTree->topLevelItem(index)->data(j,10).toInt());
		}

	      //Create the buttons for reload and close
	      qApp->processEvents();
	      QTreePushButton* cButton = new QTreePushButton;
	      cButton->setItem(item);
	      cButton->setColumn(COLUMN_CLOSE_IMAGE);
	      cButton->setToolTip(tr("close image"));
	      cButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/exit.png")));
	      connect(cButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
		      this,SLOT(CloseImage(QTreeWidgetItem*, int)));

	      QTreePushButton* rButton = new QTreePushButton;
	      rButton->setItem(item);
	      rButton->setColumn(COLUMN_RELOAD_IMAGE);
	      rButton->setToolTip(tr("reload image"));
	      rButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/rotateright.png")));
	      connect(rButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
		      this,SLOT(ReloadImage(QTreeWidgetItem*, int)));

	      DataTree->topLevelItem(index)->setExpanded(1);
	      DataTree->topLevelItem(index)->addChild(item);
	      DataTree->setItemWidget(item, COLUMN_CLOSE_IMAGE, cButton);
	      DataTree->setItemWidget(item, COLUMN_RELOAD_IMAGE, rButton);

	      //set the id of the image 
	      QString id = DataTree->topLevelItem(index)->data(COLUMN_IMAGE_NAME,32).toString();
	      item->setData(COLUMN_IMAGE_NAME,32,id.toStdString().c_str());
	      UpdateTree();
	      qApp->processEvents();
	      DisplayChanged(DataTree->topLevelItem(index),1);
	      qApp->processEvents();
	      ImageInfoChanged();
	      progress.reject();
	      QApplication::restoreOverrideCursor();
	    }
	  else
	    {
	      progress.reject();
	      QApplication::restoreOverrideCursor();
	      QString error = "Cannot import the new image.\n";
	      error += mSlicersManagers[index]->GetLastError().c_str();
	      QMessageBox::information(this,tr("Problem reading image !"),error);
	    }
	}
      else
	{
	  QString error = "Cannot import the new image.\n";
	  QMessageBox::information(this,tr("Problem reading image !"),error);
	}
    }
}


void vvMainWindow::AddField()
{
  //std::cout << "vvMainWindow::AddField()" << std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
  //check if one vector image is added
  for(int child = 0; child < DataTree->topLevelItem(index)->childCount(); child++)
    if(DataTree->topLevelItem(index)->child(child)->data(1,32).toString() == "vector")
      {
	QString error = "Cannot add more than one vector field\n";
	error += "Please remove first ";
	error += DataTree->topLevelItem(index)->child(child)->data(COLUMN_IMAGE_NAME,0).toString();
	QMessageBox::information(this,tr("Problem adding vector !"),error);
	return;
      }

  QString Extensions = "Images ( *.mhd)";
  Extensions += ";;Images ( *.vf)";
  QString file = QFileDialog::getOpenFileName(this,tr("Load deformation field"),mInputPathName,Extensions);
  if(!file.isEmpty())
    {
      mInputPathName = itksys::SystemTools::GetFilenamePath(file.toStdString()).c_str();

      //init the progress events
      QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
      vvProgressDialog progress;
      progress.show();
      qApp->processEvents();

      //open image
      std::string action = "Opening";
      progress.start(action,file.toStdString());
      qApp->processEvents();

      itk::ImageIOBase::Pointer reader = itk::ImageIOFactory::CreateImageIO(
									    file.toStdString().c_str(), itk::ImageIOFactory::ReadMode);
      if (!reader)
	{
	  QApplication::restoreOverrideCursor();
	  QString error = "Cannot read file ";
	  error += file.toStdString().c_str();
	  QMessageBox::information(this,tr("Reading problem"),error);
	}
      else
	{
	  //read the vector and put it in the current mSlicersManager
	  reader->SetFileName(file.toStdString());
	  reader->ReadImageInformation();
	  std::string component = reader->GetComponentTypeAsString(reader->GetComponentType());
	  vvSlicersManager* imageManager = mSlicersManagers[index];
	  qApp->processEvents();

	  std::string filename = itksys::SystemTools::GetFilenameWithoutExtension(file.toStdString()).c_str();
	  if(imageManager->SetVF(file.toStdString(),reader->GetNumberOfDimensions(), component))
	    {
	      //create an item in the tree with good settings
	      QTreeWidgetItem *item = new QTreeWidgetItem();
	      item->setData(0,32,file.toStdString().c_str());
	      item->setData(1,32,tr("vector"));
	      item->setData(COLUMN_IMAGE_NAME,0,filename.c_str());
	      qApp->processEvents();

	      for(int j = 1; j <= 4; j++)
		{
		  item->setData(j,10,DataTree->topLevelItem(index)->data(j,10));
		  mSlicersManagers[index]->GetSlicer(j-1)->SetVFVisibility(
			DataTree->topLevelItem(index)->data(j,10).toInt());
		}

	      //Create the buttons for reload and close
	      qApp->processEvents();
	      QTreePushButton* cButton = new QTreePushButton;
	      cButton->setItem(item);
	      cButton->setColumn(COLUMN_CLOSE_IMAGE);
	      cButton->setToolTip(tr("close vector field"));
	      cButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/exit.png")));
	      connect(cButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
		      this,SLOT(CloseImage(QTreeWidgetItem*, int)));

	      QTreePushButton* rButton = new QTreePushButton;
	      rButton->setItem(item);
	      rButton->setColumn(COLUMN_RELOAD_IMAGE);
	      rButton->setToolTip(tr("reload vector field"));
	      rButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/rotateright.png")));
	      connect(rButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
		      this,SLOT(ReloadImage(QTreeWidgetItem*, int)));

	      DataTree->topLevelItem(index)->setExpanded(1);
	      DataTree->topLevelItem(index)->addChild(item);
	      DataTree->setItemWidget(item, COLUMN_CLOSE_IMAGE, cButton);
	      DataTree->setItemWidget(item, COLUMN_RELOAD_IMAGE, rButton);

	      //set the id of the image 
	      QString id = DataTree->topLevelItem(index)->data(COLUMN_IMAGE_NAME,32).toString();
	      item->setData(COLUMN_IMAGE_NAME,32,id.toStdString().c_str());
	      UpdateTree();
	      qApp->processEvents();
	      DisplayChanged(DataTree->topLevelItem(index),1);
	      qApp->processEvents();
	      ImageInfoChanged();
	      progress.reject();
	      QApplication::restoreOverrideCursor();
	    }
	  else
	    {
	      progress.reject();
	      QApplication::restoreOverrideCursor();
	      QString error = "Cannot import the vector field for this image.\n";
	      error += imageManager->GetLastError().c_str();
	      QMessageBox::information(this,tr("Problem reading VF !"),error);
	    }
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SetVFProperty(int subsampling, int scale, int log) {
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
  if(mSlicersManagers[index]->GetSlicer(0)->GetVF())
    {
      for(int i = 0; i < 4; i++)
	{
	  mSlicersManagers[index]->GetSlicer(i)->SetVFSubSampling(subsampling);
	  mSlicersManagers[index]->GetSlicer(i)->SetVFScale(scale);
	  if(log > 0)
	    mSlicersManagers[index]->GetSlicer(i)->SetVFLog(1);
	  else
	    mSlicersManagers[index]->GetSlicer(i)->SetVFLog(0);
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SetOverlayProperty(int color) {
  //std::cout << "vvMainWindow::SetOverlayProperty : " << color << std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
  if(mSlicersManagers[index]->GetSlicer(0)->GetOverlay())
    {
      mSlicersManagers[index]->SetOverlayColor(color);
      mSlicersManagers[index]->SetColorMap(0);
      mSlicersManagers[index]->Render();
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SetFusionProperty(int opacity, int colormap,double window, double level) {
  //std::cout << "vvMainWindow::SetFusionProperty : " << opacity << std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
  if(mSlicersManagers[index]->GetSlicer(0)->GetFusion())
    {
      mSlicersManagers[index]->SetFusionOpacity(opacity);
      mSlicersManagers[index]->SetFusionColorMap(colormap);
      mSlicersManagers[index]->SetFusionWindow(window);
      mSlicersManagers[index]->SetFusionLevel(level);
      mSlicersManagers[index]->SetColorMap(0);
      mSlicersManagers[index]->Render();
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SaveAs() {
  //std::cout << "vvMainWindow::SaveAs()" << std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);

  int dimension = mSlicersManagers[index]->GetDimension();
  QStringList OutputListeFormat;
  OutputListeFormat.clear();
  if(dimension == 1)
    {
      OutputListeFormat.push_back(".mhd");
    }
  if(dimension == 2)
    {
      OutputListeFormat.push_back(".bmp");
      OutputListeFormat.push_back(".png");
      OutputListeFormat.push_back(".jpeg");
      OutputListeFormat.push_back(".tif");
      OutputListeFormat.push_back(".mhd");
      OutputListeFormat.push_back(".hdr");
      OutputListeFormat.push_back(".vox");
    }
  else if(dimension == 3)
    {
      OutputListeFormat.push_back(".mhd");
      OutputListeFormat.push_back(".hdr");
      OutputListeFormat.push_back(".vox");
    }
  else if(dimension == 4)
    {
      OutputListeFormat.push_back(".mhd");
    }
  QString Extensions = "AllFiles(*.*)";
  for(int i = 0; i < OutputListeFormat.count(); i++)
    {
      Extensions += ";;Images ( *";
      Extensions += OutputListeFormat[i];
      Extensions += ")";
    }
  QString fileName = QFileDialog::getSaveFileName(this,
						  tr("Save As"),
						  itksys::SystemTools::GetFilenamePath(
										       mSlicersManagers[index]->GetFileName()).c_str(),
						  Extensions);
  if(!fileName.isEmpty())
    {
      std::string fileformat = itksys::SystemTools::GetFilenameLastExtension(fileName.toStdString());
      if(OutputListeFormat.contains(
				    fileformat.c_str()))
	{
	  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	  std::string action = "Saving";
	  vvProgressDialog progress;
	  progress.show();
	  qApp->processEvents();
	  progress.start(action,fileName.toStdString());
	  vvConvertVTKImageToITKDataFilter *writer = new vvConvertVTKImageToITKDataFilter;
	  writer->SetOutputFileName(fileName.toStdString());
	  writer->SetInput(mSlicersManagers[index]->GetSlicer(0)->GetImage());
	  writer->Update(dimension,mSlicersManagers[index]->GetComponent());
	  QApplication::restoreOverrideCursor();
	  if(writer->GetLastError().size())
	    {
	      QString error = "Saving do not succeed\n";
	      error += writer->GetLastError().c_str();
	      QMessageBox::information(this,tr("Saving Problem"),error);
	      SaveAs();
	    }
	}
      else
	{
	  QString error = fileformat.c_str();
	  if(error.isEmpty())
	    error += "no file format specified !";
	  else
	    error += " format unknown !!!\n";
	  QMessageBox::information(this,tr("Saving Problem"),error);
	  SaveAs();
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::AddLink(QString image1,QString image2) {
  //std::cout << "vvMainWindow::AddLink(QString image1,QString image2)" << std::endl;
  for(unsigned int i = 0; i < mSlicersManagers.size();i++)
    {
      if(image1.toStdString() == mSlicersManagers[i]->GetId())
	{
	  mSlicersManagers[i]->AddLink(image2.toStdString());
	}
      if(image2.toStdString() == mSlicersManagers[i]->GetId())
	{
	  mSlicersManagers[i]->AddLink(image1.toStdString());
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::RemoveLink(QString image1,QString image2) {
  //std::cout << "vvMainWindow::RemoveLink(QString image1,QString image2)" << std::endl;
  for(unsigned int i = 0; i < mSlicersManagers.size();i++)
    {
      if(image1.toStdString() == mSlicersManagers[i]->GetId())
	{
	  mSlicersManagers[i]->RemoveLink(image2.toStdString());
	}
      if(image2.toStdString() == mSlicersManagers[i]->GetId())
	{
	  mSlicersManagers[i]->RemoveLink(image1.toStdString());
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::NOHorizontalSliderMoved() {
  //std::cout << "vvMainWindow::NOHorizontalSliderMoved()" << std::endl;
  int value = NOHorizontalSlider->value();
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_UL_VIEW,10).toInt() > 1)
	{
	  for(int j = 0; j < 4; j++)
	    {
	      mSlicersManagers[i]->SetTSliceInSlicer(value,j);
	      if(mSlicersManagers[i]->GetSlicer(j)->GetImageActor()->GetVisibility())
		UpdateTSlice(j,value);
	    }
	  mSlicersManagers[i]->GetSlicer(0)->Render();
	  break;
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::NEHorizontalSliderMoved() {
  //std::cout << "vvMainWindow::NEHorizontalSliderMoved()" << std::endl;
  int value = NEHorizontalSlider->value();
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_UR_VIEW,10).toInt() > 1)
	{
	  for(int j = 0; j < 4; j++)
	    {
	      mSlicersManagers[i]->SetTSliceInSlicer(value,j);
	      if(mSlicersManagers[i]->GetSlicer(j)->GetImageActor()->GetVisibility())
		UpdateTSlice(j,value);
	    }
	  mSlicersManagers[i]->GetSlicer(1)->Render();
	  break;
	}
    }

}
//====================================================================

//====================================================================
void vvMainWindow::SOHorizontalSliderMoved() {
  //std::cout << "vvMainWindow::SOHorizontalSliderMoved()" << std::endl;
  int value = SOHorizontalSlider->value();
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_DL_VIEW,10).toInt() > 1)
	{
	  for(int j = 0; j < 4; j++)
	    {
	      mSlicersManagers[i]->SetTSliceInSlicer(value,j);
	      if(mSlicersManagers[i]->GetSlicer(j)->GetImageActor()->GetVisibility())
		UpdateTSlice(j,value);
	    }
	  mSlicersManagers[i]->GetSlicer(2)->Render();
	  break;
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SEHorizontalSliderMoved() {
  //std::cout << "vvMainWindow::SEHorizontalSliderMoved()" << std::endl;
  int value = SEHorizontalSlider->value();
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_DR_VIEW,10).toInt() > 1)
	{
	  for(int j = 0; j < 4; j++)
	    {
	      mSlicersManagers[i]->SetTSliceInSlicer(value,j);
	      if(mSlicersManagers[i]->GetSlicer(j)->GetImageActor()->GetVisibility())
		UpdateTSlice(j,value);
	    }
	  mSlicersManagers[i]->GetSlicer(3)->Render();
	  break;
	}
    }

}
//====================================================================

//====================================================================
void vvMainWindow::NOVerticalSliderChanged() {
  //std::cout << "vvMainWindow::NOVerticalSliderChanged()" << std::endl;
  int value = NOVerticalSlider->value();
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_UL_VIEW,10).toInt() > 1)
	{
	  mSlicersManagers[i]->GetSlicer(0)->SetSlice(value);
	  break;
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::NEVerticalSliderChanged() {
  //std::cout << "vvMainWindow::NEVerticalSliderChanged()" << std::endl;
  int value = NEVerticalSlider->value();
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_UR_VIEW,10).toInt() > 1)
	{
	  mSlicersManagers[i]->GetSlicer(1)->SetSlice(value);
	  break;
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SOVerticalSliderChanged() {
  //std::cout << "vvMainWindow::SOVerticalSliderChanged()" << std::endl;
  int value = SOVerticalSlider->value();
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_DL_VIEW,10).toInt() > 1)
	{
	  mSlicersManagers[i]->GetSlicer(2)->SetSlice(value);
	  break;
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SEVerticalSliderChanged() {
  //std::cout << "vvMainWindow::SEVerticalSliderChanged()" << std::endl;
  int value = SEVerticalSlider->value();
  for(unsigned int i = 0; i < mSlicersManagers.size(); i++)
    {
      if(DataTree->topLevelItem(i)->data(COLUMN_DR_VIEW,10).toInt() > 1)
	{
	  mSlicersManagers[i]->GetSlicer(3)->SetSlice(value);
	  break;
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::UpdateSlice(int slicer, int slice) {
  //std::cout << "vvMainWindow::UpdateSlice(int slicer, int slice)" << std::endl;
  if(slicer == 0)
    NOVerticalSlider->setValue(slice);
  else if (slicer == 1)
    NEVerticalSlider->setValue(slice);
  else if (slicer == 2)
    SOVerticalSlider->setValue(slice);
  else if (slicer == 3)
    SEVerticalSlider->setValue(slice);
}
//====================================================================

//====================================================================
void vvMainWindow::UpdateTSlice(int slicer, int slice) {
  //std::cout << "vvMainWindow::UpdateTSlice(int slicer, int slice)" << std::endl;
  switch(slicer)
    {
    case 0:
      NOHorizontalSlider->setValue(slice);
      break;
    case 1:
      NEHorizontalSlider->setValue(slice);
      break;
    case 2:
      SOHorizontalSlider->setValue(slice);
      break;
    case 3:
      SEHorizontalSlider->setValue(slice);
      break;
    }
}
//====================================================================

//====================================================================
void vvMainWindow::UpdateSliceRange(int slicer, int min, int max, int tmin, int tmax) {
  //std::cout << "vvMainWindow::UpdateSliceRange(int slicer, int min, int max, int tmin, int tmax)" << std::endl;
  int position = int((min+max)/2);
  int tPosition = int((tmin+tmax)/2);
  if(slicer == 0)
    {
      NOVerticalSlider->setValue(position);
      NOVerticalSlider->setRange(min,max);
      NOHorizontalSlider->setRange(tmin,tmax);
      NOHorizontalSlider->setValue(tPosition);
    }
  else if (slicer == 1)
    {
      NEVerticalSlider->setValue(position);
      NEVerticalSlider->setRange(min,max);
      NEHorizontalSlider->setRange(tmin,tmax);
      NEHorizontalSlider->setValue(tPosition);
    }
  else if (slicer == 2)
    {
      SOVerticalSlider->setValue(position);
      SOVerticalSlider->setRange(min,max);
      SOHorizontalSlider->setRange(tmin,tmax);
      SOHorizontalSlider->setValue(tPosition);
    }
  else if (slicer == 3)
    {
      SEVerticalSlider->setValue(position);
      SEVerticalSlider->setRange(min,max);
      SEHorizontalSlider->setRange(tmin,tmax);
      SEHorizontalSlider->setValue(tPosition);
    }
}
//====================================================================

//====================================================================
void vvMainWindow::SaveNOScreenshot() {
  vtkWindowToImageFilter *w2i = vtkWindowToImageFilter::New();
  w2i->SetInput(NOViewWidget->GetRenderWindow());
  w2i->Update();
  SaveScreenshot(w2i->GetOutput());
  w2i->Delete();
}
//====================================================================

//====================================================================
void vvMainWindow::SaveNEScreenshot() {
  vtkWindowToImageFilter *w2i = vtkWindowToImageFilter::New();
  w2i->SetInput(NEViewWidget->GetRenderWindow());
  w2i->Update();
  SaveScreenshot(w2i->GetOutput());
  w2i->Delete();
}
//====================================================================

//====================================================================
void vvMainWindow::SaveSOScreenshot() {
  vtkWindowToImageFilter *w2i = vtkWindowToImageFilter::New();
  w2i->SetInput(SOViewWidget->GetRenderWindow());
  w2i->Update();
  SaveScreenshot(w2i->GetOutput());
  w2i->Delete();
}
//====================================================================

//====================================================================
void vvMainWindow::SaveSEScreenshot() {
  vtkWindowToImageFilter *w2i = vtkWindowToImageFilter::New();
  w2i->SetInput(SEViewWidget->GetRenderWindow());
  w2i->Update();
  SaveScreenshot(w2i->GetOutput());
  w2i->Delete();
}
//====================================================================

//====================================================================
void vvMainWindow::SaveScreenshot(vtkImageData* image) {
  //std::cout << "vvMainWindow::SaveScreenshot()" << std::endl;
  QString Extensions = "Images( *.png);;";
  Extensions += "Images( *.jpg);;";
  Extensions += "Images( *.bmp);;";
  Extensions += "Images( *.tif);;";
  Extensions += "Images( *.ppm)";
  QString fileName = QFileDialog::getSaveFileName(this,
						  tr("Save As"),
						  itksys::SystemTools::GetFilenamePath(
										       mSlicersManagers[0]->GetFileName()).c_str(),
						  Extensions);
  if(!fileName.isEmpty())
    {
      const char *ext = fileName.toStdString().c_str() + strlen(fileName.toStdString().c_str()) - 4;
      if (!strcmp(ext, ".bmp"))
	{
	  vtkBMPWriter *bmp = vtkBMPWriter::New();
	  bmp->SetInput(image);
	  bmp->SetFileName(fileName.toStdString().c_str());
	  bmp->Write();
	  bmp->Delete();
	}
      else if (!strcmp(ext, ".tif"))
	{
	  vtkTIFFWriter *tif = vtkTIFFWriter::New();
	  tif->SetInput(image);
	  tif->SetFileName(fileName.toStdString().c_str());
	  tif->Write();
	  tif->Delete();
	}
      else if (!strcmp(ext, ".ppm"))
	{
	  vtkPNMWriter *pnm = vtkPNMWriter::New();
	  pnm->SetInput(image);
	  pnm->SetFileName(fileName.toStdString().c_str());
	  pnm->Write();
	  pnm->Delete();
	}
      else if (!strcmp(ext, ".png"))
	{
	  vtkPNGWriter *png = vtkPNGWriter::New();
	  png->SetInput(image);
	  png->SetFileName(fileName.toStdString().c_str());
	  png->Write();
	  png->Delete();
	}
      else if (!strcmp(ext, ".jpg"))
	{
	  vtkJPEGWriter *jpg = vtkJPEGWriter::New();
	  jpg->SetInput(image);
	  jpg->SetFileName(fileName.toStdString().c_str());
	  jpg->Write();
	  jpg->Delete();
	}
      else 
	{
	  QMessageBox::information(this,tr("Problem saving screenshot !"),tr("Cannot save image.\nPlease set a file extension !!!"));
	}
    }

}
//====================================================================

//====================================================================
void vvMainWindow::GoToCursor() {
  //std::cout << "vvMainWindow::GoToCursor()" << std::endl;
  int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
  for (int column = 1; column < 5; column++)
    {
      if(DataTree->selectedItems()[0]->data(column,10).toInt() > 1)
	{
	  double* cursorPos = mSlicersManagers[index]->GetSlicer(column-1)->GetCursorPosition();
	  mSlicersManagers[index]->GetSlicer(column-1)->SetCurrentPosition(
									   cursorPos[0],cursorPos[1],cursorPos[2],cursorPos[3]);
	  mSlicersManagers[index]->UpdateViews(1,column-1);
	  mSlicersManagers[index]->UpdateLinked(column-1);
	  break;
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::PlayPause() {
  //std::cout << "vvMainWindow::PlayPause" << std::endl;
  if(playMode)
    {
      playMode = 0;
      playButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/player_play.png")));
      ImageInfoChanged();
      return;
    }
  else
    {
      /*int maxT = 0;
      for(int i = 0; i < 4; i++)
	if(maxT < horizontalSliders[i]->maximum())
	  {
	    maxT = horizontalSliders[i]->maximum();
	    mSliderPlayed = i;
	  }*/
      int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);

      //if there is time explicit instanciation
      if(mSlicersManagers[index]->GetSlicer(0)->GetImage()->GetImage().size() > 1)
	{
        for(int i = 0; i < mSlicersManagers[index]->NumberOfSlicers(); i++)
          {
	  mSlicersManagers[index]->GetSlicer(i)->SetCursorVisibility(0);
          if(DataTree->topLevelItem(index)->data(i+1,10).toInt() > 0)
            mSliderPlayed = i;
          }
	playMode = 1;
	playButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/player_pause.png")));
	QTimer::singleShot(1000/mFrameRate, this, SLOT(PlayNext()));
	}
    }
}
//====================================================================

//====================================================================
void vvMainWindow::PlayNext() {
  //std::cout << "PlayNext(" << mSliderPlayed << ")" << std::endl;
  if(playMode && !this->isHidden() && mSliderPlayed > -1 && mSliderPlayed < 4)
    {
    int index = GetSlicerIndexFromItem(DataTree->selectedItems()[0]);
    for(int i = 0; i < mSlicersManagers[index]->NumberOfSlicers(); i++)
      mSlicersManagers[index]->GetSlicer(i)->SetCursorVisibility(0);
      qApp->processEvents();
      if(horizontalSliders[mSliderPlayed]->maximum() < 2)
	{
	  playMode = 0;
	  PlayPause();
	  return;
	}
      if(horizontalSliders[mSliderPlayed]->value() == horizontalSliders[mSliderPlayed]->maximum())
	horizontalSliders[mSliderPlayed]->triggerAction(QAbstractSlider::SliderToMinimum);
      horizontalSliders[mSliderPlayed]->triggerAction(QAbstractSlider::SliderSingleStepAdd);
      QTimer::singleShot(1000/mFrameRate, this, SLOT(PlayNext()));
    }
}
//====================================================================

//====================================================================
void vvMainWindow::UpdateRenderWindows() {
  if(NOViewWidget->GetRenderWindow()) NOViewWidget->GetRenderWindow()->Render();
  if(NEViewWidget->GetRenderWindow()) NEViewWidget->GetRenderWindow()->Render();
  if(SOViewWidget->GetRenderWindow()) SOViewWidget->GetRenderWindow()->Render();
  if(SEViewWidget->GetRenderWindow()) SEViewWidget->GetRenderWindow()->Render();
}
//====================================================================

//====================================================================
void vvMainWindow::ResampleCurrentImage() {
  vvResamplerDialog resampler;
  resampler.SetSlicersManagers(mSlicersManagers);
  int n = mSlicersManagers.size();
  DD(n);
  bool b = resampler.exec();
  DD(b);
  if (b) {
    /*std::vector<std::string> f;
      f.push_back("toto.mhd");
      LoadImages(f, IMAGE);*/
    AddImage(resampler.GetOutputSlicerManager());
  }
}
//====================================================================

//====================================================================
void vvMainWindow::AddImage(vvSlicersManager * imageManager) {
  mSlicersManagers.push_back(imageManager);

  std::string filename = imageManager->GetFileName();

  //create an item in the tree with good settings
  QTreeWidgetItem *item = new QTreeWidgetItem();
  item->setData(0,32,imageManager->GetFileName().c_str());//files[i].c_str());
  item->setData(COLUMN_IMAGE_NAME,0,filename.c_str());
  qApp->processEvents();
  
  for(int j = 1; j <= 4; j++) item->setData(j,10,1);

  //Create the buttons for reload and close
  qApp->processEvents();
  QTreePushButton* cButton = new QTreePushButton;
  cButton->setItem(item);
  cButton->setColumn(COLUMN_CLOSE_IMAGE);
  cButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/exit.png")));
  connect(cButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
	  this,SLOT(CloseImage(QTreeWidgetItem*, int)));
  
  QTreePushButton* rButton = new QTreePushButton;
  rButton->setItem(item);
  rButton->setColumn(COLUMN_RELOAD_IMAGE);
  rButton->setIcon(QIcon(QString::fromUtf8(":/new/prefix1/rotateright.png")));
  connect(rButton,SIGNAL(clickedInto(QTreeWidgetItem*, int)),
	  this,SLOT(ReloadImage(QTreeWidgetItem*, int)));
  
  DataTree->addTopLevelItem(item);
  DataTree->setItemWidget(item, COLUMN_CLOSE_IMAGE, cButton);
  DataTree->setItemWidget(item, COLUMN_RELOAD_IMAGE, rButton);
  
  //set the id of the image 
  QString id = imageManager->GetFileName().c_str() + QString::number(mSlicersManagers.size()-1);
  item->setData(COLUMN_IMAGE_NAME,32,id.toStdString().c_str());
  mSlicersManagers.back()->SetId(id.toStdString());
  
  linkPanel->addImage(filename, id.toStdString());
  
  connect(mSlicersManagers.back(),SIGNAL(currentImageChanged(std::string)),
	  this,SLOT(CurrentImageChanged(std::string)));
  connect(mSlicersManagers.back(),SIGNAL(
					 UpdatePosition(int, double, double, double, double, double, double, double)),this,
	  SLOT(MousePositionChanged(int,double, double, double, double, double, double, double)));
  connect(mSlicersManagers.back(),SIGNAL(UpdateVector(int, double, double, double, double)),
	  this, SLOT(VectorChanged(int,double,double,double, double)));
  connect(mSlicersManagers.back(),SIGNAL(UpdateOverlay(int, double, double)),
	  this, SLOT(OverlayChanged(int,double,double)));
  connect(mSlicersManagers.back(),SIGNAL(UpdateFusion(int, double)),
	  this, SLOT(FusionChanged(int,double)));
  connect(mSlicersManagers.back(),SIGNAL(UpdateWindows(int, int, int)),
	  this,SLOT(WindowsChanged(int, int, int)));
  connect(mSlicersManagers.back(),SIGNAL(WindowLevelChanged(double, double,int, int)),
	  this,SLOT(WindowLevelChanged(double, double, int, int)));
  connect(mSlicersManagers.back(),SIGNAL(UpdateSlice(int,int)),
	  this,SLOT(UpdateSlice(int,int)));
  connect(mSlicersManagers.back(),SIGNAL(UpdateTSlice(int, int)),
	  this,SLOT(UpdateTSlice(int, int)));
  connect(mSlicersManagers.back(),SIGNAL(UpdateSliceRange(int,int,int,int,int)),
	  this,SLOT(UpdateSliceRange(int,int,int,int,int)));
  connect(mSlicersManagers.back(),SIGNAL(UpdateLinkManager(std::string,int,double,double,double,int)),
	  this,SLOT(UpdateLinkManager(std::string,int,double,double,double,int)));
  connect(mSlicersManagers.back(),SIGNAL(LandmarkAdded()),landmarksPanel,SLOT(AddPoint()));
  UpdateTree();
  qApp->processEvents();
  NOViewWidget->show();
  NEViewWidget->show();
  SOViewWidget->show();
  SEViewWidget->show();

  InitDisplay();
  qApp->processEvents();

}
//====================================================================
