/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvMainWindow.h,v $
 Language:  C++
 Date:      $Date: 2008/07/15 11:26:19 $
 Version:   $Revision: 1.26 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/

#ifndef vvMainWindow_h
#define vvMainWindow_h

#include <iostream>
#include <vector>

#include "ui_vvMainWindow.h"

#include "vvSlicersManager.h"
#include "vvSlicer.h"
#include "vvImage.h"
#include "vvDocumentation.h"
#include "vvQDicomSeriesSelector.h"


class vtkRenderWindowInteractor;
class vtkImageData;
class vtkRenderer;

class vvMainWindow : public QMainWindow, private Ui::vvMainWindow
{
  Q_OBJECT

public:
  vvMainWindow();
  ~vvMainWindow();
  void LoadImages(std::vector<std::string> filenames, int type);
  void AddImage(vvSlicersManager * m);

public slots:
  void OpenImages();
  void OpenImageWithTime();
  void MergeImages();
  void MergeImagesWithTime();
  void OpenDicom();
  void SaveAs();
  void CurrentImageChanged(std::string id);
  void ImageInfoChanged();
  void ShowDocumentation();
  void ChangeViewMode();
  void DisplayChanged(QTreeWidgetItem *item, int column);
  void CloseImage(QTreeWidgetItem* item, int column);
  void ReloadImage(QTreeWidgetItem* item, int column);
  void MousePositionChanged(int visibility, double x, double y, double z, double X, double Y, double Z , double value);
  void VectorChanged(int visibility, double x, double y, double z, double value);
  void OverlayChanged(int visibility, double valueOver, double valueRef);
  void FusionChanged(int visibility, double value);
  void ResampleCurrentImage();

  void WindowsChanged(int window, int view, int slice);
  void WindowLevelChanged(double window, double level,int preset, int colormap);
  void UpdateSlice(int slicer, int slice);
  void UpdateTSlice(int slicer, int slice);
  void UpdateSliceRange(int slicer, int min, int max, int tmin, int tmax);
  void WindowLevelEdited();
  void UpdateWindowLevel();
  void SwitchWindowLevel();
  void UpdateLinkManager(std::string id, int slicer, double x, double y, double z, int temps);
  void AddLink(QString image1,QString image2);
  void RemoveLink(QString image1,QString image2);

  void NOHorizontalSliderMoved();
  void NEHorizontalSliderMoved();
  void SOHorizontalSliderMoved();
  void SEHorizontalSliderMoved();

  void NOVerticalSliderChanged();
  void NEVerticalSliderChanged();
  void SOVerticalSliderChanged();
  void SEVerticalSliderChanged();

  void SaveNEScreenshot();
  void SaveNOScreenshot();
  void SaveSEScreenshot();
  void SaveSOScreenshot();

  void ShowContextMenu(QPoint point);
  void SplitImage();
  void CloseImage();
  void ReloadImage();
  void AddField();
  void SelectOverlayImage();
  void AddOverlayImage(int index, QString filename, std::string component, int dimension);
  void AddFusionImage();

  void SetVFProperty(int subsampling,int scale,int lut);
  void SetOverlayProperty(int color);
  void SetFusionProperty(int opacity,int colormap,double window,double level);

  void GoToCursor();
  void PlayPause();
  void PlayNext();
  void ChangeFrameRate(int rate){mFrameRate = rate;}

  void UpdateRenderWindows();

private:

  //variables
  std::vector<vvSlicersManager*> mSlicersManagers;
  vvDocumentation *documentation;
  vvDicomSeriesSelector *dicomSeriesSelector;

  QString mInputPathName;
  bool viewMode;
  bool playMode;

  //functions
  void UpdateTree();
  void InitDisplay();
  void DisplaySliders(int slicer, int window);
  QString GetSizeInBytes(unsigned long size);
  QString GetVectorDoubleAsString(std::vector<double> vectorDouble);
  QString GetVectorIntAsString(std::vector<int> vectorInt);
  int GetSlicerIndexFromItem(QTreeWidgetItem* item);
  void SaveScreenshot(vtkImageData* image);

  QMenu contextMenu;
  //QMenu *AddSubImageMenu;
  std::vector<QAction*> contextActions;
  std::vector<QSlider*> horizontalSliders;
  std::vector<QSlider*> verticalSliders;
  int mSliderPlayed;
  int mFrameRate;
};

#endif
